#ifndef __TIMESTEPIONIO_H__
#define __TIMESTEPIONIO_H__

#include "IonIO.h"
#include "TimestepMHD.h"

/// Manages File I/O for standard timestep advancement for the LFM Ionosphere
/**
 *  Standard timestep Input/Output managment for LFM Ionosphere;
 *  simulation advances the simulation based on the integer timestep
 *  rather than the elapsed simulation time.  I/O steps expect files
 *  named according to 0-padded integer time step in the format "[Run
 *  Name]_ion_[XXXXXXX].[File Extension]"
 * 
 *  @seealso SimulationManager, IonIO, MHDManager, TimestepMHD
 */
class TimestepIonIO : public IonIO
{
public:
  TimestepIonIO(TimestepMHD *timestep_manager);
  /// Write data for a single timestep to file
  int dump_step(char *base, ION *ion);
  /// Read data for a single timestep from file
  int read_step(char *base, ION *ion);
protected:  
  TimestepMHD *timestepManager;
private:  
};
#endif
