#ifndef __UTIONIO_H__
#define __UTIONIO_H__

#include "IonIO.h"
#include "UTMHD.h"
#include "DateTime.h"


/// Manages UTIO timestep advancement and I/O for the Ionosphere
/**
 *  UTIO (Universal Time Input/Output) managment for LFM Ionosphere
 *  simulation advances the simulation based on the elapsed simulation
 *  time rather than an integer timestep number.  I/O steps expect
 *  files named according to a date & time string in the format
 *  "[Run Name]_ion_[YYYY-MM-DDTHH-MM-SS].[File Extension]"
 * 
 *  @seealso SimulationManager, MHDManager, IonIO, TimestepIonIO
 */
class UTIonIO : public IonIO
{
public:
  UTIonIO(UTMHD *ut_manager);

  /// Write data for a single timestep to file
  int dump_step(char *base, ION *ion);
  /// Read data for a single timestep from file
  int read_step(char *base, ION *ion);
protected:
  UTMHD *UTManager;
private:
};
#endif
