!> init-fortran.F
!!

#ifndef USE_DEPRECATED_NAMELISTS
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!> initialize
!! Initialize LFM for doing computation.  This should be called early on
!! just after any input parameters are read in.
!!
!! Use xmlInputReader.C to read XML input files, store the results in
!! C/C++ structs which are accessible to FORTRAN via common blocks
!! defined in many of the *.inc include files.
!!
!! This subroutine uses "Jerry's Solar wind code" and "Derco"
!!
!! @note This subroutine replaces much of the functionality of the
!! deprecated "NAMELISTS" function in init-fortran.F.  If you would like
!! to use FORTRAN namelists (i.e. "INPUT1" file) for data input,
!! recompile MSPHERE with "-DUSE_DEPRECATED_NAMELISTS".
!!
      subroutine initialize()
! ... Local variables ..................................................
#include "global_dims.inc"
#include "mhd_P++_pointers.inc"
#include "fortran_P++_arrays.inc"
#include "param.inc"
#include "var2.inc"
#include "help.inc"
#include "bzero.inc"
#include "dipole.inc"
#include "run-time.inc"
      real*4 time4
#ifdef USE_RCM 
#include "rcm-ut.inc"
#endif
#include "info.inc"
#ifdef SPHERE
#include "meter.inc"
#endif
#include "boundx.inc"
#include "ionconst.inc"
#include "nml_data.inc"
#include "b_dbl.inc"
c
#if defined(USE_RCM) && !defined(USE_MIX)
      COMMON /RINGAV_RCM/ NRINGS_RCM,NAVER_RCM(500)
#endif
#ifdef DO_RINGAV
      real voltmp(nk)
#endif 
      character*6 flag
      LOGICAL RSTRT,DBLRSTRT,FIRST
      dimension pos(6),rpp(32)
      real kpin,njpd
      logical psphere
      logical tilt_it,GSM
      integer iday(2)
      data ut0/0.0/
      data kpin/3.0/
      data front_angle/0.0/
      data psphere/.false./
! ... Begin ............................................................
#ifdef DEBUG_MODE_ON
      write(*,*) "DEBUG:  in init-fortran.F::Namelists(...)"
#endif
#ifdef DEBUG_MODE_ON      
      if (local_proc == 0) then
         write(*,*) "The following are set via xmlInputReader:"
         write(*,*) "NSTART", NSTART
         write(*,*) " NSTOP", NSTOP
         write(*,*) " NDUMP", NDUMP
         write(*,*) " ORDER", ORDER
         
         write(*,*) " NRINGS", NRINGS
         write(*,*) " NAVER", NAVER(1:50)
         write(*,*) " NI_BIN", NI_BIN
         write(*,*) " NJ_BIN", NJ_BIN
         write(*,*) " AXFIX", AXFIX
         
         write(*,*) " GAMMA", GAMMA
         write(*,*) " CRNT", CRNT
         write(*,*) " CA", CA
         write(*,*) " PDMB", PDMB
         write(*,*) " EVMAX", EVMAX
         write(*,*) " BETMAX",BETMAX
         write(*,*) " BDIF", BDIF
         
         write(*,*) " SWFROMFILE", SWFROMFILE
         write(*,*) " TZER0", TZER0
         write(*,*) " UT0", UT0
         write(*,*) " BX_ZERO", BX_ZERO
         write(*,*) " BY_COEF", BY_COEF
         write(*,*) " BZ_COEF", BZ_COEF
         
         write(*,*) " S107", S107
         write(*,*) " PEDMIN", PEDMIN
         write(*,*) " HALLMIN", HALLMIN
         write(*,*) " SIGMA_RATIO", SIGMA_RATIO
         write(*,*) " PED0", PED0
         write(*,*) " P_HARDY", P_HARDY
         write(*,*) " C_HARDY", C_HARDY
         write(*,*) " H_HARDY", H_HARDY
         write(*,*) " ITER", ITER
         write(*,*) " ERRMAX", ERRMAX
         write(*,*) " ITMAX", ITMAX
         write(*,*) " IONMODEL", IONMODEL
         write(*,*) " NLOOP", NLOOP
         write(*,*) " LOOPTRIES", LOOPTRIES
      endif
#endif 
! #endif DEBUG_MODE_ON 

! Specify default parameter values.   
! Note that some defaults are set in xmlInputReader.C
!
c     "meter.in"
c     "help.in"
      nicool = 4
      cool = 1.
      nprint = 1
      nplotit = 1
      nionplt = 1
      x_max = 25.
      x_min = -100.
      r_max = 80.
c     "dipole.inc"
! Note: The dipole moment (GEOQMU) occurs in a few
!       places throughout the code:
!         - init-Fortran.F
!         - interface/const.h (BEQUATOR)
! Note: Although geoqmu is set here, it's 
!       overwritten by whatever is in the input
!       HDF file.
      GEOqMU = 0.3
      Rinner = 4.5
      Router = 8.
      Rstop = 20.
c     "boundx.inc"
      tilt_it = .false.
      GSM = .TRUE.
      SM = .TRUE.
      table = .true.
      iday(1) = 3
      iday(2) = 21
      time_of_day = 11.5
      RHOqIN = 1.125e-23
      VXqIN  = -4.e7
      VYqIN  = 0.
      VZqIN  = 0.
      CqIN   = 4.e6
      BXqIN  = 0.
      BYqIN  = 0.
      BZqIN  = 0.
      RHOq0  = 2.25e-25
      Cq0    = 1.5E6
      RHO_start  = 2.25e-25
      C_start    = 1.5E6
      MJIN = 4
      MJWALL = 11
c     "ionconst.inc"
      itertry = 40 
      dt_old = -1.

      call run_identifier(new_runinfo)

c     get pointer to meter common block
      imeter = loc(x)

#if defined(USE_RCM) && !defined(USE_MIX)
        nrings_rcm = nrings
        DO i=1,500
           naver_rcm(i) = naver(i)
        ENDDO
#endif

c
********************************************************************
*	JERRY'S SOLAR WIND ARRAYS ADDED IN AND READ IN HERE
*	   data from ISEE3-SW.dat
********************************************************************
** Open up and rewind read file
C
C *********
C * FIXME *
C ******************************************
C What if SW-SM-DAT file doesn't exist???
C Should prompt the user & exit gracefully!
C ******************************************

      open(unit=30,file='SW-SM-DAT',form='formatted',status='old')
** Position cursor at beginning of tab'd data
       do 10 k=1,1000
          read(30,fmt=100,end=11)flag
          if ( flag .eq. ' DATA:' ) go to 12
 10       continue
 100   format(a6)
 11    write(*,*)' data flag never encountered...'
       write (6,*) ' stopping run right here'
       stop
 12     continue
      do 20, i = 1, ntable
	read(30,*,end=21)
     $        UT(i),Njp(i),Vjx(i),Vjy(i),Vjz(i),
     +     Cs(i),BjX(i),BjY(i),BjZ(i),BjT(i),Zangle(i)
c
         itable = i
*******  DO THREE  THINGS:  ONE, CHANGE UT FROM MINUTES TO SEC
*******			    TWO, CHANGE Np FROM #/CC TO g/CC
*******                     THREE, CHANGE VEL'S FROM KM/S TO CM/S
*******			    FOUR, CHANGE B'S FROM nT TO GAUSS
*******			    FIVE, CALCULATE TIME RESOLUTION deltaT
*******       **Assume that data evenly spaced in time!!!!
*****ONE
*****             add tzero to table - ut offset. that is set tzero = ut0
         UT(i)=UT(i)*60.0+tzero-UT0
*****TWO
	 Njp(i)=Njp(i)*1.673e-24*(1.16) !add 4% helium
*****THREE
	 Vjx(i)= 1.e5*Vjx(i)
	 Vjy(i)= 1.e5*Vjy(i)
	 Vjz(i)= 1.e5*Vjz(i)
	 Cs (i)= 1.e5*Cs (i)
*****FOUR
	 BjX(i)=BjX(i)/1.e5
	 BjY(i)=BjY(i)/1.e5
	 BjZ(i)=BjZ(i)/1.e5
	 BjT(i)=BjT(i)/1.e5
20	continue
 21     continue
*****FIVE
	deltaT =( UT(2)-UT(1))
	close(30)
 201    format(f7.2,f6.1,2x,3f8.1,f8.2,4f9.4)
* 1201    format(f7.2,f6.1,2x,3f8.1,f8.2,5f9.4)
* 1201    format(11f)
999	continue
        time4 = real(time,4)
	ind = min(itable,nint((time4-tzero)/deltaT))
**************************************************************
*	END JERRY'S CODE
**************************************************************
c
c    >jgl test
        write (9,1912) local_proc
 1912   format ('In hdfdump_first  ',i3)
c
      PDMB = PDMB*0.25
      ORD2 = ORDER/2
      GAMM1 = GAMMA - 1.
      GMM1IN = 1./GAMM1
      GMqGM1 = GAMMA/GAMM1
      ALPHA = SQRT(3./GAMMA)
      PI = 3.1415926
      PI4 = 4.*PI
      PI8 = 8.*PI
      PI4INV = 1./PI4
      PI8INV = 1./PI8
      CAINV = 1./CA
      ALCON = PI4INV*CAINV**2
      TWELVI = 1./12.
      EVMAX = EVMAX*erg2eV/effmass
      PI = ACOS(0.)*2.
      PI4 = 4.*PI
      PI8 = 8.*PI
      PI8INV = 1./PI8
      PI4INV = 1./PI4
      GMINV = 1./GAMMA
      GM1 = GAMMA -1.
      GP1 = GAMMA + 1.
      GM1INV = 1./GM1
      WIDTH = SQRT(3./GAMMA)
      GGM1 = GAMMA*GM1
      GGM1IN = 1./GGM1
      BETA = (3.-GAMMA)*0.5*GGM1IN
C
      DROP(1) = 0.9
      DO 88 I=2,20
         DROP(I) = 0.9*DROP(I-1)
 88   continue
C
      geoqmu = geoqmu*(rearth**3)   ! change from Gauss at pole to mag moment
      rinner = rinner*rearth
      rstop = rstop*rearth
      router = router*rearth      ! change to cm from earth radii
      aq = 0.5*(x_max - x_min)*rearth
      bq = r_max*rearth
      xq_off = aq - x_max*rearth
      aq = 1./aq
      bq = 1./bq
      bxq = bx_zero
c  This stuff below is checked in the original code, much more difficult to do in parallel
c  code  Caveat Emptor ( or programmor )
        dr = router - rinner
        a0 = - rinner/dr
        b0 = 1./dr
        a1 = router/dr
        b1 = - 1./dr
c
c

C
C
      EQIN = 0.5*RHOQIN*(VXQIN**2+VYQIN**2+VZQIN**2+
     $     2.*GGM1IN*CQIN**2)
C
c
c     Now set the coeficents in help.inc.
      CALL DERCO(FCOEF,ORDER, local_proc)
C
*      if ( local_proc .eq. 0 ) WRITE (9,112) ORDER
* 112  FORMAT (1X,I6,'-TH ORDER CALCULATION'/' THE FLUXES F THE',
*     $   ' CENTERED DERIVATIVES USE THE FOLLOWING COEFFICIENTS')
*      if ( local_proc .eq. 0 ) WRITE (9,113) (FCOEF(I),I=1,ORDER)
* 113  format( 10(1X,8(1PE14.6)/))
c
      return
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
      entry inital
!
! ... Entry .............................................................
!      
      if (MHD_PE) then
c
c      set up the double precision bi,bj,bk
c
         do k =1,nkp1
            do j=1,njp1
               do i=1,nip1
                  bi_dbl(i,j,k) = bi_g(i,j,k)
                  bj_dbl(i,j,k) = bj_g(i,j,k)
                  bk_dbl(i,j,k) = bk_g(i,j,k)
               enddo
            enddo
         enddo
c
c
c
#ifdef DO_RINGAV
c
c     set up ringav, needs to be called before subdipol
c
         do k = 1,nl_k
            voltmp(k) = volume(nl_i/2,nl_j/2,k)
         enddo
         call ring_setup(voltmp,nl_k,nrings,naver)
c
#endif      

         call flux2ave
         call subdipol
c
c
c     reshuffle the arrays -- note we copy the guard cells from rho to rho1
c     and so on -- cuts down on communication
c
         do k=klow,khigh
            do j=jlow,jhigh
               do i=ilow,ihigh
                  rho1_g(i,j,k) = rho_g(i,j,k)
                  vx1_g(i,j,k) = vx_g(i,j,k)
                  vy1_g(i,j,k) = vy_g(i,j,k)
                  vz1_g(i,j,k) = vz_g(i,j,k)
                  c1_g(i,j,k)  = c_g(i,j,k)
                  bx1_g(i,j,k) = bx_g(i,j,k)
                  by1_g(i,j,k) = by_g(i,j,k)
                  bz1_g(i,j,k) = bz_g(i,j,k)
                  bi1_g(i,j,k) = bi_g(i,j,k)
                  bj1_g(i,j,k) = bj_g(i,j,k)
                  bk1_g(i,j,k) = bk_g(i,j,k)
               enddo
            enddo
        enddo
        endif
      return

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! ... End SUBROUTINE initialize .........................................
      end
#else
! # else /* USE_DEPRECATED_NAMELISTS */
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!> Namelists(...)
!!
!! *** WARNING *** 
!!
!!   This function is deprecated!  You should really be using 
!!   XML INPUT1 files to set up all run parameters.  See
!!   "xmlInputReader.C" and "init-fortran.F:subroutine initialize()",
!!   which replace the functionality from 
!!   init-fortran.F: subroutine NAMELISTS(...)
!!
!! *** WARNING ***
!!
!! Read an "INPUT1" file, which contains information specific to a
!! particualr run of the code.
!!
!! @param file_in - input filename
!! @param file_out - base HDF output filename (prepended to all HDF files)
!! @param lion - 
!! @param ni_bin_d - 
!! @param nj_bin_d - 
!!
      SUBROUTINE Namelists(file_in,file_out, lion, ni_bin_d, nj_bin_d)
! ... Local variables ..................................................
#include "global_dims.inc"
#include "mhd_P++_pointers.inc"
#include "fortran_P++_arrays.inc"
#include "param.inc"
#include "var2.inc"
#include "help.inc"
#include "bzero.inc"
#include "dipole.inc"
#include "run-time.inc"
      real*4 time4
#include "info.inc"
#ifdef SPHERE
#include "meter.inc"
#endif
#include "boundx.inc"
#include "ionconst.inc"
#include "nml_data.inc"
#include "b_dbl.inc"
c
#if defined(USE_RCM) && !defined(USE_MIX)
      COMMON /RINGAV_RCM/ NRINGS_RCM,NAVER_RCM(500)
#endif 
#ifdef DO_RINGAV
      real voltmp(nk)
#endif
      character*100 RUN_NAME
      character*100 HDFBASE_IN,HDFBASE_OUT
c
      character*6 flag

      LOGICAL RSTRT,DBLRSTRT,FIRST
C
	dimension pos(6),rpp(32)
	real kpin,njpd
	logical psphere
      logical tilt_it,GSM
      integer iday(2)
      NAMELIST /GO/ NSTART,NSTOP,NDUMP,
     $     ORDER,HDFBASE_IN,HDFBASE_OUT,RUN_Name
#ifdef SPHERE
      NAMELIST /RINGS/ nrings,naver,ni_bin,nj_bin,axfix
#endif
      NAMELIST /CONST/ GAMMA,CA,CRNT,PDMB,EVMAX,BETMAX,BDIF
c
      NAMELIST /SWIND/ SWFROMFILE, Tzero, UT0, bx_zero,
     $     by_coef, bz_coef
      NAMELIST /ION/S107,pedmin,hallmin,sigma_ratio,ped0,
     $     p_hardy,c_hardy,h_hardy,iter,itmax,errmax,ionmodel,nloop,
     $     looptries, nion
      data ut0/0.0/
	data kpin/3.0/
        data front_angle/0.0/
	data psphere/.false./
! ... Parameter variables ..............................................
      character*300 file_in
      character*300 file_out
! ... Begin ............................................................
#ifdef DEBUG_MODE_ON
      write(*,*) "DEBUG:  in init-fortran.F::Namelists(...)"
#endif
      call rststr(HDFBASE_IN)
      call rststr(HDFBASE_OUT)

c
c     "meter.in"
c     "help.in"
      evmax = 1.e4
      betmax = 0.9
      order = 8
      gamma = 1.66666666
      ca = 1.5e8
      nicool = 4
      cool = 1.
      crnt = 0.3
      pdmb = 3.
      nprint = 1
      nplotit = 1
      nionplt = 1
      ni_bin = ni_bin_d
      nj_bin = nj_bin_d
      x_max = 25.
      x_min = -100.
      r_max = 80.
c     "dipole.inc"
! Note: The dipole moment (GEOQMU) occurs in a few
!       places throughout the code:
!         - init-Fortran.F
!         - interface/const.h (BEQUATOR)
      GEOqMU = 0.3
      Rinner = 4.5
      Router = 8.
      Rstop = 20.
c     "boundx.inc"
      tilt_it = .false.
      GSM = .TRUE.
      SM = .TRUE.
      table = .true.
      iday(1) = 3
      iday(2) = 21
      time_of_day = 11.5
      RHOqIN = 1.125e-23
      VXqIN  = -4.e7
      VYqIN  = 0.
      VZqIN  = 0.
      CqIN   = 4.e6
      BXqIN  = 0.
      BYqIN  = 0.
      BZqIN  = 0.
      RHOq0  = 2.25e-25
      Cq0    = 1.5E6
      RHO_start  = 2.25e-25
      C_start    = 1.5E6
      tzero = 3000.0
      MJIN = 4
      MJWALL = 11
c      "ionconst.inc"
	ped0 = 5.0
	ionmodel = .false.
        looptries = 1
      sigma_ratio = 1.2
      itmax = 40
      iter = 40
      itertry = 40 
      errmax = 1.e-3
      dt_old = -1.
c
      call run_identifier(new_runinfo)
c
c     get pointer to meter common block
         imeter = loc(x)
C
C *********
C * FIXME *
C ******************************************
C What if INPUT1 file doesn't exist???
C Should prompt the user & exit gracefully!
C ******************************************
C
      OPEN (UNIT=81,FILE='INPUT1',STATUS='OLD')
C
      READ (81,NML=GO)
      WRITE (9,NML=GO)
      Open(unit=17)
      write(17,NML=GO)
*      file2hdfstring(new_go)
      close(unit=17)
#ifdef SPHERE
      READ (81,NML=RINGS)
!!!!!!!!!!!
! BUGFIX: !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! This is a temporary fix for code using the gnu C/C++ and portland
! group FORTRAN compilers...  pgf77 and pgf90 get confused when reading
! namelists that specify array data...  so we must manually specify the
! array values within the code.
!
! This bit of kludge will be removed when we figure out how to specify
! array data in a namelist.
!
! Uncomment the following lines for the temporrary fix:
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      do j =1, nj
       naver(j) = 1
      enddo
      naver(1) = 8
      naver(2) = 4
      naver(3) = 4
      naver(4) = 2
      naver(5) = 2
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      WRITE (9,NML=RINGS)
      Open(unit=17)
      write(17,NML=RINGS)
*      file2hdfstring(new_rings)
      close(unit=17)
#endif
      READ (81,NML=CONST)
      WRITE (9,NML=CONST)
      Open(unit=17)
      write(17,NML=CONST)
*      file2hdfstring(new_const)
      close(unit=17)
      READ (81,NML=SWIND)
      WRITE (9,NML=SWIND)
      Open(unit=17)
      write(17,NML=SWIND)
*      file2hdfstring(new_sw)
      close(unit=17)
      READ (81,NML=ION)
      WRITE(9,NML=ION)
      Open(unit=17)
      write(17,NML=ION)
*      file2hdfstring(new_ion)
      close(unit=17)
c
	close(unit=81)
c
#if defined(USE_RCM) && !defined(USE_MIX)
        nrings_rcm = nrings
        DO i=1,500
           naver_rcm(i) = naver(i)
        ENDDO
#endif
        ilen = lnblnk(run_name)
        new_run_name(1:ilen) = run_name(1:ilen)
c
! set the time step to nstart, we'll reset off the dump file, later.
      lion  = nion

      leng0 = lnblnk(HDFBASE_IN)
      if (leng0 .ne. 0 ) then
         file_in(1:leng0) = HDFBASE_IN(1:leng0)
      else
         leng0 = 3
         file_in(1:leng0) = 'LFM' 
      endif
      file_in(leng0+1:leng0+1) = char(0)
c
      leng0 = lnblnk(HDFBASE_OUT)
      if (leng0 .ne. 0 ) then
         file_out(1:leng0) = HDFBASE_OUT(1:leng0)
      else
         leng0 = 3
         file_out(1:leng0) = 'LFM' 
      endif
      file_out(leng0+1:leng0+1) = char(0)
c
      CLOSE(81)
c
c
********************************************************************
*	JERRY'S SOLAR WIND ARRAYS ADDED IN AND READ IN HERE
*	   data from ISEE3-SW.dat
********************************************************************
** Open up and rewind read file
C
C *********
C * FIXME *
C ******************************************
C What if SW-SM-DAT file doesn't exist???
C Should prompt the user & exit gracefully!
C ******************************************

      open(unit=30,file='SW-SM-DAT',form='formatted',status='old')
** Position cursor at beginning of tab'd data
       do 10 k=1,1000
          read(30,fmt=100,end=11)flag
          if ( flag .eq. ' DATA:' ) go to 12
 10       continue
 100   format(a6)
 11    write(*,*)' data flag never encountered...'
       write (6,*) ' stopping run right here'
       stop
 12     continue
      do 20, i = 1, ntable
	read(30,*,end=21)
     $        UT(i),Njp(i),Vjx(i),Vjy(i),Vjz(i),
     +     Cs(i),BjX(i),BjY(i),BjZ(i),BjT(i),Zangle(i)
c
         itable = i
*******  DO THREE  THINGS:  ONE, CHANGE UT FROM MINUTES TO SEC
*******			    TWO, CHANGE Np FROM #/CC TO g/CC
*******                     THREE, CHANGE VEL'S FROM KM/S TO CM/S
*******			    FOUR, CHANGE B'S FROM nT TO GAUSS
*******			    FIVE, CALCULATE TIME RESOLUTION deltaT
*******       **Assume that data evenly spaced in time!!!!
*****ONE
*****             add tzero to table - ut offset. that is set tzero = ut0
         UT(i)=UT(i)*60.0+tzero-UT0
*****TWO
	 Njp(i)=Njp(i)*1.673e-24*(1.16) !add 4% helium
*****THREE
	 Vjx(i)= 1.e5*Vjx(i)
	 Vjy(i)= 1.e5*Vjy(i)
	 Vjz(i)= 1.e5*Vjz(i)
	 Cs (i)= 1.e5*Cs (i)
*****FOUR
	 BjX(i)=BjX(i)/1.e5
	 BjY(i)=BjY(i)/1.e5
	 BjZ(i)=BjZ(i)/1.e5
	 BjT(i)=BjT(i)/1.e5
20	continue
 21     continue
*****FIVE
	deltaT =( UT(2)-UT(1))
	close(30)
 201    format(f7.2,f6.1,2x,3f8.1,f8.2,4f9.4)
* 1201    format(f7.2,f6.1,2x,3f8.1,f8.2,5f9.4)
* 1201    format(11f)
999	continue
        time4 = real(time,4)
	ind = min(itable,nint((time4-tzero)/deltaT))
**************************************************************
*	END JERRY'S CODE
**************************************************************
c
c    >jgl test
        write (9,1912) local_proc
 1912   format ('In hdfdump_first  ',i3)
c
      PDMB = PDMB*0.25
      ORD2 = ORDER/2
      GAMM1 = GAMMA - 1.
      GMM1IN = 1./GAMM1
      GMqGM1 = GAMMA/GAMM1
      ALPHA = SQRT(3./GAMMA)
      PI = 3.1415926
      PI4 = 4.*PI
      PI8 = 8.*PI
      PI4INV = 1./PI4
      PI8INV = 1./PI8
      CAINV = 1./CA
      ALCON = PI4INV*CAINV**2
      TWELVI = 1./12.
      EVMAX = EVMAX*erg2eV/effmass
      PI = ACOS(0.)*2.
      PI4 = 4.*PI
      PI8 = 8.*PI
      PI8INV = 1./PI8
      PI4INV = 1./PI4
      GMINV = 1./GAMMA
      GM1 = GAMMA -1.
      GP1 = GAMMA + 1.
      GM1INV = 1./GM1
      WIDTH = SQRT(3./GAMMA)
      GGM1 = GAMMA*GM1
      GGM1IN = 1./GGM1
      BETA = (3.-GAMMA)*0.5*GGM1IN
C
      DROP(1) = 0.9
      DO 88 I=2,20
         DROP(I) = 0.9*DROP(I-1)
 88   continue
C
      geoqmu = geoqmu*(rearth**3)   ! change from Gauss at pole to mag moment
      rinner = rinner*rearth
      rstop = rstop*rearth
      router = router*rearth      ! change to cm from earth radii
      aq = 0.5*(x_max - x_min)*rearth
      bq = r_max*rearth
      xq_off = aq - x_max*rearth
      aq = 1./aq
      bq = 1./bq
      bxq = bx_zero
c  This stuff below is checked in the original code, much more difficult to do in parallel
c  code  Caveat Emptor ( or programmor )
        dr = router - rinner
        a0 = - rinner/dr
        b0 = 1./dr
        a1 = router/dr
        b1 = - 1./dr
c
c

C
C
      EQIN = 0.5*RHOQIN*(VXQIN**2+VYQIN**2+VZQIN**2+
     $     2.*GGM1IN*CQIN**2)
C
c
c     Now set the coeficents in help.inc.
      CALL DERCO(FCOEF,ORDER, local_proc)
C
*      if ( local_proc .eq. 0 ) WRITE (9,112) ORDER
* 112  FORMAT (1X,I6,'-TH ORDER CALCULATION'/' THE FLUXES F THE',
*     $   ' CENTERED DERIVATIVES USE THE FOLLOWING COEFFICIENTS')
*      if ( local_proc .eq. 0 ) WRITE (9,113) (FCOEF(I),I=1,ORDER)
* 113  format( 10(1X,8(1PE14.6)/))
c
      return
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
      entry inital
!
! ... Entry .............................................................
!      
      if (MHD_PE) then
c
c      set up the double precision bi,bj,bk
c
         do k =1,nkp1
            do j=1,njp1
               do i=1,nip1
                  bi_dbl(i,j,k) = bi_g(i,j,k)
                  bj_dbl(i,j,k) = bj_g(i,j,k)
                  bk_dbl(i,j,k) = bk_g(i,j,k)
               enddo
            enddo
         enddo
c
#ifdef DO_RINGAV
c
c     set up ringav
c
         do k = 1,nl_k
            voltmp(k) = volume(nl_i/2,nl_j/2,k)
         enddo
         call ring_setup(voltmp,nl_k,nrings,naver)
c
c   since ringav is called in subdipol->bzz->ringav
c   we need to initialize ringav before calling subdipol
c
#endif
         call flux2ave
         call subdipol
c
c     reshuffle the arrays -- note we copy the guard cells from rho to rho1
c     and so on -- cuts down on communication
c
         do k=klow,khigh
            do j=jlow,jhigh
               do i=ilow,ihigh
                  rho1_g(i,j,k) = rho_g(i,j,k)
                  vx1_g(i,j,k) = vx_g(i,j,k)
                  vy1_g(i,j,k) = vy_g(i,j,k)
                  vz1_g(i,j,k) = vz_g(i,j,k)
                  c1_g(i,j,k)  = c_g(i,j,k)
                  bx1_g(i,j,k) = bx_g(i,j,k)
                  by1_g(i,j,k) = by_g(i,j,k)
                  bz1_g(i,j,k) = bz_g(i,j,k)
                  bi1_g(i,j,k) = bi_g(i,j,k)
                  bj1_g(i,j,k) = bj_g(i,j,k)
                  bk1_g(i,j,k) = bk_g(i,j,k)
               enddo
            enddo
        enddo
        endif
      return
! ... End SUBROUTINE Namelists ..........................................
      end
#endif 
! #endif /* USE_DEPRECATED_NAMELISTS */
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!> DERCO(FCOEF,NORDER, local_proc)
!!
!!
      SUBROUTINE DERCO(FCOEF,NORDER, local_proc)
! ... Local variables ..................................................
c      IF ( LOCAL_PROC .EQ. 0) WRITE (9,112) ORDER,(FCOEF(I),I=1,ORDER)
c      NORDER-TH ORDER CALCULATION for the fluxes F use
c      CENTERED DERIVATIVES USE THE FOLLOWING COEFFICIENTS
c
      DIMENSION FCOEF(NORDER)
      COMMON /SCRACH/
     $     A(200),B(200),C(200),F(200)
! ... Begin ............................................................
#ifdef DEBUG_MODE_ON
      write(*,*) "DEBUG:  in init-fortran.F::DERCO(...)"
#endif
C
      N = NORDER
      N2 = N/2
C
      A(1) = 1.
      DO 200 I = 2,N2
  200 A(I) = A(I-1)*FLOAT(N+2-I)/FLOAT(I-1)
C
      DO 201 I = 1,N2
  201 B(I) = A(N2+1-I)
C
      DO 202 I = 1,N2,2
  202 B(I) = B(I)/FLOAT(I)
C
      DO 203 I = 2,N2,2
  203 B(I) = - B(I)/FLOAT(I)
C
      IF ( LOCAL_PROC .EQ. 0) WRITE (9,*)(A(I),I=1,N2)
  101 FORMAT (5(1PE16.4))
      IF ( LOCAL_PROC .EQ. 0) WRITE (9,*) (B(I),I=1,N2)
      C(1) = 0.
      DO 300 I = 1,N2
  300 C(I+1) = B(I)
C
      F(1) = B(N2)
      DO 400 I = 1,N2
  400 F(I+1) = C(N2+1-I) + F(I)
C
      N2P1 = N2+1
      IF ( LOCAL_PROC .EQ. 0) WRITE(9,*) (C(I),I=1,N2P1)
      IF ( LOCAL_PROC .EQ. 0) WRITE (9,*) (F(I),I=1,N2P1)
      SUM = 0.
      DO 401 I = 1,N2
  401 SUM = SUM + F(I)
      SUM = SUM*2.
C
      DO 410 I = 1,N2
  410 F(I) = F(I)/SUM
C
      DO 500 I = 1,N2
      FCOEF(I) = F(I)
  500 FCOEF(N+1-I) = F(I)
!
! ... End SUBROUTINE DERCO ..............................................
!
      RETURN
      END

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!> dqarea
!!
!!

      function dqarea(eta,psi,dx,dy,dz)
! ... Parameter variables ..............................................
      dimension dx(3),dy(3),dz(3)
#ifdef DEBUG_MODE_ON
      write(*,*) "DEBUG:  in init-fortran.F::dqarea(...)"
#endif
C
      xeta = dx(1) + dx(3)*psi
      yeta = dy(1) + dy(3)*psi
      zeta = dz(1) + dz(3)*psi
C
      xpsi = dx(2) + dx(3)*eta
      ypsi = dy(2) + dy(3)*eta
      zpsi = dz(2) + dz(3)*eta
C
      dqarea = sqrt( (yeta*zpsi-zeta*ypsi)**2 +
     $     (zeta*xpsi-xeta*zpsi)**2 +
     $     (xeta*ypsi-xpsi*yeta)**2 )
! ... End function dqarea ..............................................
      return
      end
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
