#ifndef IM_IC_INTERFACE_H
#define IM_IC_INTERFACE_H

// 3rd party includes
#include "InterComm.h"

// LTR-specific includes
#include "IM_Interface.h"
#include "Check_IC_Status.h"
#include "IC_EndPointSet.h"

/********************************************************************//**
 * Implements data exchange between the LFM MHD code and the inner
 * magnetosphere using InterComm.  
 *
 * Note that the IM_Interface abstract base class manages data arrays
 * and pre/post processing for export/import operations.
 *
 * \see IM_Interface
 ************************************************************************/ 
class IM_IC_Interface : public IM_Interface{
public:
  IM_IC_Interface(IC_EndPointSet *EPS,
		  const floatArray &lfmx, const floatArray &lfmy, const floatArray &lfmz, 
		  const Partitioning_Type &mhdPartitioning,
		  const float &inDipoleMoment, const float &inGamma,
		  const DateTime &start, const int &delta_t);

  virtual ~IM_IC_Interface();

  void Initialize(); 
  void SendScalars(const char *name, const intArray &scalars);
  void Export();
  void Import();

private:
  void igSetup(const intArray &igDims);
  /// Warning:  parallel interpolation not fully implemented/tested!
  void igParallelSetup(const intArray &igDims);
  void igSerialSetup(const intArray &igDims);

  /**\name InterComm Variables */
  //@{
  int ic_err;
  IC_EndPointSet *epset;
  
  IC_Desc *desc;
  IC_Region *region_set[1]; 

  /**
   * The following variables are used by IC_Create_ttable_desc for
   * matching local & global indices for an array distributed across a
   * number of MPI tasks
   */
  int *tasks;
  int *localIndices;
  int *globals;
  int *igGlobalIndices;
  //@}
};

#endif
