      MODULE LFM_to_RCM_transfer_3d
      PRIVATE

      real,parameter :: rearth=6.38e8
      INTEGER, SAVE :: nri, nrj, nrk, nrbig,nbig
      INTEGER, SAVE :: ni,nj,nk,nip1,njp1,njp2,njp3,nkp1,nkp2,nim1
      REAL, SAVE :: tau_rcm, pi
      REAL, ALLOCATABLE, SAVE :: x2mid(:,:), 
     $              y2mid(:,:),
     $              x3mid(:,:,:),
     $              y3mid(:,:,:),
     $              z3mid(:,:,:),
     $              psimid_in(:),
     $              psimid_out(:),
     $              phimid (:)
      REAL, ALLOCATABLE, SAVE :: eta_r(:), chi_r(:), xi_r(:),
     $     ainterp(:,:), points(:,:,:,:),
     $     points1d (:,:)
      INTEGER, ALLOCATABLE, SAVE ::  icube(:)
      
      PUBLIC interp1d_init, interp1d

      CONTAINS

!     Calculate interpolation coefficients (ainterp) to be used in Interp1d.
!       sizes: dimensions of intermediate grid
!       xrect: igX grid
!       yrect: igY grid
!       zrect: igZ grid
!       lfmSizes:  dimensions of LFM grid
!       x2: LFM x-coordinates
!       y2: LFM y-coordinates
!       phi: LFM phi coordinates
      Subroutine interp1d_init(sizes,xrect,yrect,zrect,
     $     lfmSizes,x2,y2,phi)
      IMPLICIT NONE

      INTEGER :: L, Ldum, i,j,k
      INTEGER, INTENT (IN) :: sizes(3),lfmSizes(3)
      real,dimension(sizes(1))::xrect
      real,dimension(sizes(2))::yrect
      real,dimension(sizes(3))::zrect  

      real,dimension(lfmSizes(1)+1,lfmSizes(2)+1)::x2
      real,dimension(lfmSizes(1)+1,lfmSizes(2)+1)::y2
      real,dimension(lfmSizes(3)+1)::phi

      nri = sizes(1)
      nrj = sizes(2)
      nrk = sizes(3)
      nrbig = nri*nrj*nrk

      ALLOCATE (eta_r   (nrbig))
      ALLOCATE (chi_r   (nrbig))
      ALLOCATE (xi_r    (nrbig))
      ALLOCATE (icube   (nrbig))
      ALLOCATE (ainterp (nrbig,8))
      ALLOCATE (points  (3,nri,nrj,nrk), points1d (3,nrbig))

      DO k=1,nrk
         DO j=1,nrj
            DO i=1,nri
               points(1,i,j,k) = xrect(i)
               points(2,i,j,k) = yrect(j)
               points(3,i,j,k) = zrect(k)
            END DO
         END DO
      END DO

      points1d (1,:) = PACK (points(1,:,:,:),MASK=.TRUE.)
      points1d (2,:) = PACK (points(2,:,:,:),MASK=.TRUE.)
      points1d (3,:) = PACK (points(3,:,:,:),MASK=.TRUE.)

      pi = ATAN(1.)*4.

      ni = lfmSizes(1)
      nj = lfmSizes(2)
      nk = lfmSizes(3)
      
      nip1 = ni+1
      njp1 = nj+1
      njp2 = nj+2
      njp3 = nj+3
      nkp1 = nk+1
      nkp2 = nk+2
      nim1 = ni-1
      nbig = ni*njp2*nkp2

      ALLOCATE (x2mid(ni,njp2))
      ALLOCATE (y2mid(ni,njp2))
      ALLOCATE (x3mid(ni,njp2,nkp2))
      ALLOCATE (y3mid(ni,njp2,nkp2))
      ALLOCATE (z3mid(ni,njp2,nkp2))
      ALLOCATE (psimid_in(njp2))
      ALLOCATE (psimid_out(njp2))
      ALLOCATE (phimid (nkp2))

      DO j=1,nj
         DO i=1,ni
            x2mid(i,j+1) = 
     $           0.25*(x2(i,j)+x2(i+1,j)+x2(i,j+1)+x2(i+1,j+1))/rearth
            y2mid(i,j+1) = 
     $           0.25*(y2(i,j)+y2(i+1,j)+y2(i,j+1)+y2(i+1,j+1))/rearth
         END DO
      END DO

      DO i=1,ni
         x2mid(i,   1) = x2mid(i,2)
         x2mid(i,njp2) = x2mid(i,njp1)
         y2mid(i,   1) = 0.
         y2mid(i,njp2) = 0.
      END DO
      
      DO j=1,njp2
         psimid_in(j)  = ATAN2(y2mid(1,j),x2mid(1,j))
         psimid_out(j) = ATAN2(y2mid(ni,j),x2mid(ni,j))
      END DO
      
      DO k=1,nk
         phimid(k) = 0.5*(phi(k)+phi(k+1))
      END DO
      phimid(nkp1) = phimid(1) + 2.*pi
      DO k=1,nk
         DO j=1,njp2
            DO i=1,ni
               x3mid(i,j,k+1) = x2mid(i,j)
               y3mid(i,j,k+1) = y2mid(i,j)*COS(phimid(k))
               z3mid(i,j,k+1) = y2mid(i,j)*SIN(phimid(k))
            END DO
         END DO
      END DO
      
      DO j=1,njp2
         DO i=1,ni
            x3mid(i,j,   1) = x3mid(i,j,nkp1)
            x3mid(i,j,nkp2) = x3mid(i,j,2)
            y3mid(i,j,   1) = y3mid(i,j,nkp1)
            y3mid(i,j,nkp2) = y3mid(i,j,2)
            z3mid(i,j,   1) = z3mid(i,j,nkp1)
            z3mid(i,j,nkp2) = z3mid(i,j,2)
         END DO
      END DO
    
      CALL Intco1d (points1d, nrbig)
      DEALLOCATE (points, points1d)
      DEALLOCATE (eta_r)
      DEALLOCATE (chi_r)
      DEALLOCATE (xi_r)

      return
      END Subroutine interp1d_init

!     Use interpolation coefficients (ainterp) calculated in interp1d_init
!     to interpolate data_in onto datao
      SUBROUTINE Interp1d (datao, data_in, npts)
      IMPLICIT NONE
      INTEGER, INTENT (IN) :: npts
      REAL, INTENT (IN OUT) :: datao(npts), data_in(nip1,njp1,nkp1)
      REAL data3d(ni,njp2,nkp1)
      REAL data1d(ni*njp2*nkp1)

      INTEGER :: nlnmnn,i,j,k, izero,itemp,jtemp,ktemp
      REAL :: sum, av

      nlnmnn = nbig
      DO k=1,nk
      DO j=1,nj
      DO i=1,ni
         data3d(i,j+1,k) = data_in(i,j,k)
      END DO
      END DO
      END DO

      DO i=1,ni
         sum = 0.
         DO k=1,nk
            sum = sum + data_in(i,1,k)
         END DO
         av = sum/REAL(nk)
         DO k=1,nk
            data3d(i,1,k) = sum/REAL(nk)
         END DO
         sum = 0.
         do k=1,nk
            sum = sum + data_in(i,nj,k)
         END DO
         av = sum/REAL(nk)
         DO k=1,nk
            data3d(i,njp2,k) = sum/REAL(nk)
         END DO
      END DO

      do  j=1,njp2
      do  i=1,ni
        data3d(i,j,nkp1) = data3d(i,j,1)
      END DO
      END DO

!     Pack the 3d array as a 1d contiguous block of memory
!
!     FIXME: This wastes memory, but seems to get the job done...  Why
!     do we have it?  To remove a troublesome FORTRAN "equivalence"
!     statement.
      data1d(:) = PACK( data3d(:,:,:), MASK=.TRUE. )

      DO k=1,npts
         izero = icube(k)
         IF (izero .gt. nlnmnn .or. izero .lt. 1) then
*	    write(6,*) 'bad icube at ',i,j,k,izero
!	    write(6,*) 'Something wrong with interpolation at '
!     $           ,i,j,k,izero
            datao(k) = 0.
         ELSE
            datao(k) = data1d(izero)   * ainterp(k,1)
            datao(k) = data1d(izero+ni) * ainterp(k,2) + datao(k)
            datao(k) = data1d(izero+njp2*ni)*ainterp(k,3) + datao(k)
            datao(k) = data1d(izero+njp3*ni)*ainterp(k,4) + datao(k)
            datao(k) = data1d(izero+1) * ainterp(k,5) + datao(k)
            datao(k) = data1d(izero+ni+1) * ainterp(k,6) + datao(k)
            datao(k) = data1d(izero+njp2*ni+1)*ainterp(k,7) +datao(k)
            datao(k) = data1d(izero+njp3*ni+1)*ainterp(k,8) +datao(k)
         END IF
      END DO

      return
      END SUBROUTINE Interp1d

      SUBROUTINE Intco1d (points1d, npts)
      IMPLICIT NONE
      INTEGER, INTENT (in) :: npts
      REAL, INTENT (IN) :: points1d (3,npts)

      INTEGER :: k,j, iint, jint, n, jouter, jinner, kounter, ntries,
     $           iold, jold, ii, jj, i00, j00, k00, L
      REAL :: xdum(2), pv(3), ang_inner(njp1), newdif, phidif(nk),
     $        olddif, rinner, xzero, yzero, rzero, psicart, psx, dp,
     $        xouter, youter, router, ang0, si, sj, ssi, ssj, dx, dy,
     $        xeta, xchi, xetachi, yeta, ychi, yetachi, aa, bb, cc,
     $        disc,bt,etainc,chiinc,et1,et2,ch1,ch2,deta,dchi,dxi,
     $        xcoeff_2, ycoeff_2

      REAL, ALLOCATABLE :: phicart(:), dxiq(:,:,:), dxjq(:,:,:)

      ALLOCATE (phicart(npts))
      ALLOCATE (dxiq (2,nim1,njp2), dxjq (2,ni,njp1))

      DO k=1,npts
        IF (points1d(3,k) == 0.0 .AND. points1d(2,k) == 0.0) THEN
           phicart(k) = 0.0
        ELSE
           phicart(k) = ATAN2 (points1d(3,k),points1d(2,k))
        END IF
        if (phicart(k).lt. phimid(1)) phicart(k) = 2.*pi+phicart(k)
      END DO


      DO k=1,npts
         olddif = phicart(k) - phimid(1)
         DO n=1,nk
           newdif = phicart(k) - phimid(n+1)
           if ( newdif .le. 0.0 .and. olddif .gt. 0.0 ) then
c            here's the cell
             xi_r(k) = REAL(MOD(n+nk-1,nk)+1) + olddif/(olddif-newdif)
              EXIT
           endif
           olddif = newdif
         END DO
      END DO

      DO j=1,njp1
         ang_inner(j) = ATAN2(y2mid(1,j),x2mid(1,j))
      END DO

      DO k=1,njp2
      DO j=1,nim1
         dxiq(1,j,k) = x2mid(j+1,k)-x2mid(j,k)
         dxiq(2,j,k) = y2mid(j+1,k)-y2mid(j,k)
      END DO
      END DO
      DO k=1,njp1
      DO j=1,ni
         dxjq(1,j,k) = x2mid(j,k+1)-x2mid(j,k)
         dxjq(2,j,k) = y2mid(j,k+1)-y2mid(j,k)
      END DO
      END DO

      rinner = 0.0
      DO k=1,njp2
         rinner = MAX(rinner,x2mid(1,k)**2+y2mid(1,k)**2)
      END DO

      iint = 1
      DO k=1,npts
        xzero = points1d(1,k)
        yzero = SQRT(points1d(2,k)**2+points1d(3,k)**2)

        IF (xzero == 0.0 .AND. yzero == 0.0) THEN
          psicart = 0.0
        ELSE
          psicart = ATAN2(yzero,xzero)
        END IF
        rzero = xzero**2 + yzero**2

c   test for outer bound
        olddif = psicart - psimid_out(1)
        DO n=1,njp1
           newdif = psicart - psimid_out(n+1)
           IF ( newdif .le. 0.0 .and. olddif .ge. 0.0 ) then
c            here's the cell
              psx = REAL(n) + olddif/(olddif-newdif)
              EXIT
           END IF 
           olddif = newdif
        END DO
        jouter = psx
        dp = psx - REAL(jouter)
        IF (dp == 0.0) THEN
            xcoeff_2 = 0.0
            ycoeff_2 = 0.0
        ELSE
            xcoeff_2 = x2mid(ni,jouter+1)
            ycoeff_2 = y2mid(ni,jouter+1)
        END IF
        xouter = x2mid(ni,jouter)*(1.-dp) + xcoeff_2*dp
        youter = y2mid(ni,jouter)*(1.-dp) + ycoeff_2*dp
        router = xouter**2 + youter**2

c       check to see if its in bounds

        jinner = 1
        jouter = nj
C
C       use extra 3% as safety factor
c
        IF ( rzero > 1.03*rinner .and. rzero < 0.97*router) then
c
           ntries = 1
           iint = 6
           ang0 = ATAN2(yzero,xzero)
           do L=1,nj
              jint = l
              IF (ang0>=ang_inner(l) .and. ang0<ang_inner(l+1)) EXIT
           END DO
c
  417      continue
c
           DO kounter=1,200
              iold = iint
              jold = jint
              xdum(1) = xzero - x2mid(iint,jint)
              xdum(2) = yzero - y2mid(iint,jint)
              si = SIGN(0.5,cross_1(dxiq(1,iint,jint),xdum))
              sj = SIGN(0.5,cross_1(xdum,dxjq(1,iint,jint)))
              if ( si + sj .gt. 0.75 ) then
                 xdum(1) = xzero - x2mid(iint+1,jint+1)
                 xdum(2) = yzero - y2mid(iint+1,jint+1)
                 ssi = SIGN(0.5,cross_1(xdum,dxiq(1,iint,jint+1)))
                 ssj = SIGN(0.5,cross_1(dxjq(1,iint+1,jint),xdum))
                 if ( ssi + ssj .gt. 0.75 ) then
                    eta_r(k) = REAL(iint)+0.5
                    chi_r(k) = REAL(jint)+0.5
                    go to 1177   ! found cell !! ##############   
                 else
                    if ( ssi .lt. 0. ) jint = MAX(1,MIN(njp1,jint+1))
                    if ( ssj .lt. 0. ) iint = MAX(1,MIN(nim1,iint+1))
                 end if
              else
                 if ( si .lt. 0. ) jint = MAX(1,MIN(njp1,jint-1))
                 if ( sj .lt. 0. ) iint = MAX(1,MIN(nim1,iint-1))
              END IF
              if ( iold .eq. iint .and. jold .eq. jint ) go to 397
           END DO
c
           write(6,*)'tried maximum number of times but could not find',
     $          '  cell, j,k = ',j,k
           go to 395
  397      continue
           if ( ntries .le. 1 ) then
              ntries = ntries+1
              iint = ni
              jint = jouter
              go to 417
           elseif ( ntries .eq. 2) then
              ntries = ntries+1
              iint = ni
              jint = jouter/2
              go to 417 
           else
             write (6,*) 'not in cell, but no update, i,j = ',iint,jint
     $                          ,' i1,j1 =  ',j,k
             write (6,*) 'xzero,yzero',xzero,yzero
           endif
  395      continue
 1177      continue
c
c          figure out where in the cell we are
c
           ii = eta_r(k)
           jj = chi_r(k)
c                       regular quadrilaterals
           dx = xzero - x2mid(ii,jj)
           dy = yzero - y2mid(ii,jj)
           xeta = x2mid(ii+1,jj)-x2mid(ii,jj)
           xchi = x2mid(ii,jj+1)-x2mid(ii,jj)
           xetachi = x2mid(ii+1,jj+1)+x2mid(ii,jj)-x2mid(ii+1,jj)-
     $               x2mid(ii,jj+1)
           yeta = y2mid(ii+1,jj)-y2mid(ii,jj)
           ychi = y2mid(ii,jj+1)-y2mid(ii,jj)
           yetachi = y2mid(ii+1,jj+1)+y2mid(ii,jj)-y2mid(ii+1,jj)-
     $               y2mid(ii,jj+1)
           aa = yeta*xetachi-xeta*yetachi
           bb = xchi*yeta-xeta*ychi +yetachi*dx - xetachi*dy
           cc = ychi*dx - xchi*dy
           disc = SQRT(bb**2-4*aa*cc)/(2.*aa)
           bt = -bb/(2.*aa)
           if ( bt + disc .ge. 0. .and. bt + disc .le. 1. ) then
                etainc = bt +disc
           else
                etainc = bt - disc
           endif
           if ( ABS(xchi) .ge. ABS(ychi) ) then 
             chiinc = (dx -xeta*etainc)/(xchi+xetachi*etainc)
           else
             chiinc = (dy -yeta*etainc)/(ychi+yetachi*etainc)
           endif
           eta_r(k) = REAL(ii) + etainc
           chi_r(k) = REAL(jj) + chiinc
           if ( etainc .gt. 1. .or. etainc .lt. 0. .or.
     $          chiinc .gt. 1. .or. chiinc .lt. 0. ) then
             write (6,*) 'bad solution for position within cell',
     $          '   i1,j1,i2,j2,eta,chi  ',j,k,ii,jj,etainc,chiinc
                et1 = bt+disc
           ch1 = (dx -xeta*et1)/(xchi+xetachi*et1)
                et2 = bt-disc
           ch2 = (dx -xeta*et2)/(xchi+xetachi*et2)
             WRITE (6,*) 'bt,disc,+ and -',bt,disc,et1,ch1,et2,ch2
           end if
c
           i00 = eta_r(k)
           j00 = chi_r(k)
           k00 = xi_r(k)
           deta = eta_r(k)-REAL(i00)
           dchi = chi_r(k)-REAL(j00)
           dxi  = xi_r(k)-REAL(k00)
           icube(k) = (k00-1)*ni*njp2+(j00-1)*ni+i00
           ainterp(k,1) = (1.-deta)*(1.-dchi)*(1.-dxi)
           ainterp(k,2) = (1.-deta)*dchi*(1.-dxi)
           ainterp(k,3) = (1.-deta)*(1.-dchi)*dxi
           ainterp(k,4) = (1.-deta)*dchi*dxi
           ainterp(k,5) = deta*(1.-dchi)*(1.-dxi)
           ainterp(k,6) = deta*dchi*(1.-dxi)
           ainterp(k,7) = deta*(1.-dchi)*dxi
           ainterp(k,8) = deta*dchi*dxi
        ELSE
!       else if ( rzero > 1.03*rinner .and. rzero < 0.97*router) ...
!           write(*,*) "WARNING:  rzero=", rzero,"."
!           write(*,*) "Does intermediate grid have a pt at (0,0,0?"
!           write(*,*) "ig at k=",k,"is:"
!           write(*,*) points1d(1,k), points1d(2,k), points1d(3,k)
!           write(*,*) "The interpolation algorithm may not handle"
!           write(*,*) "points near the origin very well :-("
c         just to be safe give eta1 and chi1 values to things in the hole
           eta_r(k) = 2.0
           chi_r(k) = 2.0
           i00 = eta_r(k)
           j00 = chi_r(k)
           k00 = xi_r(k)
*          icube(k) = (k00-1)*ni*njp2+(j00-1)*ni+i00
           icube(k) = -1
c  -1 is flag to place a zero in the interpolated value
           ainterp(k,:) = 0.
        END IF
      END DO

      DEALLOCATE (phicart, dxiq, dxjq)

      return
      END SUBROUTINE Intco1d

      REAL FUNCTION Cross_1(x1,x2)
      IMPLICIT NONE
      REAL, INTENT (IN) :: x1(2), x2(2)
        cross_1 = x1(1)*x2(2) - x2(1)*x1(2)
      return
      END FUNCTION Cross_1

      END MODULE LFM_to_RCM_transfer_3d

      
