! Expose methods within Fortran 90 module
!
!     Why?  C++ cannot directly call functions/subroutines in a Fortran
!     90 module.
!

!     Wrapper that calls function defined in f90 module to calculate
!     interpolation coefficients (ainterp) to be used in Interp1d.
!
!       sizes: dimensions of intermediate grid
!       xrect: igX grid
!       yrect: igY grid
!       zrect: igZ grid
!       lfmSizes:  dimensions of LFM grid
!       x2: LFM x-coordinates
!       y2: LFM y-coordinates
!       phi: LFM phi coordinates
      subroutine interp1d_init_wrapper(igDims,igX,igY,igZ,
     $     lfmDims,x2,y2,phi)
      USE LFM_to_RCM_transfer_3d

      implicit none
      integer igDims(3)     
      real,dimension(igDims(1))::igX
      real,dimension(igDims(2))::igY
      real,dimension(igDims(3))::igZ

      integer lfmDims(3)
      real,dimension(lfmDims(1)+1,lfmDims(2)+1)::x2
      real,dimension(lfmDims(1)+1,lfmDims(2)+1)::y2
      real,dimension(lfmDims(3)+1)::phi

c     VGM: Because I don't know fortran 90 well enough, I'll leave this
c     comment here. Below does not work!
c$$$ Integer igDims(:),lfmDims(:)
c$$$ real :: igX(:),igY(:),igZ(:)
c$$$ real :: x2(:,:),y2(:,:),phi(:)
      write(*,*) "lfmDims:  ", lfmDims
      write(*,*) "igSizes:   ", igDims
      CALL interp1d_init(igDims,igX,igY,igZ, lfmDims,x2,y2,phi)
      end subroutine interp1d_init_wrapper

!     Wrapper that calls function defined in f90 module to interpolate data_in onto data_out
!     
!       data_out:  data on intermediate grid interpoalted from data_in
!       sizes_out: dimensinos of intermediate grid
!       data_in:   data on LFM grid 
!       sizes_in:  dimensions of LFM grid
      subroutine Interp1d_wrapper(data_out,sizes_out, data_in,sizes_in)
      USE LFM_to_RCM_transfer_3d

      implicit none

      integer sizes_out(3),nnn,sizes_in(3)

      real,dimension(sizes_in(1),sizes_in(2),sizes_in(3))::data_in
      real,dimension(sizes_out(1)*sizes_out(2)*sizes_out(3))::data_out

      nnn = sizes_out(1)*sizes_out(2)*sizes_out(3)
      
      CALL Interp1d(data_out,data_in,nnn)
      end subroutine Interp1d_wrapper
