#ifndef __CONST_H__
#define __CONST_H__

//!< Global interface constant definitions

//!< Maximum length of a character string
static const int STRLEN = 256;

//!< The radius of the ionosphere in cm
static const double RION = 6.5e8;

//!< The radius of Earth in cm
static const double REARTH = 6.38e8; 

 //!< Earth's dipole magnetic field at the equator in Tesla (cf. Baumjohann, p. 33)
static const double BEQUATOR = 3.0e-5;

#ifndef PI
//!< I'd like a slice of the Rhubarb, please.
static const double PI = 3.1415926;
#endif

/********************************************************************//**
 * Indices for scalars to be sent to coupler
 * 
 * Note: These must match up exactly with the scalars on the receiving
 * side (in coupler code).
 ************************************************************************/ 
namespace CPL_Interface
{
  enum CPL_Scalars{
    KILL_SIGNAL       = 0, // 111=kil; 222=last exchange; else=keep running
    YEAR              = 1,
    MONTH             = 2,
    DAY               = 3,
    HOUR              = 4,
    MINUTE            = 5,
    SECOND            = 6,
    TILT              = 7,
    LABEL             = 8,
    EXCHANGE_WITH_IM = 9,
    NUMBER_OF_SCALARS = 10
  };
  // Scalars at index KILL_SIGNAL accept the following:
  enum CPL_kill_signal{
    KILL_SIGNAL_CONTINUE = 0,
    KILL_SIGNAL_SHUTDOWN = 111,
    KILL_SIGNAL_LAST_EXCHANGE = 222
  };
  // Scalars at index EXCHANGE_WITH_ITM accept the following indices: 
  enum CPL_exchange_with_IM{
    NO_IM_EXCHANGE     = 0,
    CPL_RECV_FROM_IM   = 1,
    CPL_SEND_TO_IM     = 2,
    DUPLEX_IM_EXCHANGE = 3   // coupler receives/sends from/to inner magnetosphere
  };
}

#endif
