!     What we need to send to RCM is the magnetic field perturbation
!     relative to the dipole field, not the background field LFM is
!     using. Here are the steps to proceed:
!
!     1. Get Bi, Bj, Bk from the MHD class. These are the magnetic field
!     perturbations at the cell faces. 

!     2. We need to add the LFM background field to Bi, Bj, Bk,
!     i.e. bnqfi, bnqfj, bnqfk, as defined in subdipol.F and stored in
!     the bzero.inc common block. (this step is essentially what the
!     subroutine fixBout in fixB.F is doing.)
!
!     3. Subtract the dipole field deteremined at the cell faces. We do
!     this by invoking the initmag_dip function below which calculates
!     the field using the Stokes theorem.
!
!     4. Finally, call Bzz to convert the calculated field perturbation
!     to cell centered values.



      subroutine initmag_dip
!     Nothing goes unused! We reuse (somewhat modified) the initmag_dip
!     function here which was first hacked from the original (John's)
!     initmag function by me (VGM). The function calculates the dipole
!     field at cell centers
!
      USE bdip

      include 'global_dims.inc'
      include 'mhd_P++_pointers.inc'
      include 'fortran_P++_arrays.inc'
c
c     * x,y,z, face_i, face_j, face_k arrays are passed through
c     mhd_P++_pointers.inc
C
C     * local dimensions are known from global_dims.inc
c
c     integrate the magnetic vector potential around the edges
c     of each face to get dipole field.
      dimension aintqj(nl_i+1,nl_j,nl_k+1),aintqk(nl_i+1,nl_j+1,nl_k),
     $     aintqi(nl_i,nl_j+1,nl_k+1)

      external ax_dip,ay_dip,gint
      
      allocate (bidip(nl_i+1,nl_j,nl_k))
      allocate (bjdip(nl_i,nl_j+1,nl_k))
      allocate (bkdip(nl_i,nl_j,nl_k+1))
c
        do 300 k=1,nl_k
        do 300 j=1,nl_j+1
        do 300 i=1,nl_i+1
           aintqk(i,j,k) = (x(i,j,k+1)-x(i,j,k))*
     $          gint(
     $          ax_dip,x(i,j,k),y(i,j,k),z(i,j,k),
     $          x(i,j,k+1),y(i,j,k+1),z(i,j,k+1)
     $          ) +
     $          (y(i,j,k+1)-y(i,j,k))*
     $          gint(ay_dip,
     $          x(i,j,k),y(i,j,k),z(i,j,k),
     $          x(i,j,k+1),y(i,j,k+1),z(i,j,k+1)
     $          )
  300   continue
c
        do 310 k=1,nl_k+1
        do 310 j=1,nl_j
        do 310 i=1,nl_i+1
 310       aintqj(i,j,k) = (x(i,j+1,k)-x(i,j,k))*
     $          gint(
     $          ax_dip,x(i,j,k),y(i,j,k),z(i,j,k),
     $          x(i,j+1,k),y(i,j+1,k),z(i,j+1,k)
     $          ) +
     $          (y(i,j+1,k)-y(i,j,k))*
     $          gint(ay_dip,
     $          x(i,j,k),y(i,j,k),z(i,j,k)
     $          ,x(i,j+1,k),y(i,j+1,k),z(i,j+1,k)
     $          )
c
        do 320 k=1,nl_k+1
        do 320 j=1,nl_j+1
        do 320 i=1,nl_i
 320       aintqi(i,j,k) = (x(i+1,j,k)-x(i,j,k))*
     $          gint(ax_dip,
     $          x(i,j,k),y(i,j,k),z(i,j,k),
     $          x(i+1,j,k),y(i+1,j,k),z(i+1,j,k)
     $          ) +
     $          (y(i+1,j,k)-y(i,j,k))*
     $          gint(ay_dip,
     $          x(i,j,k),y(i,j,k),z(i,j,k),
     $          x(i+1,j,k),y(i+1,j,k),z(i+1,j,k)
     $          )
c
c     now use stokes theorem to compute the magnetic fluxes through
c     the faces from the line integrals of the vector potential.
        do 340 k=1,nl_k
        do 340 j=1,nl_j
        do 340 i=1,nl_i+1
 340       bidip(i,j,k) = (aintqj(i,j,k) - aintqj(i,j,k+1)
     $          + aintqk(i,j+1,k)
     $          - aintqk(i,j,k))/face_i(i,j,k)
c
        do 350 k=1,nl_k
        do 350 j=1,nl_j+1
        do 350 i=1,nl_i
 350       bjdip(i,j,k) =-( aintqi(i,j,k) - aintqi(i,j,k+1) +
     $          aintqk(i+1,j,k)
     $          - aintqk(i,j,k))/face_j(i,j,k)
c
        do 360 k=1,nl_k+1
        do 360 j=1,nl_j
        do 360 i=1,nl_i
 360       bkdip(i,j,k) = ( aintqi(i,j,k) - aintqi(i,j+1,k)
     $          + aintqj(i+1,j,k)
     $          - aintqj(i,j,k))/face_k(i,j,k)
c
      return
      end subroutine initmag_dip

      subroutine add_back_and_subtract_dipole_at_cell_faces(bi,bj,bk)
      !> Add background field & Subtract stored dipole
      !! (bidip, bjdip, bkdip arrays) from input
      !! magnetic field (bi,bj,bk) at cell faces 
      !!
      !! NOTE, almost identical to fixBout
      USE bdip
#include "../global_dims.inc"
#include "../mhd_P++_pointers.inc"
#include "../fortran_P++_arrays.inc"
#include "../param.inc"
#include "../bzero.inc"

      real:: bi(ilow:ihigh,jlow:jhigh,klow:khigh)
      real:: bj(ilow:ihigh,jlow:jhigh,klow:khigh)
      real:: bk(ilow:ihigh,jlow:jhigh,klow:khigh)

! Add background field (bnqf*) & subtract dipole (b*dip) on cell faces
      do k=1,nl_k
         do j=1,nl_j
            do i=1,nl_i+1
               bi(i,j,k) = bi(i,j,k)+bnqfi(i,j,k)-bidip(i,j,k)
            enddo
         enddo
      enddo
c     
      do k=1,nl_k
         do j=1,nl_j+1
            do i=1,nl_i
               bj(i,j,k) = bj(i,j,k)+bnqfj(i,j,k)-bjdip(i,j,k)
            enddo
         enddo
      enddo
c     
      do k=1,nl_k+1
         do j=1,nl_j
            do i=1,nl_i
               bk(i,j,k) = bk(i,j,k)+bnqfk(i,j,k)-bkdip(i,j,k)
            enddo
         enddo
      enddo
      
      return
      end subroutine add_back_and_subtract_dipole_at_cell_faces

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine add_background_field(bi,bj,bk)
      !! Add background field (bnqf{i,j,k}) to magnetic field (b{i,j,k})
      !> at cell centers.  This is similar to fixBout.  
#include "../global_dims.inc"
#include "../mhd_P++_pointers.inc"
#include "../fortran_P++_arrays.inc"
#include "../param.inc"
#include "../bzero.inc"

      real:: bi(ilow:ihigh,jlow:jhigh,klow:khigh)
      real:: bj(ilow:ihigh,jlow:jhigh,klow:khigh)
      real:: bk(ilow:ihigh,jlow:jhigh,klow:khigh)

! Add background field (bnqf*) & on cell faces
      do k=1,nl_k
         do j=1,nl_j
            do i=1,nl_i+1
               bi(i,j,k) = bi(i,j,k)+bnqfi(i,j,k)
            enddo
         enddo
      enddo
c     
      do k=1,nl_k
         do j=1,nl_j+1
            do i=1,nl_i
               bj(i,j,k) = bj(i,j,k)+bnqfj(i,j,k)
            enddo
         enddo
      enddo
c     
      do k=1,nl_k+1
         do j=1,nl_j
            do i=1,nl_i
               bk(i,j,k) = bk(i,j,k)+bnqfk(i,j,k)
            enddo
         enddo
      enddo
      
      return
      end subroutine add_background_field

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      subroutine subtract_analytic_dipole_at_cell_centers(bx,by,bz)
      !> Subtract analytic dipole (bxdip,bydip,bzdip functions) from
      !! input magnetic field (bx,by,bz) at cell centers.
      !!
      !! NOTE, almost identical to fixBout & subtract_dipole
      USE bdip
#include "../global_dims.inc"
#include "../mhd_P++_pointers.inc"
#include "../fortran_P++_arrays.inc"
#include "../param.inc"
#include "../bzero.inc"

      real:: bx(ilow:ihigh,jlow:jhigh,klow:khigh)
      real:: by(ilow:ihigh,jlow:jhigh,klow:khigh)
      real:: bz(ilow:ihigh,jlow:jhigh,klow:khigh)
      
      ! Cell-centers coordinates
      real:: xC, yC, zC

! Add background field (bnqf*) & subtract dipole (b*dip) on cell faces
      do k=1,nl_k
         do j=1,nl_j
            do i=1,nl_i
               ! Calc cell centers, needed by b{i,j,k}dip
               xC = 0.125*(x(i,j,k)+x(i,j+1,k)+x(i,j,k+1)+x(i,j+1,k+1) + 
     $              x(i+1,j,k)+x(i+1,j+1,k)+x(i+1,j,k+1)+x(i+1,j+1,k+1))
               yC = 0.125*(y(i,j,k)+y(i,j+1,k)+y(i,j,k+1)+y(i,j+1,k+1) + 
     $              y(i+1,j,k)+y(i+1,j+1,k)+y(i+1,j,k+1)+y(i+1,j+1,k+1))
               zC = 0.125*(z(i,j,k)+z(i,j+1,k)+z(i,j,k+1)+z(i,j+1,k+1) +
     $              z(i+1,j,k)+z(i+1,j+1,k)+z(i+1,j,k+1)+z(i+1,j+1,k+1))
               
               bx(i,j,k) = bx(i,j,k)-bxdip(xC,yC,zC)
               by(i,j,k) = by(i,j,k)-bydip(xC,yC,zC)
               bz(i,j,k) = bz(i,j,k)-bzdip(xC,yC,zC)

! For debugging purposes, you may wish to set the bx,by,bz to exactly
! the LFM numerical dipole:
!               bx(i,j,k) = bxdip(xC,yC,zC)
!               by(i,j,k) = bydip(xC,yC,zC)
!               bz(i,j,k) = bzdip(xC,yC,zC)
            enddo
         enddo
      enddo
      
      return
      end subroutine subtract_analytic_dipole_at_cell_centers


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!     Clean up dipole fields
      subroutine subtract_dipole_cleanup
      USE bdip
      deallocate (bidip)
      deallocate (bjdip)
      deallocate (bkdip)
      return
      end subroutine subtract_dipole_cleanup
