!> mhd_fortran_arrays.F
!!
!! used to get consistent definition with MHD.C
!!
#include "variables.h"
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!> mhd_fortran_arrays
!!
!! pt_array is an array of data pointers to plasma variables
!! See MHD::inital defined in MHD.C for details.  Units of many
!! variables may be found in MHD.h
!! 
      subroutine mhd_fortran_arrays(pt_array,
     $                i_info,j_info,k_info,myproc,mhd_proc)
! ... Local variables ..................................................
#include "param.inc"
#include "global_dims.inc"
#include "mhd_P++_pointers.inc"
      logical mhd_proc
! ... Parameter variables ..............................................
#ifdef PT8BYTE
      integer*8  pt_array(0:99)
#else
      integer*4  pt_array(0:99)
#endif
      integer i_info(6),j_info(6),k_info(6)
*      dimension x(:,:,:)
! ... Begin ............................................................
      MHD_PE = mhd_proc
c                             everybody should know this stuff
#ifdef DEBUG_MODE_ON
      write(*,*) "DEBUG:  in mhd_fortran_arrays.F::mhd_fortran_arr(...)"
#endif
      ni_global = i_info(6) 
      nj_global = j_info(6)
      nk_global = k_info(6)
c
c
      if ( .not. mhd_proc ) return
      mype = myproc
      ilow = i_info(1)
      ihigh = i_info(2)
      iactive = i_info(3)
      nl_i = iactive
      i_global_off = i_info(4)
      if ( i_global_off .eq. 0 ) then
         ilow_bound = .true.
      else 
         ilow_bound = .false.
      endif
      if ( i_global_off + iactive .eq. ni_global ) then
         ihigh_bound = .true.
      else
         ihigh_bound = .false.
      endif
c
c
      jlow = j_info(1)
      jhigh = j_info(2)
      jactive = j_info(3)
      nl_j = jactive
      j_global_off = j_info(4)
      if ( j_global_off .eq. 0 ) then
         jlow_bound = .true.
      else 
         jlow_bound = .false.
      endif
      if ( j_global_off + jactive .eq. nj_global ) then
         jhigh_bound = .true.
      else
         jhigh_bound = .false.
      endif
c
c
      klow = k_info(1)
      khigh = k_info(2)
      kactive = k_info(3)
      nl_k = kactive
      k_global_off = k_info(4)
      if ( k_global_off .eq. 0 ) then
         klow_bound = .true.
      else 
         klow_bound = .false.
      endif
      if ( k_global_off + kactive .eq. nk_global ) then
         khigh_bound = .true.
      else
         khigh_bound = .false.
      endif
c
c
c

      px = pt_array( _x)
      py = pt_array( _y)
      pz = pt_array( _z)
      pvol = pt_array( _vol)
      pvolinv = pt_array( _volinv) 
      pface_i = pt_array( _face_i)
      ptran_i = pt_array( _tran_i) 
      pface_j = pt_array( _face_j)
      ptran_j = pt_array( _tran_j) 
      pface_k = pt_array( _face_k)
      ptran_k = pt_array( _tran_k) 
      pedge_i = pt_array( _edge_i)
      pvec_i = pt_array( _vec_i) 
      pedge_j = pt_array( _edge_j)
      pvec_j = pt_array( _vec_j) 
      pedge_k = pt_array( _edge_k)
      pvec_k = pt_array( _vec_k)
      pcell_length = pt_array( _cell_length)
      prho_g =  pt_array( _rho)
      pvx_g =  pt_array( _vx)
      pvy_g =  pt_array( _vy)
      pvz_g =  pt_array( _vz)
      pc_g =  pt_array( _c)
      pp_g =  pt_array( _p)
      pbx_g =  pt_array( _bx)
      pby_g =  pt_array( _by)
      pbz_g =  pt_array( _bz)
      pbi_g =  pt_array( _bi)
      pbj_g =  pt_array( _bj)
      pbk_g =  pt_array( _bk)
      pei_g =  pt_array( _ei)
      pej_g =  pt_array( _ej)
      pek_g =  pt_array( _ek)

c
      prho1_g =  pt_array( _rho1)
      pvx1_g =  pt_array( _vx1)
      pvy1_g =  pt_array( _vy1)
      pvz1_g =  pt_array( _vz1)
      pc1_g =  pt_array( _c1)
      pp1_g =  pt_array( _p1)
      pbx1_g =  pt_array( _bx1)
      pby1_g =  pt_array( _by1)
      pbz1_g =  pt_array( _bz1)
      pbi1_g =  pt_array( _bi1)
      pbj1_g =  pt_array( _bj1)
      pbk1_g =  pt_array( _bk1)

c
      call rho_display(prho_g)
! ... End subroutine mhd_fortran_arrays .................................
      return
      end
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!> rho_display
!!
      subroutine rho_display(rho_g)
! ... Local variables ..................................................
#include "global_dims.inc"
! ... Parameter variables ..............................................
!     dimension rho_g(ilow:ihigh,jlow:jhigh,klow:khigh)
      real, dimension (ilow:ihigh, jlow:jhigh,klow:khigh) :: rho_g
! ... Begin ............................................................
#ifdef DEBUG_MODE_ON
      write(*,*) "DEBUG:  in mhd_fortran_arrays.F::rho_display(...)"
#endif
#ifdef DEBUG_MODE_ON
      write (6,*) 'mhd_fortran_arrays:  process ', my_pe
      write (6,*) my_pe, 'ilow ihigh jlow jhigh klow khigh:', 
     $     ilow,ihigh,jlow,jhigh,klow,khigh
      write (6,*) my_pe, 'i_global_off, j_global_off, k_global_off:',
     $     i_global_off,j_global_off,k_global_off
      
      write (6,*) my_pe, 'nl_i, nl_j, nl_k = ', nl_i, nl_j, nl_k
      write (6,*) my_pe, 'rho_g(1,1,1), rh_g(nl_i,nl_j,nl_k) = ',
     $     rho_g(1,1,1), rho_g(nl_i,nl_j,nl_k)

      write (6,*) my_pe, 
     $     'rho_g(ilow,jlow,klow)', rho_g(ilow,jlow,klow),
     $     'rho_g(ihigh,jhigh,khigh)', rho_g(ihigh,jhigh,khigh)
#endif
! ... End subroutine rho_display .......................................
      return
      end
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
