#ifndef __INTEGRATION_H__
#define __INTEGRATION_H__

#include "constants.h"

float gaussianLineIntegral( float (*f)(const float &x, const float &y, const float &z), 
			    const float &xa, const float &ya, const float &za,
			    const float &xb, const float &yb, const float &zb);

/******************************************************************************/
// Vector Potentials for initial load (and associated constants)

namespace vectorPotential
{
  static const float SINNER  = 0.96;
  static const float SOUTTER = 1.05;
  static const float X_MAX = 22.0*constants::REARTH;
  static const float X_MIN = -185.0*constants::REARTH;
  static const float R_MAX = 80.0*constants::REARTH;
  static const float XQ_OFF = (X_MAX+X_MIN)/2.0;
  static const float AQ = 2.0/(X_MAX-X_MIN);
  static const float BQ = 1.0/R_MAX;
  static const float BX_ZERO = 0.0;
  static const float BXQ = BX_ZERO;
  static const float COS_TILT = 1.0;
  static const float SIN_TILT = 0.01;

  float ax(const float &x, const float &y, const float &z);
  float ay(const float &x, const float &y, const float &z);
  float az(const float &x, const float &y, const float &z);
}

#endif
