#include "LfmIon.h"
#include "revision.h"

LfmIon::LfmIon(const int &ioType, IonVars *ion) : Model(ioType)
{
  this->ion = ion;
}

void LfmIon::readVariables(DateTime &ut)
{
  io->readMetaData(metaDoubles, metaFloats, metaInts, metaStrings);

  ut = DateTime( getMetaDouble("mjd") );

  io->readVariable(string("x2ion"), ion->grid.x2);
  io->readVariable(string("y2ion"), ion->grid.y2);
  io->readVariable(string("x_interp"), ion->grid.x_interp);
  io->readVariable(string("y_interp"), ion->grid.y_interp);

  io->readVariable(string("rho0"), ion->rho0);
  io->readVariable(string("c0"), ion->c0);

  io->readVariable(string("curnorth"), ion->currentN);
  io->readVariable(string("potnorth"), ion->potentialN);
  io->readVariable(string("SigmaH_north"), ion->sigmaHallN);
  io->readVariable(string("SigmaP_north"), ion->sigmaPedN);
  io->readVariable(string("fluxnorth"), ion->fluxN);
  io->readVariable(string("avE_north"), ion->avgEnergyN);

  io->readVariable(string("cursouth"), ion->currentS);
  io->readVariable(string("potsouth"), ion->potentialS);
  io->readVariable(string("SigmaH_south"), ion->sigmaHallS);
  io->readVariable(string("SigmaP_south"), ion->sigmaPedS);
  io->readVariable(string("fluxsouth"), ion->fluxS);
  io->readVariable(string("avE_south"), ion->avgEnergyS);
}

/**
 * FIXME: set units on variables
 */
void LfmIon::writeVariables(const DateTime &ut)
{
  setMeta(ut);

  io->writeMetaData(metaDoubles, metaFloats, metaInts, metaStrings);

  io->writeVariable(string("x2ion"), ion->grid.x2, string("not set"));
  io->writeVariable(string("y2ion"), ion->grid.y2, string("not set"));
  io->writeVariable(string("x_interp"), ion->grid.x_interp, string("not set"));
  io->writeVariable(string("y_interp"), ion->grid.y_interp, string("not set"));

  io->writeVariable(string("rho0"), ion->rho0, string("not set"));
  io->writeVariable(string("c0"), ion->c0, string("not set"));

  io->writeVariable(string("curnorth"), ion->currentN, string("not set"));
  io->writeVariable(string("potnorth"), ion->potentialN, string("not set"));
  io->writeVariable(string("SigmaH_north"), ion->sigmaHallN, string("not set"));
  io->writeVariable(string("SigmaP_north"), ion->sigmaPedN, string("not set"));
  io->writeVariable(string("fluxnorth"), ion->fluxN, string("not set"));
  io->writeVariable(string("avE_north"), ion->avgEnergyN, string("not set"));

  io->writeVariable(string("cursouth"), ion->currentS, string("not set"));
  io->writeVariable(string("potsouth"), ion->potentialS, string("not set"));
  io->writeVariable(string("SigmaH_south"), ion->sigmaHallS, string("not set"));
  io->writeVariable(string("SigmaP_south"), ion->sigmaPedS, string("not set"));
  io->writeVariable(string("fluxsouth"), ion->fluxS, string("not set"));
  io->writeVariable(string("avE_south"), ion->avgEnergyS, string("not set"));
}


///FIXME:  should I create "LfmIonAttributes" class to deal with these?  
void LfmIon::setMeta(const DateTime &ut)
{

  appendMeta("mjd", (double) ut.getMJD() );

  appendMeta("time_step", (int) 0);

  appendMeta("time", (float) 0.0);
  appendMeta("time_8byte", (double) 0.0);

  /*   Use properties (svn:keywords) to add revision meta data to HDF file.
   *   http://svnbook.red-bean.com/en/1.1/ch07s02.html
   *
   *   Subversion defines five variables:
   *      - Date
   *      - Revision
   *      - Author
   *      - HeadURL
   *      - Id (a combination of the above)
   *
   *   These will auto-updated when the property is set for a source file:
   *
   *      svn propset svn:keywords "Id" [filename]
   *
   *   For now, we use the Id...
   *
   *   Where's $GlobalRev$?
   *      http://svnbook.red-bean.com/en/1.4/svn.advanced.props.special.keywords.html
   *
   *   Set Global Revision via method described here:
   *       http://subversion.tigris.org/faq.html#version-value-in-source
   */
  string ioRevision("$Id: LfmIon.C 1598 2011-03-17 19:46:19Z schmitt $");
  appendMeta("I/O Revision", ioRevision);

  // REPOSITORY_REVISION_NUMBER passed in via "revision.h"
  appendMeta("Repository Revision", REPOSITORY_REVISION_NUMBER);

  //FIXME:  Fill in with correct dimensions?
  string fileContents("Ionosphere for 3D-MHD  ni  nj  nk  no");
  appendMeta("file_contents", fileContents);

  string writtenBy = string(__FILE__) + " " + string(__FUNCTION__) + string(": HDF4");
  appendMeta("written_by", writtenBy);
}
