#include "LfmMhd.h"
#include "revision.h"

LfmMhd::LfmMhd(const int &ioType, MhdVars *mhd) : Model(ioType)
{
  this->mhd = mhd;
}

void LfmMhd::readVariables(DateTime &ut)
{
  io->readMetaData(metaDoubles, metaFloats, metaInts, metaStrings);

  ut = DateTime( getMetaDouble("mjd") );

  io->readVariable(string("Y_grid"), mhd->grid.y);
  io->readVariable(string("Z_grid"), mhd->grid.z);

  io->readVariable(string("rho_"), mhd->rho);

  io->readVariable(string("vx_"), mhd->vx);
  io->readVariable(string("vy_"), mhd->vy);
  io->readVariable(string("vz_"), mhd->vz);

  io->readVariable(string("c_"), mhd->c);

  io->readVariable(string("bx_"), mhd->bx);
  io->readVariable(string("by_"), mhd->by);
  io->readVariable(string("bz_"), mhd->bz);

  io->readVariable(string("bi_"), mhd->bi);
  io->readVariable(string("bj_"), mhd->bj);
  io->readVariable(string("bk_"), mhd->bk);

  io->readVariable(string("ei_"), mhd->ei);
  io->readVariable(string("ej_"), mhd->ej);
  io->readVariable(string("ek_"), mhd->ek);
}

void LfmMhd::writeVariables(const DateTime &ut)
{
  setMeta(ut);

  io->writeMetaData(metaDoubles, metaFloats, metaInts, metaStrings);
  
  // Here's some code to add strings:
  //io->writeString(string("dataset name"), string("string value"));

  io->writeVariable(string("X_grid"), mhd->grid.x, string("cm"));
  io->writeVariable(string("Y_grid"), mhd->grid.y, string("cm"));
  io->writeVariable(string("Z_grid"), mhd->grid.z, string("cm"));

  io->writeVariable(string("rho_"), mhd->rho, string("gm/cm^3"));

  io->writeVariable(string("vx_"), mhd->vx, string("cm/sec"));
  io->writeVariable(string("vy_"), mhd->vy, string("cm/sec"));
  io->writeVariable(string("vz_"), mhd->vz, string("cm/sec"));

  io->writeVariable(string("c_"), mhd->c, string("cm/sec"));

  io->writeVariable(string("bx_"), mhd->bx, string("Gauss"));
  io->writeVariable(string("by_"), mhd->by, string("Gauss"));
  io->writeVariable(string("bz_"), mhd->bz, string("Gauss"));

  io->writeVariable(string("bi_"), mhd->bi, string("Gauss*cm^2 flux through face"));
  io->writeVariable(string("bj_"), mhd->bj, string("Gauss*cm^2 flux through face"));
  io->writeVariable(string("bk_"), mhd->bk, string("Gauss*cm^2 flux through face"));

  io->writeVariable(string("ei_"), mhd->ei, string("Gauss-cm^2/s vxB dl on edge"));
  io->writeVariable(string("ej_"), mhd->ej, string("Gauss-cm^2/s vxB dl on edge"));
  io->writeVariable(string("ek_"), mhd->ek, string("Gauss-cm^2/s vxB dl on edge"));
}


///FIXME:  Should I create an "LfmMhdAttributes" class to deal with these?
void LfmMhd::setMeta(const DateTime &ut)
{
  appendMeta("no_solar_wind", "no  SW-SM-DAT");

  appendMeta("mjd", (double) ut.getMJD());

  appendMeta("time_step", (int) 0);

  appendMeta("time", (float) 0.0);
  appendMeta("time_8byte", (double) 0.0);

  appendMeta("tilt_angle", (float) 0.0);

  /*   Use properties (svn:keywords) to add revision meta data to HDF file.
   *   http://svnbook.red-bean.com/en/1.1/ch07s02.html
   *
   *   Subversion defines five variables:
   *      - Date
   *      - Revision
   *      - Author
   *      - HeadURL
   *      - Id (a combination of the above)
   *
   *   These will auto-updated when the property is set for a source file:
   *
   *      svn propset svn:keywords "Id" [filename]
   *
   *   For now, we use the Id...
   *
   *   Where's $GlobalRev$?
   *      http://svnbook.red-bean.com/en/1.4/svn.advanced.props.special.keywords.html
   *
   *   Set Global Revision via method described here:
   *       http://subversion.tigris.org/faq.html#version-value-in-source
   */
  string ioRevision("$Id: LfmMhd.C 1598 2011-03-17 19:46:19Z schmitt $");
  appendMeta("I/O Revision", ioRevision);

  // REPOSITORY_REVISION_NUMBER passed in via "revision.h"
  appendMeta("Repository Revision", REPOSITORY_REVISION_NUMBER );

  //FIXME:  Set NORDER=8 from a variable.
  stringstream fileContents_ss;
  fileContents_ss << "3D-MHD  "
		  <<  mhd->grid.x.getLength(0) << "  "
		  <<  mhd->grid.x.getLength(1) << "  "
		  <<  mhd->grid.x.getLength(2) << "  "
		  <<  "8";
  appendMeta("file_contents", fileContents_ss.str() );

  stringstream dipoleMoment_ss; 
  /* Note: 
   *    The old Fortran LFM added some text to "dipole_moment"
   *    with (8 decimal digits):
   *          "Geoqmu (cgs) =    [geoqmu]" 
   *    appended with two characters: "char(12)//char(0)".
   *    char(12) is form feed '\f'; char(0) is NULL terminator '\0'.  
   *
   *    The Fortran FORMAT defined for [geoqmu] is
   *
   *        format('Geoqmu (cgs) = ',1pe16.7)
   *
   *    '1p' is a scale factor to omit leading zeros.
   *    '16' is the total width of the right-hand-side float
   *    '.7' will show the first 7 digits
   *
   *    Find more on FORMAT statments via:
   *    http://www.math.utah.edu/software/c-with-fortran.html
   *
   *    Let's try and use stringstrema to set the proper format for
   *    dipole moment.  Hopefully this will be backwards compatible
   *    with the LFM's fortran I/O routines in "hdftake-para".  Note
   *    that the LFM behaves unpredictably without this extra garbge
   *    on the "dipole_moment" string attribute.
   */
  dipoleMoment_ss << std::scientific << std::setw(16) << std::setprecision(7)
		  << "Geoqmu (cgs) =    " << constants::GEOQMU << '\f' << '\0';
  appendMeta("dipole_moment", dipoleMoment_ss.str() );

  string writtenBy = string(__FILE__) + " " + string(__FUNCTION__) + string(": HDF4");
  appendMeta("written_by", writtenBy );
}
