#include "MhdGrid.h"

MhdGrid::MhdGrid(const int &n_i, const int &n_j, const int &n_k)
{
  ni=n_i;
  nj=n_j;
  nk=n_k;
  nip1 = ni+1;
  njp1 = nj+1;
  nkp1 = nk+1;
  
  x.redim(nip1,njp1,nkp1);
  y.redim(nip1,njp1,nkp1);
  z.redim(nip1,njp1,nkp1);

  setGrid();
}

void MhdGrid::setGrid()
{
  floatArray phi(nkp1);
  floatArray x2(nip1, njp1);
  floatArray y2(nip1, njp1);

  readGrid(phi, x2, y2);
  computeXYZgrid(phi, x2, y2);
}

void MhdGrid::readGrid(floatArray &phi, floatArray &x2, floatArray &y2)
{
  ifstream gridFile;
  string gridFileName = getGridFilename();
  gridFile.open(gridFileName.c_str(), ios::in);  
  if (!gridFile){
    // Not at $LTRROOT?  Let's try searching relative to "./grids/GRIDOUT-nixnjxnk"
    // FIXME:  Replace this with try{...} catch{...} blocks!
    gridFile.clear();
    int nChar = gridFileName.find("grids/GRIDOUT-");
    gridFileName.replace(0,nChar,"./");

    gridFile.open(gridFileName.c_str(), ios::in);  

    if (!gridFile){
      cerr << "Error opening MHD grid file \"" << gridFileName << "\"" << endl;
      exit(1);
    }
  }
  
  // mjin & mjwall are currently unused by reader. What are these for?
  int mjin, mjwall;
  gridFile >> nip1 >> njp1 >> mjin >> mjwall >> nkp1;
  assert(nip1 == ni+1);
  assert(njp1 == nj+1);
  assert(nkp1 == nk+1);

  for (int k=0; k < nkp1; k++){
    gridFile >> phi(k);
  }

  for (int j=0; j < njp1; j++){
    for (int i=0; i < nip1; i++){
      gridFile >> x2(i,j);
      x2(i,j) *= constants::REARTH;
    }
  }

  for (int j=0; j < njp1; j++){
    for (int i=0; i < nip1; i++){
      gridFile >> y2(i,j);
      y2(i,j) *= constants::REARTH;     
    }
  }

  // Fix pesky back axis
  static const float small_axis = 1.0e-6;
  for (int i=0; i < nip1; i++){
    y2(i,0) = small_axis;
    y2(i,nj) = small_axis;
  }

  gridFile.close();
}


string MhdGrid::getGridFilename()
{
  stringstream ss;
  char *ltrData = getenv("LTRROOT");
  if (ltrData == NULL){
    cerr << "WARNING:  " << __FILE__ << " (L" << __LINE__ << ") " << __FUNCTION__ << "(...) "
	 << "${LTRROOT} environment variable is undefined.  Guessing directory containing GRIDOUT files."
	 << endl;
  }

  ss << ltrData << "/LFM-para/src/startup/grids/GRIDOUT-" << ni << "x" << nj << "x" << nk;

  return ss.str();
}

void MhdGrid::computeXYZgrid(const floatArray &phi, const floatArray &x2, const floatArray &y2)
{
  static const float deg2rad = constants::PI/180.0;

  for (int k = 0; k < nkp1; k++) {
    float cosphi = cos(deg2rad*phi(k));
    float sinphi = sin(deg2rad*phi(k));
    for (int j = 0; j < njp1; j++) {
      for (int i = 0; i < nip1; i++) {
	x(i,j,k) = x2(i,j);

	if ( fabs(y2(i,j)) < 1.0e-5*fabs(x2(i,j)) ){
	  y(i,j,k) = 1.0e-9*fabs(x2(i,j))*cosphi;
	  z(i,j,k) = 1.0e-9*fabs(x2(i,j))*sinphi;
	}
	else{
	  y(i,j,k) = y2(i,j)*cosphi;
	  z(i,j,k) = y2(i,j)*sinphi;
	}
      }
    }
  }
}
