#include "Model.h"

Model::Model(const int &ioLibraryType)
{
  switch(ioLibraryType){
    
  case IO::HDF4:
    io = new Hdf4;
    
    break;
  //case IO::HDF5:
  //  io = new Hdf5;
  //  break;
    
  default: 
    cerr << "ERROR:  Undefined I/O type (" << ioLibraryType << ")." << endl;    
  }
}

Model::~Model()
{
  if (io != NULL){   
    delete io;
    io = NULL;    
  } 
}


void Model::read(const string &filename, DateTime &ut)
{
  io->open(filename, IO::READ);
  readVariables(ut);  // must be defined in Model subclass
  io->close();  
}

void Model::write(const string &filename, const DateTime &ut)
{
  io->open(filename, IO::WRITE);
  writeVariables(ut);  // must be defined in Model subclass
  io->close();  
}


void Model::appendMeta(const string &name, const double &value)
{
  metaDoubles.insert( pair<string, double>(name, value) );
}

void Model::appendMeta(const string &name, const float &value)
{
  metaFloats.insert( pair<string, float>(name, value) );
}

void Model::appendMeta(const string &name, const int &value)
{
  metaInts.insert( pair<string, int>(name, value) );
}

void Model::appendMeta(const string &name, const string &value)
{
  metaStrings.insert( pair<string, string>(name, value) );
}

double Model::getMetaDouble(const string &name)
{
  return metaDoubles[name];
}

float Model::getMetaFloat(const string &name)
{
  return metaFloats[name];
}

int Model::getMetaInt(const string &name)
{
  return metaInts[name];
}

string Model::getMetaString(const string &name)
{
  return metaStrings[name];
}
