#ifndef __MODEL_H__
#define __MODEL_H__


#include "DateTime.h"
#include "Io.h"
#include "Hdf4.h"

#include <map>
#include <string>

/**
 * Abstract base class that defines an interface for LTR models that
 * need to do I/O.  You might recognize Model-Io interaction as an
 * implementation of the Strategy pattern.  See the "Gang of Four"
 * book for more information.
 */
class Model
{
public:
  Model(const int &ioLibraryType);
  ~Model();

  void read(const string &filename, DateTime &ut);
  void write(const string &filename, const DateTime &ut);

  void appendMeta(const string &name, const double &value);
  void appendMeta(const string &name, const float  &value);
  void appendMeta(const string &name, const int    &value);
  void appendMeta(const string &name, const string &value);

  double getMetaDouble(const string &name);
  float  getMetaFloat(const string &name);
  int    getMetaInt(const string &name);
  string getMetaString(const string &name);

protected:
  
  virtual void readVariables(DateTime &ut) = 0;
  virtual void writeVariables(const DateTime &ut) = 0;

  Io *io;

  map<string, double> metaDoubles;
  map<string, float> metaFloats;
  map<string, int> metaInts;
  map<string, string> metaStrings;
};

#endif
