#include "constants.h"
#include <sstream>
#include <iostream>
#include "Params.h"

#include "MhdVars.h"
#include "IonVars.h"

#include "LfmMhd.h"
#include "LfmIon.h"
#include "Io.h"

using namespace std;

std::string getFilename(const Params &params, const string &modelString)
{
  stringstream filename;

  filename << params.basename << "_" << modelString << "_";
  if (params.isUtio){
    filename << params.ymdhms.getDateTimeString();
  }
  else {
    filename << setw(7) << setfill('0') << params.step0;
  }
  filename << ".hdf";

  return filename.str();
}

int main(int argc, char **argv)
{
  Params params;
  params.input();

  MhdVars mhd(params.ni, params.nj, params.nk); 
  IonVars ion(params.nj, params.nk);

  Model *lfmMhd = new LfmMhd(IO::HDF4, &mhd);
  lfmMhd->appendMeta("tzero", params.tzero);
  lfmMhd->appendMeta("run_description", params.runIdentifier);
  Model *lfmIon = new LfmIon(IO::HDF4, &ion);
  lfmIon->appendMeta("run_description", params.runIdentifier);

  lfmMhd->write(getFilename(params, "mhd"), params.ymdhms);
  lfmIon->write(getFilename(params, "ion"), params.ymdhms);

  delete lfmMhd;
  delete lfmIon;

  return 0;
}
