C NAME:
C   get_utio_namelist
C PURPOSE:
C   Initializes the UTIO common block using a fortran name list
C   and return each member of the common block
C CATEGORY
C  SPECIAL
C CALLING SEQUENCE
C   CALL get_utio_namelist(
C       $  oeventYr,oeventMon,oeventDay,oeventHr,oeventMin,oeventSec,
C       $  ostartYr,ostartMon,ostartDay,ostartHr,ostartMin,ostartSec,
C       $  ostopYr,ostopMon,ostopDay,ostopHr,ostopMin,ostopSec,
C       $  ordumpInt,orcorFac,ocaBase)
C       )
C
C INPUTS
C   None
C OUTPUT
C   Full popluated utioinfo common block including determination of Modified 
C   Julian dates for all input times.
C
C   Event date & time YYYY-MM-DD HH:MM:SS
C      oeventYr,oeventMon,oeventDay,oeventHr,oeventMin,oeventSec
C
C   Start date & time YYYY-MM-DD HH:MM:SS
C      ostartYr,ostartMon,ostartDay,ostartHr,ostartMin,ostartSec
C
C   Stop date & time YYYY-MM-DD HH:MM:SS
C      ostopYr,ostopMon,ostopDay,ostopHr,ostopMin,ostopSec
C
C
      SUBROUTINE get_utio_namelist(
     $  oeventYr,oeventMon,oeventDay,oeventHr,oeventMin,oeventSec,
     $  ostartYr,ostartMon,ostartDay,ostartHr,ostartMin,ostartSec,
     $  ostopYr,ostopMon,ostopDay,ostopHr,ostopMin,ostopSec,
     $  ordumpInt,orcorFac,ocaBase)

      INTEGER oeventYr,oeventMon,oeventDay,oeventHr,oeventMin,oeventSec
      INTEGER ostartYr,ostartMon,ostartDay,ostartHr,ostartMin,ostartSec
      INTEGER ostopYr,ostopMon,ostopDay,ostopHr,ostopMin,ostopSec
      REAL*8 ordumpInt, orcorFac
      CHARACTER*256 ocaBase

#     include "utio.inc"

      NAMELIST /UTIO/
     $       ieventyr,ieventmon,ieventDay,ieventhr,ieventmin,ieventsec,
     $       istartyr,istartmon,istartDay,istarthr,istartmin,istartsec,
     $       istopyr,istopmon,istopDay,istophr,istopmin,istopsec,
     $       rdumpint,rcorfac,cabase

      character cmon*3 
      data cmon /'XXX'/

#ifdef DEBUG_MODE_ON
      write(*,*) "DEBUG:  in utio_namelist.F::get_utio_namelist(...)"
#endif
c
c     60*60*24 = 86400.0 seconds in a day
c
      rsid = 86400.0
      OPEN(UNIT=81,FILE='INPUT-UTIO',STATUS='OLD')
#ifndef CRAY
      READ(81,UTIO)
      WRITE(9,UTIO)
#else
      READ (81,NML=UTIO)
      WRITE(9,NML=UTIO)
#endif
      CLOSE(81)

      oeventYr  = ieventyr
      oeventMon = ieventmon
      oeventDay = ieventDay
      oeventHr  = ieventhr
      oeventMin = ieventmin
      oeventSec = ieventsec

      ostartYr  = istartyr
      ostartMon = istartmon
      ostartDay = istartDay
      ostartHr  = istarthr
      ostartMin = istartmin
      ostartSec = istartsec

      ostopYr  = istopyr
      ostopMon = istopmon
      ostopDay = istopDay
      ostopHr  = istophr
      ostopMin = istopmin
      ostopSec = ostopsec

      ordumpInt = rdumpint
      orcorFac = rcorfac
      ocaBase = cabase

      RETURN
      END
