#ifndef CPP_STRUCT_TEST_H_
#define CPP_STRUCT_TEST_H_

#include <cassert>
#include <iostream>

#include "site-stuff.h"
#include "run-time.h"
#include "ionconst.h"
#include "swind.h"

/// Run-time common block defined in run-time.inc
extern RTime RTIME;
extern IONCONST IONCONST;
extern WIND WIND;

#ifdef LC_NOUNSC
#define SETCOMMONBLOCK setcommonblock
#define GETCOMMONBLOCK getcommonblock
#define GETLOGICAL getlogical
#define GETTABLE gettable
#define GETSM getsm
#define GETSWFROMFILE getswfromfile
#elif LC_UNSC
#define SETCOMMONBLOCK setcommonblock_
#define GETCOMMONBLOCK getcommonblock_
#define GETLOGICAL getlogical_
#define GETTABLE gettable_
#define GETSM getsm_
#define GETSWFROMFILE getswfromfile_
#else
error "Must define LC_NOUNSC or LC_UNSC for your environment."
#endif

extern "C" { 
  void SETCOMMONBLOCK( void ); 
  void GETCOMMONBLOCK( int *, int *, int * ); 
  void GETLOGICAL(bool *); 
  bool GETTABLE(void); 
  bool GETSM(void); 
  bool GETSWFROMFILE(void); 
}

class CppStructTest : public CxxTest::TestSuite
{
public:
  void testSetCommonBlockData( void )
  {    
    NSTART = 0;
    NDUMP = 0;
    NSTOP = 0;
    TS_ASSERT_EQUALS(NSTART,0);
    TS_ASSERT_EQUALS(NDUMP,0);
    TS_ASSERT_EQUALS(NSTOP,0);
    SETCOMMONBLOCK();
    TS_ASSERT_EQUALS(NSTART,0);
    TS_ASSERT_EQUALS(NDUMP,5);
    TS_ASSERT_EQUALS(NSTOP,10);
  }
  void testSetStructData( void )
  {
    NSTART = 0;
    NDUMP = 0;
    NSTOP = 0;
    TS_ASSERT_EQUALS(NSTART, 0);
    TS_ASSERT_EQUALS(NDUMP, 0);
    TS_ASSERT_EQUALS(NSTOP, 0);

    NSTART = 100;
    NDUMP = 200;
    NSTOP = 400;
    
    TS_ASSERT_EQUALS(NSTART, 100);
    TS_ASSERT_EQUALS(NDUMP, 200);
    TS_ASSERT_EQUALS(NSTOP, 400);

    int i,j,k;
    GETCOMMONBLOCK(&i, &j, &k);

    TS_ASSERT_EQUALS(i, 100);
    TS_ASSERT_EQUALS(j, 200);
    TS_ASSERT_EQUALS(k, 400);    
  }

  /* IONMODEL boolean in common block was set properly via INPUT1.xml,
   * but the Fortran data structure was getting confused on IBM
   * compilers.  Changed code so IONMODEL = logical*1 (in FORTRAN) and
   * added this test. Seems to work.
   * 
   * IONMODEL defined in ionconst.h/ionconst.inc
   */
  void testBooleanToLogical( void )
  {
    bool logical = false;

    IONMODEL = true;
    GETLOGICAL(&logical);
    TS_ASSERT_EQUALS(logical, IONMODEL);

    IONMODEL = false;
    GETLOGICAL(&logical);
    TS_ASSERT_EQUALS(logical, IONMODEL);       

    IONMODEL = true;
    GETLOGICAL(&logical);
    TS_ASSERT_EQUALS(logical, IONMODEL);
  }

  /* Test "table" logical from swind.h/boundx.inc */
  void testTablelogical( void )
  {
    // "table" defined in swind.h & boundx.inc
    bool logical = false;
    TABLE = true;
    logical = GETTABLE();
    TS_ASSERT_EQUALS(logical, TABLE);

    TABLE = false;
    logical = GETTABLE();
    TS_ASSERT_EQUALS(logical, TABLE);       

    TABLE = true;
    logical = GETTABLE();
    TS_ASSERT_EQUALS(logical, TABLE);
  }


  /* Test "SM" logical from swind.h/boundx.inc */
  void testSMlogical( void )
  {
    // "SM" defined in swind.h & boundx.inc
    bool logical = false;
    SM = true;
    logical = GETSM();
    TS_ASSERT_EQUALS(logical, SM);

    SM = false;
    logical = GETSM();
    TS_ASSERT_EQUALS(logical, SM);       

    SM = true;
    logical = GETSM();
    TS_ASSERT_EQUALS(logical, SM);
  }


  /* Test "SWFromFile" logical from swind.h/boundx.inc */
  void testSWFromFilelogical( void )
  {
    // "SWFROMFILE" defined in swind.h & boundx.inc
    bool logical = false;
    SWFROMFILE = true;
    logical = GETSWFROMFILE();
    TS_ASSERT_EQUALS(logical, SWFROMFILE);

    SWFROMFILE = false;
    logical = GETSWFROMFILE();
    TS_ASSERT_EQUALS(logical, SWFROMFILE);       

    SWFROMFILE = true;
    logical = GETSWFROMFILE();
    TS_ASSERT_EQUALS(logical, SWFROMFILE);
  }
};
#endif /*CPP_STRUCT_TEST_H_*/
