#ifndef XMLINPUT_TEST_H_
#define XMLINPUT_TEST_H_

#include "site-stuff.h"
/* "SWIND" namelist */
#include "swind.h"
extern WIND WIND;
/* "CONST" namelist */
#include "help.h"
extern HELPER HELP;
/* "GO" namelist */
#include "info.h"
extern INFO INFO;
/* "ION" namelist */
#include "ionconst.h"
extern IONCONST IONCONST;
/* "RINGS" namelist */
#include "meter.h"
extern METER METER;
/* "GO" namelist */
#include "run-time.h"
extern RTime RTIME;

#include <cassert>
#include <iostream>
#include "DateTime.h"

#include <cxxtest/TestSuite.h>

bool xmlInputReader(const char * in_xmlFilename, const int in_ni_bin_d, const int in_nj_bin_d,
		    char *out_HDFBASE_IN, char *out_HDFBASE_OUT, 
		    int &out_lion,
		    float &out_time_dump_interval,
		    DateTime &out_startDateTime,
		    DateTime &out_stopDateTime,
		    float &out_ion_exchange_frequency,
		    bool &out_use_intercomm);

class XmlInputTest : public CxxTest::TestSuite
{
 public:
  char HDFBASE_IN[256];
  char HDFBASE_OUT[256];
  char RUN_NAME[256];
  
  int ion;
  
  float out_time_dump_interval;
  DateTime out_startDateTime;
  DateTime out_stopDateTime;
  float out_ion_exchange_frequency;
  bool out_use_intercomm;
  
  void setUp(){
    xmlInputReader("INPUT1-uber-ts.xml", 10, 10, 
		   HDFBASE_IN, HDFBASE_OUT, ion,
		   out_time_dump_interval,
		   out_startDateTime,
		   out_stopDateTime,
		   out_ion_exchange_frequency,
		   out_use_intercomm);
  }

  void test_ReadSTEPS( void )
  {
    TS_ASSERT_EQUALS(NSTART, 0);
    TS_ASSERT_EQUALS(NSTOP, 20000);
    TS_ASSERT_EQUALS(NDUMP, 250);
  }

  void test_ReadUTIO( void )
  {
    TS_ASSERT_DELTA(out_startDateTime.getMJD(), 0.0, 1e-6);
    TS_ASSERT_DELTA(out_startDateTime.getMJD(), 0.0, 1e-6);
  }
  
  void test_ReadGO( void )
  {
    TS_ASSERT_EQUALS(ORDER, 12);
    TS_ASSERT_EQUALS(strcmp(HDFBASE_IN, "TorBaseRun"), 0);
    TS_ASSERT_EQUALS(strcmp(HDFBASE_OUT, "TorBaseRun"), 0);
    TS_ASSERT_EQUALS(strcmp(NEW_RUN_NAME, "TorBaseRun"), 0);
    TS_ASSERT_EQUALS(out_use_intercomm, true);
  }
  void test_ReadRINGS( void )
  {   
    TS_ASSERT_EQUALS(NRINGS, 2);
    /* verify NAVER */
    TS_ASSERT_EQUALS(NI_BIN, 6);
    TS_ASSERT_EQUALS(NJ_BIN, 8);
    TS_ASSERT_DELTA(AXFIX, 2.5, 1e-6);
  }
  void test_ReadRINGS_NAVER_array(void)
  {
    TS_ASSERT_EQUALS(NAVER[0], 4);
    TS_ASSERT_EQUALS(NAVER[1], 2);
    for (int i=2; i < 24; i++)
      TS_ASSERT_EQUALS(NAVER[i], 1);
    for (int i=25; i < 500; i++)
      TS_ASSERT_EQUALS(NAVER[i], 0);
  }
  void test_ReadCONSTANTS( void )
  {
    TS_ASSERT_DELTA(CA, 1.1e8, 1e-6);
    TS_ASSERT_DELTA(GAMMA, 6.123, 1e-6);
    TS_ASSERT_DELTA(CRNT, 0.725, 1e-6);
    TS_ASSERT_DELTA(PDMB, 18.1415, 1e-6);
    TS_ASSERT_DELTA(EVMAX, 2.0e6, 1e-6);
    TS_ASSERT_DELTA(BETMAX, 0.09, 1e-6);
    TS_ASSERT_DELTA(BDIF, 3.5, 1e-6);
  }
  void test_ReadSWIND( void )
  {
    TS_ASSERT_EQUALS(SWFROMFILE, false);
    TS_ASSERT_DELTA(TZERO, 15.0, 1e-6);
    TS_ASSERT_DELTA(BX_ZERO, 2.0, 1e-6);
    TS_ASSERT_DELTA(BY_COEF, 2.0, 1e-6);
    TS_ASSERT_DELTA(BZ_COEF, 2.0, 1e-6);
  }
  void test_ReadION( void )
  {
    TS_ASSERT_DELTA(S107, 100.0, 1e-6);
    TS_ASSERT_DELTA(PED0, 2.0, 1e-6);
    TS_ASSERT_DELTA(PEDMIN, 5.0, 1e-6);
    TS_ASSERT_DELTA(HALLMIN, 4.0, 1e-6);
    TS_ASSERT_DELTA(SIGMA_RATIO, 4, 1e-6);
    TS_ASSERT_DELTA(P_HARDY, 2.4, 1e-6);
    TS_ASSERT_DELTA(C_HARDY, 4.8, 1e-6);
    TS_ASSERT_DELTA(H_HARDY, 1.66, 1e-6);
    TS_ASSERT_DELTA(ITMAX, 40, 1e-6);
    TS_ASSERT_DELTA(ERRMAX, 4.0e-2, 1e-6);
    TS_ASSERT_EQUALS(IONMODEL, false);
    TS_ASSERT_EQUALS(NLOOP, 1);
    TS_ASSERT_EQUALS(LOOPTRIES, 4);    
    TS_ASSERT_DELTA(out_ion_exchange_frequency, 5.0, 1e-6);
  }
};

#endif /*XMLINPUT_TEST_H_*/
