#ifndef XMLINPUT_TEST_H_
#define XMLINPUT_TEST_H_

#include "site-stuff.h"
/* "SWIND" namelist */
#include "swind.h"
extern WIND WIND;
/* "CONST" namelist */
#include "help.h"
extern HELPER HELP;
/* "GO" namelist */
#include "info.h"
extern INFO INFO;
/* "ION" namelist */
#include "ionconst.h"
extern IONCONST IONCONST;
/* "RINGS" namelist */
#include "meter.h"
extern METER METER;
/* "GO" namelist */
#include "run-time.h"
extern RTime RTIME;

#include <cassert>
#include <iostream>
#include "DateTime.h"

#include <cxxtest/TestSuite.h>

bool xmlInputReader(const char * in_xmlFilename, const int in_ni_bin_d, const int in_nj_bin_d,
		    char *out_HDFBASE_IN, char *out_HDFBASE_OUT, 
		    int &out_lion,
		    float &out_time_dump_interval,
		    DateTime &out_startDateTime,
		    DateTime &out_stopDateTime,
		    float &out_ion_exchange_frequency,
		    bool &out_use_intercomm);

class XmlInputTest : public CxxTest::TestSuite
{
 public:
  char HDFBASE_IN[256];
  char HDFBASE_OUT[256];
  char RUN_NAME[256];
  
  int ion;
  
  float out_time_dump_interval;
  DateTime out_startDateTime;
  DateTime out_stopDateTime;
  float out_ion_exchange_frequency;
  bool out_use_intercomm;
  
  void setUp(){
    xmlInputReader("INPUT1-ts.xml", 10, 10, 
		   HDFBASE_IN, HDFBASE_OUT, ion,
		   out_time_dump_interval,
		   out_startDateTime,
		   out_stopDateTime,
		   out_ion_exchange_frequency,
		   out_use_intercomm);
  }

  void test_ReadSTEPS( void )
  {
    TS_ASSERT_EQUALS(NSTART, 0);
    TS_ASSERT_EQUALS(NSTOP, 20000);
    TS_ASSERT_EQUALS(NDUMP, 250);
  }

  void test_ReadUTIO( void )
  {
    TS_ASSERT_DELTA(out_startDateTime.getMJD(), 0.0, 1e-6);
    TS_ASSERT_DELTA(out_startDateTime.getMJD(), 0.0, 1e-6);
  }
  
  void test_ReadGO( void )
  {
    TS_ASSERT_EQUALS(ORDER, 8);
    TS_ASSERT_EQUALS(strcmp(HDFBASE_IN, "ChangeName"), 0);
    TS_ASSERT_EQUALS(strcmp(HDFBASE_OUT, "ChangeName"), 0);
    TS_ASSERT_EQUALS(out_use_intercomm, true);
  }
  void test_DefaultReadRINGS( void )
  {   
    TS_ASSERT_EQUALS(NRINGS, 2);
    /* verify NAVER */
    TS_ASSERT_EQUALS(NI_BIN, 10);
    TS_ASSERT_EQUALS(NJ_BIN, 10);
    TS_ASSERT_DELTA(AXFIX, 0.5, 1e-6);
  }
  void test_DefaultReadRINGS_NAVER_array(void)
  {
    TS_ASSERT_EQUALS(NAVER[0], 4);
    TS_ASSERT_EQUALS(NAVER[1], 2);
    for (int i=2; i < 24; i++)
      TS_ASSERT_EQUALS(NAVER[i], 1);
    for (int i=25; i < 500; i++)
      TS_ASSERT_EQUALS(NAVER[i], 0);
  }
  void test_DefaultReadCONSTANTS( void )
  {
    TS_ASSERT_DELTA(CA, 1.1e8, 1e-6);
    TS_ASSERT_DELTA(GAMMA, 1.66666666, 1e-6);
    TS_ASSERT_DELTA(CRNT, 0.3, 1e-6);
    TS_ASSERT_DELTA(PDMB, 3.0, 1e-6);
    TS_ASSERT_DELTA(EVMAX, 1.e4, 1e-6);
    TS_ASSERT_DELTA(BETMAX, 0.9, 1e-6);
    TS_ASSERT_DELTA(BDIF, 0.0, 1e-6);
  }
  void test_DefaultReadSWIND( void )
  {
    TS_ASSERT_EQUALS(SWFROMFILE, false);
    TS_ASSERT_DELTA(TZERO, 3000.0, 1e-6);
    TS_ASSERT_DELTA(BX_ZERO, 0.0, 1e-6);
    TS_ASSERT_DELTA(BY_COEF, 0.0, 1e-6);
    TS_ASSERT_DELTA(BZ_COEF, 0.0, 1e-6);
  }
  void test_DefaultReadION( void )
  {
    TS_ASSERT_DELTA(S107, 100.0, 1e-6);
    TS_ASSERT_DELTA(PED0, 5.0, 1e-6);
    TS_ASSERT_DELTA(PEDMIN, 0.5, 1e-6);
    TS_ASSERT_DELTA(HALLMIN, 0.0, 1e-6);
    TS_ASSERT_DELTA(SIGMA_RATIO, 1.2, 1e-6);
    TS_ASSERT_DELTA(P_HARDY, 1.19, 1e-6);
    TS_ASSERT_DELTA(C_HARDY, 2.4, 1e-6);
    TS_ASSERT_DELTA(H_HARDY, 1.33, 1e-6);
    TS_ASSERT_DELTA(ITMAX, 400, 1e-6);
    TS_ASSERT_DELTA(ERRMAX, 2.0e-3, 1e-6);
    TS_ASSERT_EQUALS(IONMODEL, true);
    TS_ASSERT_EQUALS(NLOOP, 10);
    TS_ASSERT_EQUALS(LOOPTRIES, 5);    
    TS_ASSERT_DELTA(out_ion_exchange_frequency, 5.0, 1e-6);
  }
};

#endif /*XMLINPUT_TEST_H_*/
