#include "IM_Grids.h"

using namespace std;

/********************************************************************//**
 * Constructor. 
 ************************************************************************/ 
IM_Grids::IM_Grids()
{
  cerr << __FILE__ <<  " (Line " << __LINE__ << "):  " << __FUNCTION__ << "(...) "
       << "Warning:  Grid class must define the LATITUDE array to extend "
       << "beyond the RCM grid (typically between 0 & 60 degrees).." << endl;
}

/********************************************************************//**
 * Create the 2d Inner Magnetosphere grid from the latitude &
 * longitude arrays.  This 2d grid is used for interpolation within
 * the IM_Interface derived class.
 *
 * We convert the latitude & longitude arrays from spherical to
 * cartesian coordinates.
 ************************************************************************/ 
void IM_Grids::setupIMgrid_2d()
{
  // Overture's interpolatePoints function requires that we unfold the
  // array into two dimensions:
  IMgrid_2d.redim(nLat*nLon, 2);

  // Set grid positions we need to interpolate to.
  for (int j=0; j < nLon; j++){
    for (int i=0; i < nLat; i++){
      // Convert from spherical (colatitude & degrees around the pole) to cartesian coordinates
      IMgrid_2d(i+j*nLat, 0) = sin(colatitudes(i))*cos(longitudes(j));
      IMgrid_2d(i+j*nLat, 1) = sin(colatitudes(i))*sin(longitudes(j));
    }
  }
}
