#ifndef ITM_FE_INTERFACE_H
#define ITM_FE_INTERFACE_H

/**
 * \file ITM_FE_Interface.h
 * \brief ITM_FE_Interface class declaration
 * \class ITM_FE_Interface
 * 
 * \brief Contains functions for communications with the ITM
 *
 * \author Slava Merkin (vgm at bu.edu)
 * \since 06-2007
 *
 * This class contains functions and variables that establish
 * communication with the ITM model.
*/

#include "ITM_Interface.h"
#include "InterComm.h"

// Set the precision to the LEFT of the decimal point
// This is used for the TIEGCM adhoc interface
// This macro Shamelessly taken from
// https://secure.codeproject.com/KB/stl/ostringstream.aspx?display=Print
#define leftprec(x,y) (x - (((int)floor(x) - (((int)floor(x))% ((int)(pow(10,y)))))))

class ITM_FE_Interface : public ITM_Interface {

 public:
  /**\brief Class constructor*/
  ITM_FE_Interface(Params *, IC_XJD *);

  /**\brief Class constructor*/
  virtual ~ITM_FE_Interface();

  /**\brief Import data to ITM*/
  void Import(realCompositeGridFunction &);

  /**\brief Export data from ITM*/
  void Export(const intArray &, realCompositeGridFunction &);

 private:

  /** \name File Exchange members. */
  //@{
  /**\brief Returns true if the ITM is busy */
  bool isITMOld()  { return ( strcmp( "0", readLockFile(ITMLockFile).c_str() ) == 0 ); }
  /**\brief Returns true if the ITM is ready and waiting for MIX */
  bool isITMNew() { return ( strcmp( "1", readLockFile(ITMLockFile).c_str() ) == 0 ); }

  /**\brief Returns true if the MIX is busy */
  bool isMIXOld()  { return ( strcmp( "0", readLockFile(MIXLockFile).c_str() ) == 0 ); }
  /**\brief Returns true if the MIX is ready and waiting for MIX */
  bool isMIXNew() { return ( strcmp( "1", readLockFile(MIXLockFile).c_str() ) == 0 ); }
  
  /**\brief Set the ITM lock file to "0" (aka busy)  */
  bool setITMOld(void)   { return writeLockFile(ITMLockFile, "0"); }
  /**\brief Set the ITM lock file to "1" (aka ready) */
  bool setITMNew(void)  { return writeLockFile(ITMLockFile, "1"); }

  /**\brief Set the MIX lock file to "0" (aka busy)  */
  bool setMIXOld(void)   { return writeLockFile(MIXLockFile, "0"); }
  /**\brief Set the MIX lock file to "1" (aka ready) */
  bool setMIXNew(void)  { return writeLockFile(MIXLockFile, "1"); }

  /**\brief  Write #status to the MIX lock file */
  bool writeLockFile(const string &filename, const string &status);

  /**\brief  Read contents of lock file */
  string readLockFile(const string &filename);

  /**\brief filename of MIX lock file. */
  string MIXLockFile;
  /**\brief filename of ITM lock file. */
  string ITMLockFile;
  //@}

};

#endif
