#ifndef MHD_IC_INTERFACE_H
#define MHD_IC_INTERFACE_H

/**
 * \file MHD_IC_Interface.h
 * \brief MHD_IC_Interface declaration
 *
 * \class MHD_IC_Interface 
 * \brief The MHD_IC_Interface class utilizes InterComm for receiving
 * and sending data from/to the MHD model.
 *
 * \author Slava Merkin (vgm at bu.edu)
 * \author Peter Schmitt (schmitt@ucar.edu)
 * \since 09-2008
 *
 * InterComm interface for communication with the MHD model.
 *
 * \see MHD_Interface
 */

#include "MHD_Interface.h"

#include "Check_IC_Status.h"

class MHD_IC_Interface : public MHD_Interface {

 public:
  /** \brief The class constructor. */
  MHD_IC_Interface(Params *params, IC_XJD *XJD);

  /** \brief The class destructor. */
  virtual ~MHD_IC_Interface();

  /** \brief The function for import from MHD to ionosphere. */
  void Import(realCompositeGridFunction &mhdFuncs);

  /** \brief Sends potential back to MHD. */
  void Export(realCompositeGridFunction &mhdFunctions1, realCompositeGridFunction &mhdFunctions2);
 
  /** \brief Receives a scalar array from the MHD code. Intended to
   * communicate scalars such as UT and kill signal */
  realArray getScalars();

 private:

  /** \name Class members for InterComm communication. */
  //@{
  int ic_err;
  IC_Desc* desc1shell; //!< Descriptor for 1-shell (first shell) array transmission.
  IC_Desc* desc2shell; //!< Descriptor for 2-shell (first and second shells) array transmission.
  IC_Region* regions1shell[1]; //!< Region set for 1-shell array transmission.
  IC_Region* regions2shell[1]; //!< Region set for 2-shell array transmission.
  //@}

  void exchangeRION();
};

#endif
