#define BOUNDS_CHECK 
#include "MHDInnerBoundaryInterface.h"
#include "CPL_IC_Interface.h"
#include "CPL_FE_Interface.h"
#include <iostream>
#include <fstream>
#include <string>

using std::ifstream;
using std::ofstream;
using std::cout;
using std::cin;
using std::ios;
using std::cerr;
using std::endl;
using std::string;

void parse_xjd(char *, char *, char *, int &);

int main(int argc, char *argv[]){
  int io_pe = 0;
  int localTasks = 1;
  int remoteTasks = 1;
  char localName[256];
  char *xjdName = argv[1];

  if ( !xjdName ) { 
    cout << "Please specify the xjd-file name. Usage: jpara_wrapper <xjd file name>" << endl;
    exit(1);
  }

  /* Let's figure out the component name and the number of processors
     from the xjd-file */
  parse_xjd(xjdName, "mhd", localName, localTasks);
  cout << "Local component name: \"" << localName << "\"\n"
       << "Local number of processors: " << localTasks << endl;

  Optimization_Manager::Initialize_Virtual_Machine(localName,localTasks,argc,argv);
  int my_pe = Communication_Manager::My_Process_Number; 

  Communication_Manager::Sync();

  Range ioProcessRange(0,0), fullProcessRange(0,localTasks-1);
  Partitioning_Type ioProcess(ioProcessRange), allProcesses(fullProcessRange);
  /* This is just to test P++ partitioning
  allProcesses.partitionAlongAxis(0,TRUE,2);
  allProcesses.partitionAlongAxis(1,TRUE,2);
  allProcesses.partitionAlongAxis(2,FALSE,2);
  floatArray qq(8,8,8);
  qq.partition(allProcesses);
  int iLocalBase = qq.getLocalBase(0);
  int iLocalBound = qq.getLocalBound(0);
  int iGlobalBase = qq.getBase(0);
  int iGlobalBound = qq.getBound(0);

  int jLocalBase = qq.getLocalBase(1);
  int jLocalBound = qq.getLocalBound(1);
  int jGlobalBase = qq.getBase(1);
  int jGlobalBound = qq.getBound(1);

  int kLocalBase = qq.getLocalBase(2);
  int kLocalBound = qq.getLocalBound(2);
  int kGlobalBase = qq.getBase(2);
  int kGlobalBound = qq.getBound(2);

  cout << "("<<iLocalBase << "," << jLocalBase << "," << kLocalBase <<
    ")  (" << iLocalBound << "," << jLocalBound << "," << kLocalBound <<
    ")  (" << iGlobalBase << "," << jGlobalBase << "," << kGlobalBase <<
    ")  (" << iGlobalBound << "," << jGlobalBound << "," << kGlobalBound << ")" <<  endl;
  exit(1);
  */
  /* ------------ READ AND ASSIGN LFM GRID FROM THE GRIDOUT FILE ---------- */
  int nip1, njp1, mjin,mjwall,nkp1;
  cout << "Reading GRIDOUT ...............";
  
  ifstream gridFile;
  for ( int i=0; i<localTasks; i++) {
    if ( i!=my_pe ) { continue; }

    gridFile.open("input/GRIDOUT",ios::in);
    if (!gridFile) {
      cerr << "Could not open the file" <<endl;
      exit(1);
    }

    gridFile >> nip1 >> njp1 >> mjin >> mjwall >> nkp1;
  }
  Communication_Manager::Sync();

  int ni = nip1 - 1;
  int nj = njp1 - 1;
  int nk = nkp1 - 1;
  int nk2p3 = nk/2+3;

  doubleArray phi(nkp1);
  doubleArray x2(nip1,njp1), y2(nip1,njp1);
  doubleArray x(nip1,njp1,nkp1), y(nip1,njp1,nkp1), z(nip1,njp1,nkp1);
  doubleArray bx(ni,nj,nk), by(ni,nj,nk), bz(ni,nj,nk);
  doubleArray rho(2,nj,nk), csound(2,nj,nk);

  phi.partition(ioProcess);
  x2.partition(ioProcess);
  y2.partition(ioProcess);
  x.partition(ioProcess);
  y.partition(ioProcess);
  z.partition(ioProcess);
  rho.partition(ioProcess);
  csound.partition(ioProcess);
  bx.partition(ioProcess);
  by.partition(ioProcess);
  bz.partition(ioProcess);

  Optimization_Manager::setOptimizedScalarIndexing(On);
  if  (my_pe != io_pe)
    {
      gridFile.close();
      cout <<"done"<<endl;
    }
  else 
    {
      for (int k = 0; k < nkp1; k++) {gridFile >> phi(k);}

      for (int j = 0; j < njp1; j++) {
	for (int i = 0; i < nip1; i++) {
	  gridFile >> x2(i,j);
	}
      }

      for (int j = 0; j < njp1; j++) {
	for (int i = 0; i < nip1; i++) {
	  gridFile >> y2(i,j);
	}
      }
      gridFile.close();


      for (int k = 0; k < nkp1; k++) {
	for (int j = 0; j < njp1; j++) {
	  for (int i = 0; i < nip1; i++) {
	    x(i,j,k) = x2(i,j);
	    y(i,j,k) = y2(i,j)*cos(PI/180.*phi(k));
	    z(i,j,k) = y2(i,j)*sin(PI/180.*phi(k));
	  }
	}
      }

      cout <<"done"<<endl;
      /* ------------------ END READ AND ASSIGN GRID --------------------------- */

      /* ---------------------- READ MAGNETIC FIELD DATA ----------------------- */
      cout << "Reading magnetic field ........";
      ifstream dataFile("input/bx_by_bz.dat",ios::in);
      if (!dataFile) {
	cerr << "Could not open the file magnetic field file" <<endl;
	exit(1);
      }


      for (int k = 0; k < nk; k++) {
	for (int j = 0; j < nj; j++) {
	  for (int i = 0; i < ni; i++) {
	    dataFile >> bx(i,j,k) >> by(i,j,k)  >> bz(i,j,k);
	  }
	}
      }
      dataFile.close();

      cout <<"done"<<endl;
      /* ---------------------- END READ MAGNETIC FIELD DATA-------------------- */

      /* ---------------------- READ DENSITY AND SOUND SPEED DATA ----------------------- */
      cout << "Reading density ........";
      ifstream rhoFile("input/density.dat",ios::in);
      ifstream cFile("input/csound.dat",ios::in);

      for (int i = 0; i < 2; i++) {
	for (int k = 0; k < nk; k++) {
	  for (int j = 0; j < nj; j++) {
	    rhoFile >> rho(i,j,k);
	    cFile >> csound(i,j,k);
	  }
	}
      }
      cFile.close();
      rhoFile.close();

      cout <<"done"<<endl;
      /* ---------------------- END READ DENSITY AND SOUND SPEED DATA-------------------- */
    }  
  Optimization_Manager::setOptimizedScalarIndexing(Off);

  doubleArray ej_pot, ek_pot, velocity;

  x.partition(allProcesses);
  y.partition(allProcesses);
  z.partition(allProcesses);
  bx.partition(allProcesses);
  by.partition(allProcesses);
  bz.partition(allProcesses);
  rho.partition(allProcesses);
  csound.partition(allProcesses);
  ej_pot.partition(allProcesses);
  ek_pot.partition(allProcesses);
  velocity.partition(allProcesses);

  x.setBase(1);
  y.setBase(1);
  z.setBase(1);
  bx.setBase(1);
  by.setBase(1);
  bz.setBase(1);
  rho.setBase(1);
  csound.setBase(1);


  std::cerr << "NOTE: in order to run \"jpara_wrapper\" using file exchanges, you must "
	    << "manually edit MIX/src/jpara_wrapper.C, comment out the line containing \n"
	    << "    ionosphere = new CPL_IC_Interface(...)\n"
	    << "and un-comment the line containing\n"
	    << "    ionosphere = new CPL_FE_Interface(...)\n"
	    << "For regular LFM-MIX, this option is set in the INPUT1.xml file.\n";
  
  //MHDInnerBoundaryInterface ionosphere(xjdName, localName, x, y, z, 2, nj, nk);
  MHDInnerBoundaryInterface *ionosphere;

  //
  // InterComm communication:
  int ic_err;
  IC_EndPointSet *epset = new IC_EndPointSet(xjdName, localName, ic_err);
  ionosphere = new CPL_IC_Interface(epset, x, y, z, 2, nj, nk);
  // We commit the InterComm arrays here because this function commits *all* regions created previously.
  // In our case, both CPL_IC_Interface and IM_IC_Interface define arrays that need to be committed.
  epset->commitArrays(ic_err);
  epset->printErrorMessage("     Arrays commited. Status",ic_err);

  // 
  // File Exchange communication (No PVM required, but it's slower!)
  //ionosphere = new CPL_FE_Interface(x, y, z, 2, nj, nk);
 
  doubleArray scalars(CPL_Interface::NUMBER_OF_SCALARS);
  scalars.partition(ioProcess);
  scalars(CPL_Interface::KILL_SIGNAL) = 0.; // Kill signal (111 is to kill. We don't kill :)
  scalars(CPL_Interface::YEAR) = 2001.;
  scalars(CPL_Interface::MONTH) = 3.;
  scalars(CPL_Interface::DAY) = 13.;
  scalars(CPL_Interface::HOUR) = 2.;
  scalars(CPL_Interface::MINUTE) = 12.;
  scalars(CPL_Interface::SECOND) = 11.;
  scalars(CPL_Interface::TILT) = -0.7853259891E-01; // TILT, set to 0.5477 for 16 July 2000)
  scalars(CPL_Interface::LABEL) = 0; // Bogus Time step


  for (int i = 0; i< 1; i++) {
    ionosphere->sendScalars(scalars);
    ionosphere->Export(bx, by, bz, rho, csound);
    ionosphere->Import(ej_pot, ek_pot, velocity);
  }
  // And now kill
  scalars(CPL_Interface::KILL_SIGNAL) = 111.;
  ionosphere->sendScalars(scalars);

  // Write out ej_pot, ek_pot, and velocity for MIX testing purposes
  ej_pot.partition(ioProcess);
  ek_pot.partition(ioProcess);
  velocity.partition(ioProcess);

  Optimization_Manager::setOptimizedScalarIndexing(On);
  if  (my_pe == io_pe) {
    FILE *ejFile, *ekFile, *velocityFile;

    ejFile = fopen("ej.dat","w");
    for (int j = 1; j < nj+1; j++) {
      for (int k = 1; k < nkp1+1; k++) {
	fprintf(ejFile,"%13.8e\t%13.8e\n",ej_pot(1,j,k),ej_pot(2,j,k));
      }
    }
    fclose(ejFile);

    ekFile = fopen("ek.dat","w");
    for (int j = 1; j < njp1+1; j++) {
      for (int k = 1; k < nk+1; k++) {
	fprintf(ekFile,"%13.8e\t%13.8e\n",ek_pot(1,j,k),ek_pot(2,j,k));
      }
    }
    fclose(ekFile);

    velocityFile = fopen("velocity.dat","w");
    for (int j = 1; j < nj+1; j++) {
      for (int k = 1; k < nk+1; k++) {
	fprintf(velocityFile,"%13.8e\t%13.8e\t%13.8e\n",
		velocity(1,j,k,1),velocity(1,j,k,2),velocity(1,j,k,3));
      }
    }
    fclose(velocityFile);
  }
  Optimization_Manager::setOptimizedScalarIndexing(Off);

#ifdef WRITE_DX
  /* ---------------- WRITE ASCII FILE TO BE READ BY OPENDX ------------------ */

  // Get the file to read the grid
  ifstream ionGridFile("iongrid.p3d",ios::in);
  if (!ionGridFile) {
    cerr << "Could not open the file" <<endl;
    exit(1);
  }
  int nz;
  // Read in the grid dimensions and write out on the screen
  ionGridFile >> nj >> nk>> nz;
  doubleArray ionGrid(nj,nk,3);
  
  //  Read in the vertex coordinates from the file
  for (int Rind = 0; Rind < 3; Rind++) {
    for (int Kind = 0; Kind < nk; Kind++) {
      for (int Jind = 0; Jind < nj; Jind++) {
	ionGridFile >> ionGrid(Jind,Kind,Rind);
      }
    }
  }
  ionGridFile.close();

  ofstream jparaFile("jpara.ascii.dx",ios::out);
  for (int Kind = 0; Kind < nk; Kind++) {
    for (int Jind = 0; Jind < nj; Jind++) {
      jparaFile << ionGrid(Jind,Kind,0)<<"\t" << ionGrid(Jind,Kind,1)<<
	"\t" << ionGrid(Jind,Kind,2)<<"\t"<<curnt(Jind+1,Kind+1)<<"\n";
    }
  }
  jparaFile.close();

  /* -------------------------------------------------------------------- */
#endif //WRITE_DX
  
  Optimization_Manager::Exit_Virtual_Machine();
  delete epset;
  delete ionosphere;
  return 0;
}
