      program ITM_demo

      integer,parameter ::
     | nlat   = 36,
     | nlon   = 72,
     | nlatp1 = nlat+1,
     | nlonp1 = nlon+1
      
      real ting_pot_interp(nlonp1,nlat)  ! potential
      real ting_eng_interp(nlonp1,nlat)  ! energy
      real ting_flux_interp(nlonp1,nlat) ! flux

      real mix_ped(nlonp1,nlatp1)   ! Pedersen Conductance
      real mix_hall(nlonp1, nlatp1) ! Hall conductance
      real mix_wfac(nlonp1, nlatp1) ! height-integrated neutral wind currents geographic

ccccc
ccccc Main Loop      
ccccc
      do i=1,1
         call Import(ting_pot_interp, ting_eng_interp, ting_flux_interp)
         ! TIEGCM uses pot, eng, flux to compute ped, hall, wfac
         ! solveITM is just a hack that reads preacomputed data
         ! from "MIX-ITM-inputs/tingout.dat"...
         call solveITM(mix_ped, mix_hall, mix_wfac)
         call Export(mix_ped, mix_hall, mix_wfac)
      enddo

      end program ITM_demo

!-----------------------------------------------------------------------
      subroutine Import(pot,eng,flux)
! ... Local variables ..................................................
      ! ITM Grid 
      integer,parameter ::
     | nlat   = 36,
     | nlon   = 72,
     | nlatp1 = nlat+1,
     | nlonp1 = nlon+1
      real lat(nlat)
      real lon(nlonp1)
      ! Local helper variables:
!      real pi, rad2deg, deg2rad
      integer i, j
      character(len=60) :: lockfile
      character(len=60) :: tieOutFile
      ! Read these varibles from tieOutFile:
      integer nday, nhour, nmin, nsec
      real latInput(nlat)
      real lonInput(nlonp1)
! ... Parameter variables ..............................................
      real pot(nlonp1,nlat)  ! potential
      real eng(nlonp1,nlat)  ! energy
      real flux(nlonp1,nlat) ! flux
! ... Begin ............................................................
      write(*,*) "ITM Import(pot, eng, flux)"

!!!!! Setup lat/lon grid
      do j=1,nlat
         lat(j)=(-87.5+5.0*(j-1))
      enddo
      do i=1,nlonp1
         lon(i)= (-180.0+5.0*(i-1))
      enddo

!!!!! Read the data in      
      lockfile="mhd_status"
      
ccccc
ccccc check to make sure file exists before proceeding
ccccc
      ier=1
      do while (ierr .ne. 0)
       open(57,file=lockfile,status='old',iostat=ierr)
       close(57)
      enddo
ccccc
ccccc The lockfile exists.  Wait for "new" data.
ccccc
      do while(mhd_data.ne.1)
#ifdef AIX
         call sleep_(5)
#else
         call sleep(5)
#endif
         close(57)
         open(57,file=lockfile)
         read(57,331)mhd_data
         write(*,*) "Waiting for 'mhd_status'=1"
         close(57)
      enddo
 331  format(1I2)
ccccc
ccccc open data file 'ting-inputs.txt' and read data
ccccc
      tieOutFile = "MIX-ITM-inputs/ting-inputs.txt"
      open(58,file=tieOutFile)
      read(58,581)nday,nhour,nmin,nsec
 581  format(i3,2x,i2,2x,i2,2x,i2)
      write(*,*) "nday, nhour, nmin, nsec:"
      write(*,581)nday,nhour,nmin,nsec
      do j=1,nlat
         do i=1,nlonp1
            read(58,'5e12.4') latInput(j),lonInput(i),
     |           pot(i,j),eng(i,j),flux(i,j)
            ! Ensure proper longitude
            if (ABS(lonInput(i) - lon(i)) > 1e-5) then
               write(*,500) i,j,lon(i),lat(j),lonInput(i), latInput(j)
            endif
            ! Ensure proper latitude
            if (ABS(latInput(j) - lat(j)) > 1e-5) then
               write(*,500) i,j,lon(i),lat(j),lonInput(i), latInput(j)
            endif
            
         enddo
      enddo
      close(58)
 500  format("ERROR: (lon,lat) = (",i2,",",i2,"):  ",
     |     " Expected (",f,",",f,"), ", 
     |     "got (", f,",", f,")")
ccccc 
ccccc Done Importing.
ccccc
      end subroutine Import
!-----------------------------------------------------------------------
      subroutine Export(ped,hall,wfac)
! ... Local variables ..................................................
      ! ITM Grid 
      integer,parameter ::
     | nlat   = 36,
     | nlon   = 72,
     | nlatp1 = nlat+1,
     | nlonp1 = nlon+1
      real lat(nlat)
      real lon(nlonp1)
      ! Local helper variables:
      integer i, j
      character(len=60) :: lockfile
      character(len=60) :: mixOutFile
      ! Record these varibles to mixOutFile:
      integer nday, nhour, nmin, nsec
! ... Parameter variables ..............................................
      real ped(nlonp1,nlat)   ! Pedersen Conductance
      real hall(nlonp1, nlat) ! Hall conductance
      real wfac(nlonp1, nlat) ! height-integrated neutral wind currents geographic
! ... Begin ............................................................
      write(*,*) "ITM Export(ped, hall, wfac)"

!!!!! Setup lat/lon grid
      do j=1,nlat
         lat(j)=(-87.5+5.0*(j-1))
      enddo
      do i=1,nlonp1
         lon(i)= (-180.0+5.0*(i-1))
      enddo

ccccc
ccccc open data file 'ting-outputs.txt' and write data
ccccc
      mixOutFile="ting-outputs.txt"
      open(56,file=mixOutFile)
      !             doy, hh, mm, ss
      write(56,333) 123, 00, 00, 00
 333  format(i3,2x,i2,2x,i2,2x,i2)
      do j=1,nlat
         do i=1,nlonp1
            write(56,'5e12.4') lat(j), lon(i),
     |           ped(i,j), hall(i,j), wfac(i,j)
         enddo
      enddo
      close(56)
ccccc 
ccccc Done writting the data out. Set ting_status to new.
ccccc
      lockfile="ting_status"
      open(56,file=lockfile)
      write(56,"(A1)")'1'
      close(56)
ccccc 
ccccc Done Exporting.
ccccc
      end subroutine Export
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
      subroutine solveITM(ped,hall,wfac)
! ... Local variables ..................................................
      ! ITM Grid 
      integer,parameter ::
     | nlat   = 36,
     | nlon   = 72,
     | nlatp1 = nlat+1,
     | nlonp1 = nlon+1
      real lat(nlat)
      real lon(nlonp1)
      ! Local helper variables:
      integer i, j
      character(len=60) :: lockfile
      ! Record these varibles to mixOutFile:
      integer nday, nhour, nmin, nsec
! ... Parameter variables ..............................................
      real ped(nlonp1,nlat)   ! Pedersen Conductance
      real hall(nlonp1, nlat) ! Hall conductance
      real wfac(nlonp1, nlat) ! height-integrated neutral wind currents geographic
! ... Begin ............................................................
ccccc
ccccc Since this is a demo, open "MIX-ITM-inputs/tingoutputs.dat" and read ped,hall,wfac
ccccc In the real world, TIEGCM would compute ped,hall,wfac here...
ccccc
      open(56,file="MIX-ITM-inputs/tingout.dat")
      read(56,'i3,2x,i2,2x,i2,2x,i2') nday,nhour,nmin,nsec
      do j=1,nlat
         do i=1,nlonp1
            read(56,'5e12.4') lat(j),lon(i),
     |           ped(i,j),hall(i,j),wfac(i,j)
         enddo
      enddo
      close(56)      
      end subroutine solveITM
!-----------------------------------------------------------------------
