C+
C NAME:
C	DATE_DOY
C PURPOSE:
C	Converts dates (iYr,iMon,iDay) to daynumber iDoy and v.v.
C CATEGORY:
C	Time keeping
C CALLING SEQUENCE:
C	subroutine DATE_DOY(ID,iYr,cMon,iMon,iDay,iDoy)
C INPUTS:
C	ID	INTEGER		ID=0  date (yr,month,day) ----> doy of year
C				ID=1  doy of year ----> date (yr,month,day)
C	iYr	INTEGER		year; the year xxxBC should be entered as -xxx+1
C
C	ID = 0:
C	 cMon    CHARACTER*3	month	(either cMon or iMon must be specified)
C	 iMon    INTEGER	value 1-12   (see PROCEDURE)
C	 iDay    INTEGER	day of month
C
C 	ID = 1:
C	 iDoy    INTEGER	day of year
C OUTPUTS:
C	ID = 0:
C	 iDoy    INTEGER	day of year
C
C	ID = 1:
C	 cMon    CHARACTER*3	month
C	 iMon    INTEGER	1-12
C	 iDay    INTEGER	day of month
C CALLS:
C	uppercase
C RESTRICTIONS:
C >>>	For ID=0 (date --> doy) the input for the month may be given in the form
C	of a char-string cMon or as an INTEGER iMon. cMon takes precedence
C	over iMon: first cMon is checked for valid input; if cMon is not valid
C	iMon is used (if iMon is invalid too, the program is aborted).
C >>>	ADVICE: ALWAYS use cMon for input.
C	Upon output the values of cMon and iMon are always consistent.
C	The returned value for cMon is always in uppercase.
C PROCEDURE:
C	Dates before 5 october 1582 are interpreted as Julian dates; after 15
C	october as Gregorian dates.
C >>>	5 october 1582 (Julian) = 15 october 1582 (Gregorian).
C MODIFICATION HISTORY:
C	1989-1990, Paul Hick (MPAE,UCSD/CASS; pphick@ucsd.edu)
C-
      SUBROUTINE date_doy(id,iyr,cmon,imon,iday,idoy)

      INTEGER	id
      INTEGER	iyr
      CHARACTER	cmon*3
      INTEGER	imon
      INTEGER	iday
      INTEGER	idoy

      CHARACTER*3 months(12)
     &/'JAN','FEB','MAR','APR','MAY','JUN','JUL','AUG','SEP','OCT',
     & 'NOV','DEC'/

      INTEGER idays(12)
     &/ 31,   28,   31,   30,   31,   30,   31,   31,   30,   31,   30,
     &   31  /

      IF (id .NE. 0 .AND. id .NE. 1) STOP 'DATE_DOY: invalid id'

      leap = 2
      IF (iyr/4*4 .EQ. iyr .AND. (iyr .LT. 1582 .OR. iyr/100*100 .NE. iYr
     &	  .OR. iYr/400*400 .EQ. iyr)) leap = 1	!It's a leap year
      idays(2) = 30-leap			!Days in february

      IF (id .EQ. 0) THEN			!Date ---> daynumber
	i = 1
	DO WHILE (i .LE. 12 .AND. cmon .NE. months(i))
	  i = i+1
	END DO
	IF (i .LE. 12) THEN
	  imon = i
	ELSE			!If cmon does not make sense, try imon
	  IF (imon .LT. 1 .OR. imon .GT. 12) 
     $      STOP 'DATE_DOY: invalid month'
	  cmon = months(imon)
	END IF
	IF (iday .LT. 1 .OR. iday .GT. idays(imon))
     $          STOP 'DATE_DOY: invalid iday'
	  i = imon-1				!Number of months already past
	  idoy = i*31-MIN(i/2,3)-MAX(0,(i-7)/2)+iday !Day number
	  IF (i .GE. 2) idoy = idoy-leap	!Correction for February
	  IF (iyr .EQ. 1582 .AND. idoy .GT. 278) THEN !278=15 oct (Greg.)
	    IF (idoy .LT. 288) 
     $        STOP 'DATE_DOY: date in gap between calendars'
	    idoy = idoy-10			!Substract 10 day gap
	  END IF
      ELSE					!Daynumber ---> date
	IF (idoy .LT. 1 .OR. idoy .GT. 367-leap)
     $    STOP 'DATE_DOY: invalid idoy'
	IF (iyr .EQ. 1582) THEN
	  IF (idoy .GT. 355) STOP 'DATE_DOY: invalid idoy for 1582'
	  idays(10) = 21	!Substract 10 day gap for october
	END IF
	iday = idoy
	imon = 1				!Suppose it's in january
	DO WHILE ((iday-1)/idays(imon) .GE. 1)
	  iday = iday-idays(imon)		!Month iMon already past
	  imon = imon+1			!Try next month
	END DO
	cmon = months(imon)
	IF (iyr .EQ. 1582 .AND. imon .EQ. 10 .AND. iday .GE. 5) 
     $    iday = iday+10                      !Add 10 day gap
      END IF					
      RETURN
      END
