!==================================================================      
      SUBROUTINE Torcm (rec, itimei, ierr) 
      USE Rcm_mod_subs, ONLY : isize,jsize, jwrap, kcsize, iesize, &
                               vm, bmin, xmin, ymin, pmin, rmin,v, & 
                               alamc, etac, ikflavc, fudgec,eeta,  &
                               alam, eta, bi, bj, etab, ikflav,    &
                               mpoint, npoint,itrack,ksize,        &
                               imin_j, bndloc, vbnd, boundary,     &
                               colat, aloct, bir, sini,            &
                               Read_array, Write_array,            &
                               LUN, DTR, label,                    &
                               Get_boundary, Write_grid, outp,     &
                               ibnd_type, RTD, rcmdir,             &
                               write_qtcond,                       &
                               iprec,rprec
     USE conversion_module
     USE rice_housekeeping_module

! NOTE: This version fixes the rcm boundary condition at rec=1
!===================================================================
!
! purpose: to setup rcm for a run, converts mhd information to 
!          rcm information
!
! inputs:
!     rec_in   = record to be updated with new values
!               rec == 1: resets the rcm and initializes the files
!                rec > 1: only boundary conditions are updated
!    itimei = time to write to rcm records
!
! outputs:
!       ierr = error flag if there is a problem
!
!===================================================================
!
!  last update:  05.18.90
!                04.13.95
!                      96
!                20.01.00 frt mc version
!                    8.03 frt computes integral average pressure
!                06.01.04 frt cleaned up version
!
!===================================================================

      IMPLICIT NONE
      INTEGER(iprec), INTENT (IN) :: rec, itimei
      INTEGER(iprec), INTENT (IN OUT) :: ierr

      REAL(rprec) :: xp,yp,red
      REAL(rprec) :: eetabnd_max
      REAL(rprec), PARAMETER :: offseti = 0.0  
      REAL(rprec), PARAMETER :: ti0 = 5.0E+8, tiote = 7.8
      REAL(rprec), PARAMETER :: small_eta = 1.0E+4
      REAL(rprec), PARAMETER :: xline = -25.0
      REAL(rprec), PARAMETER :: lmin = 2.10
      REAL(rprec), PARAMETER :: big_vm = -1.0e5      
      
      INTEGER(iprec) :: kin,jmid,ibnd, inew, iold
      INTEGER(iprec) :: jm,jp,ii,itmax
      INTEGER(iprec) :: min0,i,j,k,n,ns
      INTEGER(iprec), PARAMETER :: write_rcm2=1
      INTEGER(iprec), PARAMETER :: n_smooth = 5
      LOGICAL,PARAMETER :: use_ellipse = .true.
      LOGICAL,PARAMETER :: set_boundary_with_mach = .false.
      LOGICAL,PARAMETER :: set_boundary_with_beta = .true.
      REAL(rprec), PARAMETER :: max_beta = 1.0 ! max averaged beta to set the boundary

      ierr = 0
      CALL Begin (rec, ierr)     ! Read in MHD data

      IF (ierr < 0) RETURN


      ! Save current RCM arrays before modifying them.
      ! T=0 or restart is a special case:

      ! If T>0, we already have old array values
      ! T=0 case means we have to defer this to after
      ! magnetic field arrays are processed.
      IF (rec > 1) then  
        bndloc_old = bndloc
        imin_j_old = imin_j
      END IF  
      



      ! Trace B-field lines starting from RCM ionospheric grid
      ! points, compute flux-tube volume of field lines that are
      ! closed, and mark open ones with a mask array OPEN
      ! (1=open, 0-closed). For open field lines, FTV is set to big_vm:

      CALL Calc_ftv (big_vm,ierr)
      IF (ierr < 0) RETURN



      ! Now, set RCM high-latitude grid boundary. Initially,
      ! for each MLT, we find the grid point with highest
      ! latitude and still on closed field lines; the point
      ! next to it (down in latitude) is set to be boundary.
      ! Result of this subsection is to populate arrays BNDLOC
      ! and IMIN_J with values:

      do j=1,jdim
         bndloc(j) = 2  ! if everything else fails, can use this...
         do i=idim,2,-1
            if(open(i,j) == 0)then
               bndloc(j) = i + 1
               exit
            endif
         end do
         IF (L_write_vars_debug) then
            write(*,*)' bndy ',bndloc(j),j,vm(ceiling(bndloc(j)),j)
         END IF
      end do

      ! fits an ellipse to the boundary
      if (use_ellipse)then
               CALL Set_ellipse(idim,jdim,rmin,pmin,vm,big_vm,bndloc,open)
      end if

      IF(set_boundary_with_mach)then
       write(6,*)' boundary before set_bndy_w_mach'
       write(6,*) bndloc
       CALL Set_bndy_w_mach(idim,jdim,rmin,pmin,vm,big_vm,bndloc,open)
       write(6,*)' boundary after set_bndy_w_mach'
       write(6,*)bndloc
      END IF

      IF(set_boundary_with_beta)then
       do j=1,jdim
        do i=ceiling(bndloc(j)),idim-1
          IF(beta_average(i,j) > max_beta)then
            bndloc(j) = i+1
            vm(1:i,j) = big_vm
            open(1:i,j) = 0
          END IF
        end do
       end do
      END IF

      ! smooth boundary location 
      do ns=1,n_smooth
         write(6,*)' smoothing rcm boundary, ns =', ns
         call smooth_boundary_location(idim,jdim,jwrap,bndloc)
         call reset_rcm_vm(idim,jdim,bndloc,big_vm,imin_j,vm,open) ! adjust Imin_j
      end do

      if (n_smooth < 1) call reset_rcm_vm(idim,jdim,bndloc,big_vm,imin_j,vm,open)
    

! reset mapping points
       do j=1,jdim
        do i=1,imin_j(j)-1
          rmin(i,j) = 0.0
          pmin(i,j) = 0.0
        end do
       end do
! find the inner boundary of the lfm in the rcm grid, key on pressure
       do j=1,jdim
         inner_bndy(j) = idim ! default
        do i=imin_j(j),isize
         if(press(i,j) < 1.0e-15)then
           inner_bndy(j) = i+1
           exit
         endif
        end do
       end do
       
       IF (L_write_vars_debug) then
          write(*,*)' location of lfm inner boundary in the rcm is:'
          write(*,*)inner_bndy
       END IF
          
!---->Set new EETA from lfm code pressure. 
!     On open field lines, values of ETA will be zero:

      CALL Gettemp (ierr)
      IF (ierr < 0) RETURN
      
      CALL Read_alam (kcsize, alamc, ikflavc, fudgec, almdel, almmax, almmin, iesize, ierr)
      IF (ierr < 0) RETURN

      CALL Press2eta       ! this populates EETA_NEW array


      ! Here is the second IF promised at the beginning.
      ! It must come after calls to calc_ftv and press2eta.
      ! this is where we must set initial conditions:

      IF (rec == 1) THEN
         bndloc_old = bndloc
         imin_j_old = imin_j
         eeta       = eeta_new  ! this is initial conditions on plasma
      END IF


      ! set a crude plasmasphere in the rcm - off for now
!     call set_plasmasphere(idim,jdim,kdim,rmin,pmin,vm,eeta_new,imin_j)

         
      ! just in case:
      imin_j     = CEILING(bndloc)
      imin_j_old = CEILING(bndloc_old)


    
   ! Set boundary conditions on plasma (EETA) for all MLT's and energy levels:
   DO k=1,kdim
   DO j=1,jdim

      inew = imin_j(j)
      iold = imin_j_old(j)

      ! values on the (new) boundary and outside are from MHD:
      eeta (1:inew,j,k) = eeta_new (inew,j,k)

      IF (inew < iold) then

          ! There are newly-acquired points inside the modeling region,
          ! assign MHD-produced values to them:

         eeta (inew+1:iold,j,k) = eeta_new(inew+1:iold,j,k)

      END IF

   END DO
   END DO


! get the inner boundary eeta local time
      do j=1,jsize
       do k=1,ksize
        eetabnd(j,k) = eeta(imin_j(j),j,k)
        if(eetabnd(j,k) <= 0.0)then
          write(6,*)' warning: eetabnd <= 0 at j,k =',j,k
          write(6,*)' vm(',i,',',j,')=',vm(i,j)
        end if
       end do
      end do
! this is off for now
! set eeta to start with and empty magnetosphere
!     if(rec == 1)then
!     do j=1,jsize
!      do i=imin_j(j)+1,isize
!       do k = 1,ksize
!        eeta(i,j,k) = 1.0
!       end do
!      end do
!     end do
!     end if
      


! smooth eeta at the boundary
      CALL Smooth_eta_at_boundary(idim,jdim,kdim,jwrap,eeta,imin_j)

! now reset eeta outside the rcm to be eeta at the boundary
      do j=1,jdim
       do i=1,imin_j(j)-1
         eeta(i,j,:) = eeta(imin_j(j),j,:)
       end do
      end do



!-----------------------Write to rcmu2.dat-----------------
! this file is for plotting and/or debugging. RCM itselt does not need it.

      IF (L_write_rcmu2 ) then
          IF (rec == 1) THEN
              OPEN (LUN, FILE=rcmdir//'rcmu2.dat',form='unformatted', STATUS = 'replace')
              WRITE (LUN) idim,jdim,kdim
          ELSE
              OPEN (LUN,file=rcmdir//'rcmu2.dat',form='unformatted', status='old',position='append')
          END IF
          WRITE (LUN) itimei, alamc, x0, y0, z0, rmin, pmin,&
                      open, vm, press, den, bmin, ti, te, beta_average,eeta_new
          CLOSE (LUN)
      END IF
!----------------- end write to rcmu2.dat-------------------


      ! Complete preparing B-field arrays for RCM. We already have
      ! vm, bndloc, rmin, and pmin. Still need xmin and Ymin:

      xmin = rmin * COS (pmin)
      ymin = rmin * SIN (pmin)
      DO j = 1, jsize
        xmin(1:imin_j(j)-1,j) = xmin(imin_j(j),j) 
        ymin(1:imin_j(j)-1,j) = ymin(imin_j(j),j) 
      END DO

      DO j = 1, jsize; DO i = imin_j(j),isize
         IF (vm(i,j) <= 0.0) STOP 'vm problem in TORCM'
      END DO; END DO

      RETURN
      END SUBROUTINE Torcm





      SUBROUTINE Calc_ftv (big_vm,ierr) 
      USE conversion_module
      USE constants, ONLY : mass_proton,gamma,one_over_gamma,mu0
      USE RCM_mod_subs,ONLY : isize,jsize,kcsize,bmin, vm, sini,rmin,pmin,&
                              iprec,rprec
      USE rice_housekeeping_module
      IMPLICIT NONE
      REAL(rprec), INTENT (IN) :: big_vm
      INTEGER(iprec), INTENT (OUT) :: ierr
!
!===================================================================
!
! calc_ftv: computes flux tube volumes, mapping parameters (r,p)
!            be, sini, open/closed flag, and pressure on each 2D
!            ionospheric grid point
!            uses H. Voigts field line tracer for simplicity
!
! inputs: 
!      idim = dimension in latitude
!      jdim = dimension in local time
!      kdim = channel dimensions (not used here)
!  x0,y0,z0 = location of grid points in ionosphere
!   big_vm  = value to set open field lines
!      dir0 = direction that field line tracer must go 
!
!  outputs:
!        be = magnitude of b field in the equatorial plane
!        vm = flux tube volume**(-2/3)
!             note: vm for open fieldlines is flagged with a
!             large value (big_vm) (input)
!      sini = dip angle in the ionosphere from ground
!         r = equatorial r location of mapping in Re
!         p = equatorial lt location of mapping in radians
!     open  = 2d array flagging open/undefined (0) or closed (-1) pnts
!             on rcm grid
!    press  = pressure on field line, data from the mhd code in Pa
!    den    = density on field line, data from the mhd code in ple/cc 
!
!===================================================================
!
!  this routine maps either to the magnetopause        
!  or to the conjugate hemisphere and generates        
!  a potential distribution on a grid in the polar cap 
!  it does this by tracing field lines                 
!  using the trac tracer.                             
!  uses gehavo for the field line tracing              
!  created 03/90 last mod 08/95                       
!
      INCLUDE 'rcmdir.h'
!
      character(LEN=20) grid
      character(LEN=80) header
      integer(iprec),parameter :: imax = 2000
      real(rprec):: xx(imax), yy(imax), zz(imax) ,bbb(imax)
      real(rprec):: ftv(imax)

      real(rprec):: xs,ys,zs,xf,yf,zf,xe,ye,ze
      real(rprec):: dir,gla,glo,pot,poten,ftv1
      real(rprec):: ds,pp1,pp2,pp3
      real(rprec):: press1,dens1,x1,y1,z1,beta1
      real(rprec):: bxe,bye,bze,bbe
      real(rprec):: pi

      real, parameter :: press_min =0.0 !1.e-11 ! Pa

      integer(iprec) :: jj,iopen,ival,i,j,iiit,ii,numm,k
      integer(iprec) :: i0,j0
      integer(iprec), parameter:: use_avg = 1
      REAL(rprec), PARAMETER :: den_min = 0.0e6 ! in ples/m^3
      real(rprec):: rade
! check indices
      if(idim.ne.isize)stop ' error in idim in calc_ftv, aborting'
      if(jdim.ne.jsize)stop ' error in jdim in calc_ftv, aborting'
      if(kdim.ne.kcsize)stop ' error in kdim in calc_ftv, aborting'

      pi = acos(-1.0)

! startup with inputting the parameters.

      IF (L_write_tracing_debug) then
         open(unit=46,status='unknown', &
              file=rcmdir//'fline.dat',form='formatted')
         open(unit=47,status='unknown', &
              file=rcmdir//'endpoint.dat', form='formatted')
         open(unit=48,status='unknown', &
              file=rcmdir//'equator.dat', form='formatted')
         open(unit=50,status='unknown', &
              file=rcmdir//'ftv2.dat', form='formatted')
      END IF

      DO jj = 1, jdim
      DO ii = 1, idim

         xs  = x0   (ii,jj)
         ys  = y0   (ii,jj)
         zs  = z0   (ii,jj)
         dir = dir0 (ii,jj)

         IF (xs == 0.0 .AND. ys == 0.0 .AND. zs == 0.0) GO TO 9000
         IF (dir == 0.0) THEN
            WRITE (6,*)'>>>>>>>>dir = 0'
            STOP 'STOPPPING IN CALC_FTV'
         END IF

! delineate curves to be plotted

         iopen = -1  ! default is a closed field line  

         xf = 0.0
         yf = 0.0
         zf = 0.0 
         vm (ii,jj) = big_vm
         ftv1 = vm (ii,jj)

         CALL Mapd (xs, ys, zs, xf, yf, zf, xe, ye, ze, dir, iopen, &
                    xx, yy, zz, bbb, ftv, imax, ival, ierr)  
         IF (ierr < 0) STOP 'ierr negative in calc_fv3'

         open(ii,jj) = iopen
      
!        if equatorial plane is crossed then zs = 0.0
!        if(zs.eq.0.0)then
            IF (L_write_tracing_debug) WRITE (48,*)xe,ye,ze
!        end if

         gla = 0.0
         glo = 0.0  
         i = 0
         j = 0
         pot = 0.0
         IF (iopen > 0) THEN
            poten = yf
            IF (poten == 0.0) THEN 
!              check for border
               iopen = 0
               write(6,909)i,j,gla,glo,xf,yf,zf,'  <<undef+op'
            ELSE
               write(6,910)i,j,gla,glo,xf,yf,zf,poten
               pot = poten
            END IF
         END IF
! 
  909    FORMAT (2i3,5f12.5,a10)
  910    FORMAT (2i3,6f12.5)

! this prevents the same closed field line being plotted twice

! write out endpoints
       if(iopen.le.-1)call atran10(xf,yf,zf,gla,glo)
       IF (L_write_tracing_debug) write(47,470)xs,ys,zs,xf,yf,zf,gla,glo               
  470  format(8(1x,f10.4))

! flux tube volume calculation
! added average pressure and density calculation 8/03 frt

      if(iopen.le.-1)then
      ftv1 = 0.0
      press1 = 0.0
      dens1 = 0.0
      beta1 = 0.0
      do i=1,ival-1

      ds = sqrt((xx(i+1)-xx(i))**2+(yy(i+1)-yy(i))**2+ &
                (zz(i+1)-zz(i))**2)

      ftv1 = ftv1 + ds*2.0/(bbb(i+1)+bbb(i))

      x1 = (xx(i+1)+xx(i))/2.
      y1 = (yy(i+1)+yy(i))/2.
      z1 = (zz(i+1)+zz(i))/2.

      call bgrid(2,x1,y1,z1,pp1,pp2,pp3,ierr) ! in Pa
      if(ierr.lt.0)return

      press1 = press1 + pp1**(one_over_gamma)*ds*2.0/(bbb(i+1)+bbb(i))

      beta1 = beta1 + 2*mu0*pp1*(2.0/(bbb(i+1)+bbb(i)))**3*1.0e18*ds ! convert to normalized units

      call bgrid(3,x1,y1,z1,pp1,pp2,pp3,ierr)
      if(ierr.lt.0)return

      dens1 = dens1 + pp1*ds*2.0/(bbb(i+1)+bbb(i)) ! in kg/m^3
   
      end do

! to catch some errors
      if(abs(ftv1-ftv(ival)).gt.1.e-2)then
        write(6,*)' warning,  descrepency in ftv  '
        write(6,*)' xs,ys,zs =',xs,ys,zs
        write(6,*)' xf,yf,zf =',xf,yf,zf
        write(6,*)' ii,jj =',ii,jj  
        write(6,*)' ival = ',ival
        write(6,*)' ftv from trac =',ftv(ival)
        write(6,*)' ftv from integration =',ftv1
!       ierr = -1
!       return
      end if

      ftv1 = ftv(ival)

      if(ftv1.ne.0.0)vm(ii,jj) = ftv1**(-0.666667)

      else

       vm(ii,jj) = big_vm
       open(ii,jj) = 0
       ftv1 = big_vm

      end if
! now get the bfield in the equator

      call bfield_lfm(xe,ye,ze,bxe,bye,bze,bbe,ierr)
      if(ierr.lt.0)return

      bmin(ii,jj) = bbe

      if(use_avg == 0)then
! get the pressure also
      call bgrid(2,xe,ye,ze,pp1,pp2,pp3,ierr)
      if(ierr.lt.0)return
   
      press(ii,jj) = pp1 ! in kg/m^3

! get the mass density to get the ple density 
      call bgrid(3,xe,ye,ze,pp1,pp2,pp3,ierr)
      if(ierr.lt.0)return
   
      den(ii,jj) = pp1/mass_proton ! in ple m^3
      
      else ! use the average values
        press(ii,jj) = 0.0
        den(ii,jj) = 0.0
       if(ftv1 /=0 )then
        press(ii,jj) = (press1/ftv1)**gamma ! weighted by gamma
        den(ii,jj) = dens1/ftv1/mass_proton ! in ple m^3
       end if
      end if
! record the average plasma beta
        beta_average(ii,jj) = 0.0
       if(ftv1 /=0 )then
        beta_average(ii,jj) = (beta1/ftv1) ! average beta
       end if

! floor the density coming out of the lfm
      rade = sqrt(xe**2+ye**2)
      den(ii,jj) = max(den(ii,jj),den_min/rade)

! make sure that the density is a finite number in the closed
! field line region
      if (den(ii,jj) <=0.and.iopen.le.-1)then
!        write(6,*)' den <=0 at ii,jj =',ii,jj,' resetting to 1'
         den(ii,jj) = 0.
      end if

! zero pressure resets vm
      if(press(ii,jj).le.0.0.and.open(ii,jj).eq.-1)then
!       vm(ii,jj) = big_vm
!       open(ii,jj) = 0
!       write(6,*)' warning in calc_ftv, pressure le 0 at ii,jj =',ii,jj
!       write(6,*)' setting to background pressure ',press_min
        press(ii,jj) = 0.0
!       open(ii,jj) = -2
      end if

! now get the locations r and p    

      rmin(ii,jj) = sqrt(xe**2+ye**2)

      if(xe.eq.0.0.and.ye.eq.0.0)then
        pmin(ii,jj) = 0.0
        vm(ii,jj) = big_vm
        open(ii,jj) = 0
      else
        pmin(ii,jj) = atan2(ye,xe)
      end if

      if(pmin(ii,jj).lt.0.0)pmin(ii,jj)=pmin(ii,jj)+2.0*pi

 2001  continue

      END DO
      END DO


 999  continue              

 9000 CONTINUE
      IF (L_write_tracing_debug) then
         CLOSE (46)
         CLOSE (47)
         CLOSE (48)
         CLOSE (50)
      END IF

      RETURN 
      END SUBROUTINE Calc_ftv

!====================================================================
!
      SUBROUTINE Press2eta 

! ====================================================================
!
!  purpose: 
!     convert mhd/lfm pressure quantities to rcm quantities
! 
!  input:  
!     idim     number of grid points in latitudinal direction
!     jdim      number of grid points in longitudinal direction
!     vm(idim,jdim)       (flux tube vloume)^(-2/3) at each grid point
!     tempi(idim,jdim)     ion number density on that field line
!     tempe(idim,jdim)     electron number density on that field line
!     open(idim,jdim)      label to define if fieldline is open (0/1)
!                          or closed (-1)
!     press(idim,jdim)     total pressure on that field linee
!     kdim            number of energy channels
!     alam(kdim)      the energy invarant of each energy channel
!     almdel(kdim)    the width  of each energy channel
!     almmax(kdim)    the max alam of each energy channel
!     almmin(kdim)    the min alam of each energy channel
!     iflav(kdim)     the species of each energy channel
!     numspe(3)        number of channels of each species, e-, H+, O+
!
!  output:
!     eeta_new(idim,jdim,kdim)       flux tube content at this time
! 
!  other:    
!     mass of proton      weight of ion (kg)
!     mass of electron      weight of electron (kg)
!     boltz    Boltzman constant
!     eV       electron volts
!     trans    conversion factor (re/nt*1e6 (cc->m^3))
!
! ====================================================================
! notes:
!    This subroutine makes use of the Erf (actually the double 
!    precision version derf, erf returns erronous results), which is
!    not part of the standard fortran functions, but is very common.
!
!    1/19/2000 - frt
!
! --------------------------------------------------------------------
!
      USE conversion_module
      USE RCM_mod_subs, ONLY : ikflavc,vm,alamc,iprec,rprec
      USE CONSTANTS, ONLY : boltz,ev
      IMPLICIT NONE
      real(rprec):: xmin,xmax
      real(rprec):: ptemp,eta_correction
!
      real(rprec):: Erf
      EXTERNAL Erf
!
      real(rprec),parameter :: trans=6.375e15
      real(rprec),parameter :: eps=1.0e-30
      real(rprec):: pi,sqrtpi,factor,fac0,fac1,t
      integer(iprec) :: i,j,k

      pi     = ACOS(-1.0)
      sqrtpi = SQRT(pi)
    
! factor is a modification to the temperature to get a reasonable
! population in the higher energy channels.  Usually 100 suffices,
! set it to 1 for production runs.

      factor = 1.0
      if(factor.gt.1.)then
       write(*,*)' increasing temp in press2eta by a factor', factor
       ti = factor * ti
       te = factor * te
      end if
!
      DO j = 1, jdim
      DO i = 1, idim
!        IF (open(i,j) /= -1 .AND. vm(i,j) > 0.0) THEN
!        IF (open(i,j) /= -1 .OR. press(i,j) <= 0.0 ) THEN
         IF (open(i,j) /= -1 ) THEN
            eeta_new (i,j,:) = 0.0
         ELSE
            DO k = 1, kdim
               IF (ikflavc(k) == 1) THEN  ! electrons
                  t = te (i,j)
               ELSE  IF (ikflavc(k) == 2) THEN ! ions (protons)
                  t = ti (i,j)
               ELSE
                  STOP 'ILLEGAL IKFLAVC(K) IN PRESS2ETA'
               END IF
               if (t > 0.)then
                fac0 = trans*den(i,j)/((vm(i,j))**1.5)
                xmax = SQRT(ev*ABS(almmax(k))*vm(i,j)/boltz/t)
                xmin = SQRT(ev*ABS(almmin(k))*vm(i,j)/boltz/t)
                fac1 = (Erf(xmax)-Erf(xmin)) -2.0/sqrtpi* &
                       (xmax*EXP(-xmax**2)-xmin*EXP(-xmin**2))
                fac1 = MAX (fac1,eps)
                eeta_new(i,j,k)= fac0 * fac1
               else
                eeta_new(i,j,k) = 0.0
               endif
            END DO
         END IF
      END DO
      END DO
! now check to see if we get the original pressure back

      DO j=1, jdim
       DO i=1, idim
        IF (open(i,j) < 0) then
         ptemp = 0.0
         DO k=1,kdim
          ptemp = ptemp + &
           1.674E-35*ABS(alamc(k))*eeta_new(i,j,k)*vm(i,j)**2.5 
         END DO
         eta_correction = 0.0
         IF (ptemp/= 0)eta_correction = press(i,j)/ptemp
         DO k=1,kdim
          eeta_new(i,j,k) = eeta_new(i,j,k)*eta_correction
         END DO
         ptemp = 0.0
         DO k=1,kdim
          ptemp = ptemp + &
           1.674E-35*ABS(alamc(k))*eeta_new(i,j,k)*vm(i,j)**2.5 
         END DO
         IF (ABS(ptemp-press(i,j)) > 0.01*press(i,j))then
          Write(6,*)' Warning, pressures do not match at i,j =',i,j
          Write(6,*)' input pressure = ',press(i,j)
          Write(6,*)' RCM pressure = ',ptemp
!         PAUSE
         END IF
        END IF
       END DO
      END DO

!
      RETURN
      END SUBROUTINE Press2eta
!
!
!
!
!----------------------------atran10
!
      subroutine atran10(xs,ys,zs,gla,glo)
      USE Rcm_mod_subs, ONLY : iprec,rprec
      implicit none
      real(rprec) :: xs,ys,zs,gla,glo,rrr,pi,rho
      real(rprec) :: atan5 
                         
      pi = acos(-1.0)

      gla = 0.0
      glo = 0.0    
      rrr = sqrt(xs**2+ys**2+zs**2)
      if(rrr.eq.0.0)return 

      xs = xs/rrr
      ys = ys/rrr
      zs = zs/rrr

      rho = sqrt(xs**2+ys**2)   

      glo = 0.0
      if(rho.ne.0.)then
      glo = atan5(-ys,xs)    
      glo = glo/pi*180. 
      if(glo.lt.0.0)glo=glo+360.
      end if   

      gla = atan5(rho,zs)
      gla = 90.-gla*180./pi

      return
      end        
! -----------------------------atan5
      function atan5(y,x)
      USE Rcm_mod_subs, ONLY : iprec,rprec
      implicit none

      real(rprec) :: atan5,pi,y,x
      pi = acos(-1.0)
      atan5 = 0.00
      if((x.ne.0.0).or.(y.ne.0.0))   atan5 = atan2(y,x)
 
      return
      end


      SUBROUTINE Mapd (xs, ys, zs, xf, yf, zf, xe, ye, ze, dir, iopen, &
                       xx, yy, zz, bbb, ftv, imax, ival, ierr)

!==============================================================
!
! mapd:  does a field line tracing from starting location 
!        x0,y0,z0 to ending location xf,yf,zf
!
! inputs:
! xs,ys,zs = starting location
! xf,yf,zf = ending location,either in the ionosphere or
!            some defined boundary
! xe,ye,ze = equatorial crossing point (if there is one)
!      dir = direction for mapping (either -1/+1) wrt field line
!     imax = max size of 3 1D arrays that have field line coordinates
!
! outputs:
!
!    iopen = flag if point is connected to inosphere (-1) or bndy (0/1)
!   xx,yy,zz = coordinates of a single field line (size imax)
!        bbb = magnitude of field along field line track
!        ftv = flux tube volume as a function of field line location
!       ival = ending index where field line tracing stopped
!       ierr = error flag (=0 ok)
!
!==============================================================

      USE intermediate_grid, ONLY : xmin_ig,xmax_ig,&
                                    ymin_ig,ymax_ig,&
                                    zmax_ig,zmin_ig
      USE tracer_params
      USE Rcm_mod_subs, ONLY : iprec,rprec
      IMPLICIT NONE

      integer(iprec) :: imax
      real(rprec):: xx(imax), yy(imax), zz(imax), bbb(imax),ftv(imax)
      real(rprec):: dx1,dy1,dz1,bb,h,ep,er
      real(rprec):: xs,ys,zs,xf,yf,zf,xe,ye,ze,dir
      real(rprec):: dr,r,rrr   
      real(rprec):: xn,yn,zn,xm,ym,zm,fn,fm,ff
      real(rprec):: bbx,bby,bbz,bbt
      real(rprec):: rn,rm
      real(rprec),parameter :: grr = 1
      real(rprec),parameter :: bbmin = 10 ! min bfield value
      integer(iprec) :: iopen,ival,icr1,i 
      integer(iprec) :: ierr

      xe = 0.0
      ye = 0.0
      ze = 990.0
      icr1 = 1
 
      er = er1
      iopen = -1.0
      h  = 0.5 
! used to be 0.1

      ep = dir

      if(dir.eq.0.0)then
       write(6,*)
       write(6,*)' ***********warning dir=0 in mapd **********'
       write(6,*)
      end if
! reset ftv
      do i=1,imax
       ftv(i) = 0.0
      end do

      xm    = xs
      ym    = ys
      zm    = zs
      fm    = 0.0
                             
      xx(1) = xm
      yy(1) = ym
      zz(1) = zm 
      ftv(1) = 0.0

      CALL Bfield_lfm (xs, ys, zs, bbx, bby, bbz, bbt, ierr)
      IF (ierr < 0) STOP 'ERROR IN MAPD'
      bbb(1) = bbt          
      h = 0.1
      DO i = 2,imax

 400     CALL Tracf2 (xm,ym,zm,fm,xn,yn,zn,fn,er,ep,h,ierr) 
         CALL Bfield_lfm(xn,yn,zn,bbx,bby,bbz,bbt,ierr)
         IF (ierr < 0) STOP 'ERROR IN MAPD'
         r = SQRT(xn*xn + yn*yn + zn*zn)
!        safety switch                   
         dr = SQRT((xm-xn)**2+(ym-yn)**2+(zm-zn)**2)
         IF (dr < 1.0e-3 .AND. r <= 2.0) RETURN
         bb = bbt
         bbb(i) = bb
 
!        set limits on h  
         IF (h > 0.1) h=0.1
! 
!        check to see if z=0 is crossed
!        look for crossing point for bfield
!
         IF (zm*zn <= 0.0) THEN
            IF (zm-zn.ne.0.0) THEN
               xe = (xn*zm-xm*zn)/(zm-zn)
               ye = (yn*zm-ym*zn)/(zm-zn)
               ze = 0.0
            ELSE
               xe = xn
               ye = yn
               ze = zn
            END IF
         END IF

! .......need to be careful about what error limits exist and where...
! 
!        condition open field line
!-----------note er is now input
         IF (r < 4.0) THEN
            er = er2
         ELSE
            er = er1
         END IF
! for ftv calc
         IF (bbt < bbmin) THEN
            er = er2
         END IF
! 
!        condition closed field line:
         IF (r < grr .AND. i > 10) THEN
            iopen = - 1
!           reset to r = grr by interpolation
!
            rn=sqrt(xn**2+yn**2+zn**2)
            rm=sqrt(xm**2+ym**2+zm**2)

            xf = (xm*(grr-rn)+xn*(rm-grr))/(rm-rn)
            yf = (ym*(grr-rn)+yn*(rm-grr))/(rm-rn)
            zf = (zm*(grr-rn)+zn*(rm-grr))/(rm-rn)
            ff = (fm*(grr-rn)+fn*(rm-grr))/(rm-rn)

            xn = xf
            yn = yf
            zn = zf
            fn = ff

            xx(i) = xf
            yy(i) = yf
            zz(i) = zf
            ftv(i) = ff

            RETURN
         END IF
!
!        condition don't know
! 
         if((xn.le.xmin_ig).or.(yn.le.ymin_ig).or.(zn.le.zmin_ig).or. &
         (xn.ge.xmax_ig).or.(yn.ge.ymax_ig).or.(zn.ge.zmax_ig).or. &
                         (bb.lt.0.1)) &
                         then
 404         continue
             iopen = 0
             xf = xn
             yf = yn
             zf = zn
             ff = fn

             xx(i) = xf
             yy(i) = yf
             zz(i) = zf
             ftv(i) = ff

             return
         END IF
 
         xx(i) = xn
         yy(i) = yn
         zz(i) = zn
         ftv(i) = fn
                                   
         ival = i

         xm = xn
         ym = yn
         zm = zn
         fm = fn
 
      END DO
 
      write(6,*)' i greater than ',imax
      write(6,*)'xn,yn,zn,fn',xn,yn,zn,fn
      RETURN
      END SUBROUTINE Mapd
     



      SUBROUTINE Begin (rec, ierr)
!-----------------------------------------------------
!      sets up the mapping program
!-----------------------------------------------------
      USE intermediate_grid, ONLY : xmin_ig,xmax_ig,&
                                    ymin_ig,ymax_ig,&
                                    zmax_ig,zmin_ig
      USE tracer_params
      USE dipole_params
      USE Rcm_mod_subs, ONLY : iprec,rprec
      implicit none
      integer, intent(in) :: rec
      real(rprec):: glatb,glatf,nglat
      real(rprec):: glb,glf
      real(rprec):: dx1,dy1,dz1,bb,h,ep,er
      real(rprec):: ti,pi
      real(rprec):: x,y,z,bx,by,bz
      integer(iprec) :: nglatst,ngl
      integer(iprec) :: iswr,iswp,iswt
      integer(iprec) :: iswit
      integer(iprec) :: ierr
      character(LEN=2) sym
      data iswr /1/
      data iswp /1/
      data iswt /1/
      character(LEN=80) header

      INCLUDE 'rcmdir.h'

      tilt = 0.0
      ti = tilt
      h  = 0.1 
      pi = acos(-1.0)

! used to be 0.1
 
! * * initialize the magnetosphere-parameters * * *
 
!      no tilt used here
!     set up the programs gehavo and norm
                 
      iswit = 0
      call bgrid(iswit,x,y,z,bx,by,bz,ierr) 
      if(ierr.lt.0)return
      iswit = 1

      write(6,*)' starting h = ',h

      write(6,*)' er1 = ',er1,'  er2 = ',er2
      
      write(6,*)' xmax_ig =',xmax_ig,' xmin_ig =',xmin_ig
      write(6,*)' ymax_ig =',ymax_ig,' ymin_ig =',ymin_ig
      write(6,*)' zmax_ig =',zmax_ig,' zmin_ig =',zmin_ig

      RETURN
      END SUBROUTINE Begin
!
!-----------------------------------------------------

      subroutine tracf(xa,ya,za,fa,xn,yn,zn,fn,er,ep,h,ierr)

!================================================================
! purpose:
! field line tracing program, based on the Voigt first order tracer
! keeps track of changes in flux tube volume to adjust stepsizes
!
! inputs:
!     xa,ya,za: starting location
!           fa: starting flux tube volume value
!           er: error tolerance for tracer
!           ep: direction (along =1, against -1) along field line
!            h: stepsize (new value will be outputted)
! outputs:
!     xf,yf,zf: ending location
!           vf: end flux tube volume value    
!            h: new stepsize
!         ierr: error flag
!
!================================================================
      USE Rcm_mod_subs, ONLY : iprec,rprec
      implicit none
      real(rprec):: xa,ya,za,fa,xn,yn,zn,fn
      real(rprec):: dx1,dy1,dz1,bf1,h,ep,er
      real(rprec):: dxt,dyt,dzt,bft
      real(rprec):: xn1,yn1,zn1,fn1
      real(rprec):: xn2,yn2,zn2,fn2
      real(rprec):: xn3,yn3,zn3,fn3
      real(rprec):: dx2,dy2,dz2,bf2,df2
      real(rprec):: rtes1,rtes2,rtes3,rtesf,rtes
      real(rprec):: delx,dely,delz,delf
      real(rprec):: h1,h2
      real,parameter :: ftv_fact =200.
      integer(iprec) :: ierr

      call bfield_lfm(xa,ya,za,dx1,dy1,dz1,bf1,ierr)
      if(ierr.lt.0.or.bf1.eq.0.0)return
  1   continue

      h1 = h * ep
      h2 = 0.5 * h1

      xn1 = xa + h1*dx1
      yn1 = ya + h1*dy1
      zn1 = za + h1*dz1
      call bfield_lfm(xn1,yn1,zn1,dxt,dyt,dzt,bft,ierr)
      if(ierr.lt.0.or.bft.eq.0.0)return

      fn1 = fa + 2.0*abs(h1/(bf1+bft))

      xn2 = xa + h2*dx1
      yn2 = ya + h2*dy1
      zn2 = za + h2*dz1

      call bfield_lfm(xn2,yn2,zn2,dxt,dyt,dzt,bft,ierr)
      if(ierr.lt.0.or.bft.eq.0.0)return

      fn2 = fa + 2.0*abs(h2/(bf1+bft))

      call bfield_lfm(xn2,yn2,zn2,dx2,dy2,dz2,bf2,ierr)
      if(ierr.lt.0.or.bf2.eq.0.0)return

      xn3 = xn2 + h2*dx2
      yn3 = yn2 + h2*dy2
      zn3 = zn2 + h2*dz2

      call bfield_lfm(xn3,yn3,zn3,dxt,dyt,dzt,bft,ierr)
      if(ierr.lt.0.or.bft.eq.0.0)return

      fn3 = fn2 + 2.0*abs(h2/(bf2+bft))

      delx = xn1 - xn3
      dely = yn1 - yn3
      delz = zn1 - zn3                
      delf = fn1 - fn3                

      rtes1 = delx*delx
      rtes2 = dely*dely
      rtes3 = delz*delz
! decrease tolerance of ftv by ftv_fact
      rtesf = delf*delf*ftv_fact

      rtes = sqrt(rtes1 + rtes2 + rtes3 + rtesf)
!      rtes = rtes/sqrt(xa**2+ya**2+za**2) 

      if (rtes-er) 4,4,2
  4   continue

      xn = xn3 - delx
      yn = yn3 - dely
      zn = zn3 - delz
      fn = fn3 - delf

!      bb = bf2
      if(rtes-(0.25*er)) 7,6,6

  7   h = 2.0*h
  6   go to 5

  2   continue
      h = 0.5*h
      go to 1

  5   continue
      return
      end
!-----------------------------------------------------

      subroutine tracf2(xa,ya,za,fa,xn,yn,zn,fn,er,ep,h,ierr)

!================================================================
! purpose:
! field line tracing program, based on the Voigt first order tracer
! keeps track of changes in flux tube volume to adjust stepsizes
! basically an adaptive euler method
!
! inputs:
!     xa,ya,za: starting location
!           fa: starting flux tube volume value
!           er: error tolerance for tracer
!           ep: direction (along =1, against -1) along field line
!            h: stepsize (new value will be outputted)
! outputs:
!     xf,yf,zf: ending location
!           vf: end flux tube volume value    
!            h: new stepsize
!         ierr: error flag
! f95 version 6.04 frt
!================================================================
      USE Rcm_mod_subs, ONLY : iprec,rprec
      implicit none
      real(rprec):: xa,ya,za,fa,xn,yn,zn,fn
      real(rprec):: dx1,dy1,dz1,bf1,h,ep,er
      real(rprec):: dxt,dyt,dzt,bft
      real(rprec):: xn1,yn1,zn1,fn1
      real(rprec):: xn2,yn2,zn2,fn2
      real(rprec):: xn3,yn3,zn3,fn3
      real(rprec):: dx2,dy2,dz2,bf2,df2
      real(rprec):: rtes1,rtes2,rtes3,rtesf,rtes
      real(rprec):: delx,dely,delz,delf
      real(rprec):: h1,h2
      real(rprec), parameter :: ftv_fact = 1
      integer(iprec), parameter :: maxits = 5000
      integer(iprec) :: ierr,it

      call bfield_lfm(xa,ya,za,dx1,dy1,dz1,bf1,ierr)
      if(ierr.lt.0.or.bf1.eq.0.0)return
      
      rtes = 100*er
      
      do it=1,maxits

      h1 = h * ep
      h2 = 0.5 * h1

      xn1 = xa + h1*dx1
      yn1 = ya + h1*dy1
      zn1 = za + h1*dz1
      call bfield_lfm(xn1,yn1,zn1,dxt,dyt,dzt,bft,ierr)
      if(ierr.lt.0.or.bft.eq.0.0)return

      fn1 = fa + 2.0*abs(h1/(bf1+bft))

      xn2 = xa + h2*dx1
      yn2 = ya + h2*dy1
      zn2 = za + h2*dz1

      call bfield_lfm(xn2,yn2,zn2,dxt,dyt,dzt,bft,ierr)
      if(ierr.lt.0.or.bft.eq.0.0)return

      fn2 = fa + 2.0*abs(h2/(bf1+bft))

      call bfield_lfm(xn2,yn2,zn2,dx2,dy2,dz2,bf2,ierr)
      if(ierr.lt.0.or.bf2.eq.0.0)return

      xn3 = xn2 + h2*dx2
      yn3 = yn2 + h2*dy2
      zn3 = zn2 + h2*dz2

      call bfield_lfm(xn3,yn3,zn3,dxt,dyt,dzt,bft,ierr)
      if(ierr.lt.0.or.bft.eq.0.0)return

      fn3 = fn2 + 2.0*abs(h2/(bf2+bft))

      delx = xn1 - xn3
      dely = yn1 - yn3
      delz = zn1 - zn3                
      delf = fn1 - fn3                

      rtes1 = delx*delx
      rtes2 = dely*dely
      rtes3 = delz*delz
! decrease tolerance of ftv by ftv_fact
      rtesf = delf*delf*ftv_fact

      rtes = sqrt(rtes1 + rtes2 + rtes3 + rtesf)
!      rtes = rtes/sqrt(xa**2+ya**2+za**2) 

      if(rtes > er)then
       h = 0.5*h
      else
! tolerences satisfied, update the position and exit 
! (extrapolates)
       xn = xn3 - delx
       yn = yn3 - dely
       zn = zn3 - delz
       fn = fn3 - delf
! increase h if it rtes is smaller than 0.25 *er
       if(rtes < (0.25*er)) h = 2.0*h
       return
      
      endif
      
      end do ! main iteration loop
      
! if you get to this point, there is a problem
      write(*,*)' ---------------------------------'
      write(*,*)' maxits exceeded in tracf, aborting '
      write(*,*)' xa,ya,za,fa =',xa,ya,za,fa
      write(*,*)' xn,yn,zn,fn =',xn,yn,zn,fn
      write(*,*)' h =',h,' er =',er,' rtes =',rtes
      write(*,*)' ---------------------------------'
      stop
      
      end subroutine tracf2
!----------------------------------------------------------

      subroutine Bfield_lfm(x,y,z,dx,dy,dz,bf,ierr)

!-----------------bfield_lfm----------------------
! returns bfieild at a specified point (x,y,z) in the form
! dx = bx/bf, dy=by/Bf, dz = bz/bf from the intermediate grid
!--------------------------------------------------

      USE Rcm_mod_subs, ONLY : iprec,rprec
      implicit none
      real(rprec) :: x,y,z,dx,dy,dz,bf,bx,by,bz
      real(rprec) :: xtrans,ytrans,ztrans,b1,b2,b3,hx,hy,hz
      integer(iprec) :: ierr
! since the grid is good for z > 0 only assume symmetry

      hx = 0.0
      hy = 0.0
      hz = 0.0
      xtrans = x
      ytrans = y
      ztrans = z

        CALL Bgrid (1,xtrans,ytrans,ztrans,bx,by,bz,ierr)
        if(ierr.lt.0)return

      bf = sqrt(bx**2+by**2+bz**2)
      call dip(0.0,x,y,z,hx,hy,hz)
!
      bx =   bx + ( hx)
      by =   by + ( hy)
      bz =   bz + ( hz)
      bf = sqrt(bx**2+by**2+bz**2)

      IF (bf.eq.0.0)then
!      IF ((bf-1.e-8.lt.0.0).and.(bf+1.e-8.gt.0.0))then
      write(6,*)' error in bfield, b=0 at'
        write(6,*)x,y,z
        write(6,*)' grid field =',bx,by,bz
        write(6,*)' dipole field =',hx,hy,hz
        ierr = -2
        return
      END IF

      dx = bx/bf
      dy = by/bf
      dz = bz/bf

      RETURN
      END SUBROUTINE Bfield_lfm
!
!
!-------------------------------------------------
!
      SUBROUTINE Bgrid (isw, xp_in, yp_in, zp_in, bxp, byp, bzp, ierr)

! modified to use module 7/09 frt

!========================================================
! returns bfield from lfm (delta B or pressure)
! depending on the input flag
! isw = 0 reads in the data
! isw = 1 returns bfield (delta B) in nT
! isw = 2 returns pressure in Pa
! isw = 3 returns mass density in kg/m^3
! isw = 4 returns the mask value, which will only be 1 
!         if all 8 points are nozero
! isw = 5 returns velocity in km/s
!========================================================

       USE intermediate_grid
       implicit none

       integer i,j,k
       integer, SAVE :: ilast,jlast,klast

       real(rprec):: xtrans,ytrans,ztrans
       real(rprec):: xp,yp,zp,bxp,byp,bzp
       real(rprec):: xp_in,yp_in,zp_in
       real(rprec):: v1,v2,v3,v4,v5,v6,v7,v8,vt
       real(rprec):: b1,b2,b3,b4,b5,b6,b7,b8
       real(rprec):: b,r,hx,hy,hz
       real(rprec):: bxmax,bymax,bzmax,pmax,pmin,rhomax,rhomin
       real(rprec):: xbxmax,ybxmax,zbxmax
       real(rprec):: xbymax,ybymax,zbymax
       real(rprec):: xbzmax,ybzmax,zbzmax
       real(rprec):: xpmax,ypmax,zpmax
       real(rprec):: xpmin,ypmin,zpmin
       real(rprec):: xrhomax,yrhomax,zrhomax
       real(rprec):: xrhomin,yrhomin,zrhomin
       real, parameter :: rmin =1.0
       integer :: ip,jp,kp
       integer,parameter :: isym = 0
       integer :: ierr
       integer :: isw

       character(LEN=50) file_in

       bxp = 0.0
       byp = 0.0
       bzp = 0.0

       IF (isw == 0) THEN     ! read data from file
          ierr=0

          ilast = 0
          jlast = 0
          klast = 0

!         read in the grid

          CALL Bin (ierr)
          IF (ierr /= 0) STOP 'ERROR READING DATA FROM FILE IN BGRID'

          write(6,*)' BGRID: reading done  '

!         set ranges of position

          xmin_ig = x_ig(2) ! frt 8.02
          xmax_ig = x_ig(idim_ig-2) 
          ymin_ig = y_ig(2)
          ymax_ig = y_ig(jdim_ig-2) 
          zmax_ig = z_ig(kdim_ig-2)
          zmin_ig = z_ig(2)

      ELSE       ! calculating requested values

! check to see if the point is outside the grid, if so return null

         xp = xp_in
         yp = yp_in
         zp = zp_in
         ierr = 0 ! reset ierr to zero
         if(isym.eq.1)zp=abs(zp_in) ! assumes a symmetric grid

         IF (xp < x_ig(1) .OR. xp > x_ig(idim_ig) .OR. &
             yp < y_ig(1) .OR. yp > y_ig(jdim_ig) .OR. &
             zp < z_ig(1) .OR. zp > z_ig(kdim_ig)) THEN
!            WRITE (6,*) ' BGRID: POINT IS OUTSIDE THE BOX, RETURN!'
             RETURN
         END IF

!        first locate where you are in the grid
!        if the location is found then a non-zero ip,jp,kp will 
!        be generated
!        ilast, jlast and klast are nonzero and the locations on the grid
!        where the subroutine was called last time

         ip = 0
         IF (ilast /= 0)then
          IF(  x_ig(ilast) <= xp .AND. x_ig(ilast+1) >= xp) ip = ilast
         END IF
         IF (ip.eq.0) then    !          find it
            DO i=1,idim_ig-1
              if (x_ig(i).le.xp.and.x_ig(i+1).ge.xp) then
               ip = i
               exit
              endif
            end do
         end if
         if (ip.eq.0)then
          write(6,*)' ip not found for x = ',xp
          STOP 'BGRID: UNRECOVERABLE ERROR'
         end if

         jp = 0
         IF (jlast /= 0)THEN
          IF(y_ig(jlast) <= yp .AND. y_ig(jlast+1) >= yp) jp = jlast
         END IF
         if(jp.eq.0)then
          do j=1,jdim_ig-1
           if(y_ig(j).le.yp.and.y_ig(j+1).ge.yp)then 
            jp = j
            exit
           endif
          end do
         endif
         if (jp.eq.0)then
          write(6,*)' jp not found for y = ',yp
          STOP 'BGRID: UNRECOVERABLE ERROR'
         endif

         kp = 0
         IF(klast.ne.0)THEN
          IF(z_ig(klast).le.zp.and.z_ig(klast+1).ge.zp) kp = klast
         END IF
         if(kp.eq.0)then
          do k=1,kdim_ig-1
           if(z_ig(k).le.zp.and.z_ig(k+1).ge.zp)then 
            kp = k
            exit
           endif
          end do
         endif
         if(kp.eq.0)then
          write(6,*)' kp not found for z = ',zp
          STOP 'BGRID: UNRECOVERABLE ERROR'
         endif

!        record where you where so it may be used the next time this
!        routine is called
         ilast = ip
         jlast = jp
         klast = kp

! now do the interpolation

!  (i,j+1,k+1) +-------------+(i+1,j+1,k+1)
!             / 7  /  8     /|
!            /----/--------/8|
!           /  3 /   4    /| |
! (i,j+1,k) +------------+4|/|
!          |     |       | / |
!          |  3  |   4   |/|5+ (i+1,j,k+1)
!          |-----+-------|2|/
!          |     |       | /
!          |  1  |   2   |/
!          +-------------+
!       (i,j,k)     (i+1,j,k)

!   volume label    vertex
!       1         (i  ,j  ,k  )
!       2         (i+1,j  ,k  )
!       3         (i  ,j+1,k  )
!       4         (i+1,j+1,k  )
!       5         (i  ,j  ,k+1)
!       6         (i+1,j  ,k+1)
!       7         (i  ,j+1,k+1)
!       8         (i+1,j+1,k+1)
!       

         v1 = (xp-x_ig(ip))*  (yp-y_ig(jp))*    (zp-z_ig(kp))
         v2 = (x_ig(ip+1)-xp)*(yp-y_ig(jp))*    (zp-z_ig(kp))
         v3 = (xp-x_ig(ip))*  (y_ig(jp+1)-yp)*  (zp-z_ig(kp))
         v4 = (x_ig(ip+1)-xp)*(y_ig(jp+1)-yp)*  (zp-z_ig(kp))
         v5 = (xp-x_ig(ip))*  (yp-y_ig(jp))*    (z_ig(kp+1)-zp)
         v6 = (x_ig(ip+1)-xp)*(yp-y_ig(jp))*    (z_ig(kp+1)-zp) 
         v7 = (xp-x_ig(ip))*  (y_ig(jp+1)-yp)*  (z_ig(kp+1)-zp) 
         v8 = (x_ig(ip+1)-xp)*(y_ig(jp+1)-yp)*  (z_ig(kp+1)-zp) 

         vt = (x_ig(ip+1)-x_ig(ip))*(y_ig(jp+1)-y_ig(jp))*(z_ig(kp+1)-z_ig(kp))

!        write(6,*)' vol diff ',vt-v1-v2-v3-v4-v5-v6-v7-v8

!        now for the values

         if(isw.eq.2)then   ! return pressures

            b1 = press_ig(ip  ,jp  ,kp)
            b2 = press_ig(ip+1,jp  ,kp)
            b3 = press_ig(ip  ,jp+1,kp)
            b4 = press_ig(ip+1,jp+1,kp)
            b5 = press_ig(ip  ,jp  ,kp+1)
            b6 = press_ig(ip+1,jp  ,kp+1)
            b7 = press_ig(ip  ,jp+1,kp+1)
            b8 = press_ig(ip+1,jp+1,kp+1)

! remove weight if any points are zero

            if(b1.eq.0.0)v8=0.0
            if(b2.eq.0.0)v7=0.0
            if(b3.eq.0.0)v6=0.0
            if(b4.eq.0.0)v5=0.0
            if(b5.eq.0.0)v4=0.0
            if(b6.eq.0.0)v3=0.0
            if(b7.eq.0.0)v2=0.0
            if(b8.eq.0.0)v1=0.0
 
            vt = v1+v2+v3+v4+v5+v6+v7+v8

            if(vt.le.0.0) THEN
!              WRITE (6,*) ' BGRID: ZERO INTERPOLATION WEIGHT!'
               RETURN
            END IF

            bxp = (v1*b8+v2*b7+v3*b6+v4*b5+v5*b4+v6*b3+v7*b2+v8*b1)/vt
            byp = bxp
            bzp = bxp

         else if(isw.eq.3)then ! return mass density

            b1 = rho_ig(ip  ,jp  ,kp)
            b2 = rho_ig(ip+1,jp  ,kp)
            b3 = rho_ig(ip  ,jp+1,kp)
            b4 = rho_ig(ip+1,jp+1,kp)
            b5 = rho_ig(ip  ,jp  ,kp+1)
            b6 = rho_ig(ip+1,jp  ,kp+1)
            b7 = rho_ig(ip  ,jp+1,kp+1)
            b8 = rho_ig(ip+1,jp+1,kp+1)

! remove weight if any points are zero

            if(b1.eq.0.0)v8=0.0
            if(b2.eq.0.0)v7=0.0
            if(b3.eq.0.0)v6=0.0
            if(b4.eq.0.0)v5=0.0
            if(b5.eq.0.0)v4=0.0
            if(b6.eq.0.0)v3=0.0
            if(b7.eq.0.0)v2=0.0
            if(b8.eq.0.0)v1=0.0
 
            vt = v1+v2+v3+v4+v5+v6+v7+v8

            if(vt.le.0.0) THEN
!              WRITE (6,*) ' BGRID: ZERO INTERPOLATION WEIGHT!'
               RETURN
            END IF

            bxp = (v1*b8+v2*b7+v3*b6+v4*b5+v5*b4+v6*b3+v7*b2+v8*b1)/vt
            byp = bxp
            bzp = bxp

         else if(isw.eq.4)then

! mask value, returns a 1 only if all 8 points are 1
            b1 = imask_ig(ip  ,jp  ,kp)
            b2 = imask_ig(ip+1,jp  ,kp)
            b3 = imask_ig(ip  ,jp+1,kp)
            b4 = imask_ig(ip+1,jp+1,kp)
            b5 = imask_ig(ip  ,jp  ,kp+1)
            b6 = imask_ig(ip+1,jp  ,kp+1)
            b7 = imask_ig(ip  ,jp+1,kp+1)
            b8 = imask_ig(ip+1,jp+1,kp+1)

!           bxp = b1*b2*b3*b4*b5*b6*b7*b8
            bxp = floor((b1+b2+b3+b4+b5+b6+b7+b8)/8.)

         else if(isw.eq.5)then

! lfm velocity in km/s
            b1 = vel_ig(ip  ,jp  ,kp)
            b2 = vel_ig(ip+1,jp  ,kp)
            b3 = vel_ig(ip  ,jp+1,kp)
            b4 = vel_ig(ip+1,jp+1,kp)
            b5 = vel_ig(ip  ,jp  ,kp+1)
            b6 = vel_ig(ip+1,jp  ,kp+1)
            b7 = vel_ig(ip  ,jp+1,kp+1)
            b8 = vel_ig(ip+1,jp+1,kp+1)

! remove weight if any points are zero

            if(b1.eq.0.0)v8=0.0
            if(b2.eq.0.0)v7=0.0
            if(b3.eq.0.0)v6=0.0
            if(b4.eq.0.0)v5=0.0
            if(b5.eq.0.0)v4=0.0
            if(b6.eq.0.0)v3=0.0
            if(b7.eq.0.0)v2=0.0
            if(b8.eq.0.0)v1=0.0
 
            vt = v1+v2+v3+v4+v5+v6+v7+v8

            if(vt.le.0.0) THEN
!              WRITE (6,*) ' BGRID: ZERO INTERPOLATION WEIGHT!'
               RETURN
            END IF

            bxp = (v1*b8+v2*b7+v3*b6+v4*b5+v5*b4+v6*b3+v7*b2+v8*b1)/vt
            byp = bxp
            bzp = bxp

         else
! bfield
            b1 = bx_ig(ip  ,jp  ,kp)
            b2 = bx_ig(ip+1,jp  ,kp)
            b3 = bx_ig(ip  ,jp+1,kp)
            b4 = bx_ig(ip+1,jp+1,kp)
            b5 = bx_ig(ip  ,jp  ,kp+1)
            b6 = bx_ig(ip+1,jp  ,kp+1)
            b7 = bx_ig(ip  ,jp+1,kp+1)
            b8 = bx_ig(ip+1,jp+1,kp+1)

            bxp = (v1*b8+v2*b7+v3*b6+v4*b5+v5*b4+v6*b3+v7*b2+v8*b1)/vt

            b1 = by_ig(ip  ,jp  ,kp)
            b2 = by_ig(ip+1,jp  ,kp)
            b3 = by_ig(ip  ,jp+1,kp)
            b4 = by_ig(ip+1,jp+1,kp)
            b5 = by_ig(ip  ,jp  ,kp+1)
            b6 = by_ig(ip+1,jp  ,kp+1)
            b7 = by_ig(ip  ,jp+1,kp+1)
            b8 = by_ig(ip+1,jp+1,kp+1)

            byp = (v1*b8+v2*b7+v3*b6+v4*b5+v5*b4+v6*b3+v7*b2+v8*b1)/vt

            b1 = bz_ig(ip  ,jp  ,kp)
            b2 = bz_ig(ip+1,jp  ,kp)
            b3 = bz_ig(ip  ,jp+1,kp)
            b4 = bz_ig(ip+1,jp+1,kp)
            b5 = bz_ig(ip  ,jp  ,kp+1)
            b6 = bz_ig(ip+1,jp  ,kp+1)
            b7 = bz_ig(ip  ,jp+1,kp+1)
            b8 = bz_ig(ip+1,jp+1,kp+1)

            bzp = (v1*b8+v2*b7+v3*b6+v4*b5+v5*b4+v6*b3+v7*b2+v8*b1)/vt
      
            if (isym.eq.1.and.zp_in.lt.0.0) then
               bxp = - bxp
               byp = - byp
            end if

         END IF
      END IF
      
      RETURN
      END SUBROUTINE Bgrid

!---------------------------------------------------

!FIXME: torcm.f90 could be refactored: I think all the functionality
!       of Bin should be moved to subroutine ImportMagnetosphere().

      SUBROUTINE Bin(ierr)
        use intercomm_module, only : ImportMagnetosphere
        USE intermediate_grid, only : idim_ig, jdim_ig, kdim_ig, &
             x_ig, y_ig, z_ig, &      !> Intermediate grid positions
             bx_ig, by_ig, bz_ig, &   !> Accumulate magnetic field
             press_ig, &              !> Accumulate pressure
             rho_ig, &                !> Accumulate density
             vel_ig, &                !> Accumulate speed
             imask_ig                 !> Mask (0,1)... is this necessary?
          
       USE dipole_params, only : dm
      USE CONSTANTS, ONLY : radius_earth_m
      USE Rcm_mod_subs, ONLY : rprec
      USE rice_housekeeping_module,only : L_write_vars_debug,&
                                          L_write_int_grid_debug
      IMPLICIT NONE

      INTEGER, INTENT (OUT) ::  ierr

!=========================================================
! Imports &converts mhd results to rcm (gsm) 
!=========================================================

      integer :: in,i,j,k
      integer :: istart,iend,iit,jstart,jend,jit,kstart,kend,kit
      integer :: index
      real(rprec):: bmax,bb
      real*8 :: mjdtime
      real(rprec):: dmin

      
! Import magnetosphere variables and store results in variables in
! intermediate_grid module.  
!
! Note:  The following calls MUST occur before this:
!         - intermediate_grid::setupIg()
!         - intercomm_module::Initialize()
      call ImportMagnetosphere()
! Mask should be hard-coded to zer0 here.
      imask_ig = 0

      if(L_write_vars_debug)then
         write(*,*)' x_ig(1) =',x_ig(1)
         write(*,*)' x_ig(idim_ig) =',x_ig(idim_ig)
         write(*,*)' y_ig(1) =',y_ig(1)
         write(*,*)' y_ig(jdim_ig) =',y_ig(jdim_ig)
         write(*,*)' z_ig(1) =',z_ig(1)
         write(*,*)' z_ig(kdim_ig) =',z_ig(kdim_ig)
      end if
      if(L_write_int_grid_debug)then
! also write intermediate grid to a file
         call write_intermediate_grid(mjdtime,ierr)         
      end if
! write out max and min values of arrays
          call print_max(idim_ig,jdim_ig,kdim_ig,'       Bx(nT) ',bx_ig,x_ig,y_ig,z_ig)
          call print_min(idim_ig,jdim_ig,kdim_ig,'       Bx(nT) ',bx_ig,x_ig,y_ig,z_ig)
          call print_max(idim_ig,jdim_ig,kdim_ig,'       By(nT) ',by_ig,x_ig,y_ig,z_ig)
          call print_min(idim_ig,jdim_ig,kdim_ig,'       By(nT) ',by_ig,x_ig,y_ig,z_ig)
          call print_max(idim_ig,jdim_ig,kdim_ig,'       Bz(nT) ',bz_ig,x_ig,y_ig,z_ig)
          call print_min(idim_ig,jdim_ig,kdim_ig,'       Bz(nT) ',bz_ig,x_ig,y_ig,z_ig)
          call print_max(idim_ig,jdim_ig,kdim_ig,' pressure(Pa)',press_ig,x_ig,y_ig,z_ig)
          call print_min(idim_ig,jdim_ig,kdim_ig,' pressure(Pa)',press_ig,x_ig,y_ig,z_ig)
          call print_max(idim_ig,jdim_ig,kdim_ig,'  rho(kg/m^3)',rho_ig,x_ig,y_ig,z_ig)
          call print_min(idim_ig,jdim_ig,kdim_ig,'  rho(kg/m^3)',rho_ig,x_ig,y_ig,z_ig)
          call print_max(idim_ig,jdim_ig,kdim_ig,'    vel(km/s)',vel_ig,x_ig,y_ig,z_ig)
          call print_min(idim_ig,jdim_ig,kdim_ig,'    vel(km/s)',vel_ig,x_ig,y_ig,z_ig)

      RETURN
      END SUBROUTINE Bin

! -------------atran5--------------
      subroutine atran5(colat,aloct,x,y,z)

! converts colat and aloct to x,y,z
! 9/97 frt
! note: assumes zero tilt

      USE Rcm_mod_subs, ONLY : rprec
      implicit none
      real(rprec):: colat,aloct,x,y,z
      real(rprec):: grr,pi,dpi

      data grr /1.0/

      pi = acos(-1.0)

      dpi = pi/180.

      x = grr*cos(aloct*dpi)*sin(colat*dpi)
      y = grr*sin(aloct*dpi)*sin(colat*dpi)
      z = grr*cos(colat*dpi)

      return
      end

! 
!
!--------------------------------------------------
!
!
      SUBROUTINE Gettemp ( ierr)
!
! routine to compute an estimate for temperature given
! the pressure assume that alam*vm = energy = kt
! 2/00 frt
!
! inputs:
! idim,jdim - rcm grid dimensions
!       r,p - equatorial mapping location of a grid point in Re and rad
!     press - press in Pa
!        vm - flux tube volume^(-2/3) in (nt/Re/^(-2/3)   
!      open - open/closed flag (-1 = closed)
!       den - ple density in ple/m^3
! outputs:
!      ti,te - temperaure if ions and electrons in kelvin
!      ierr  - error flag
!
!-------------------------------------------------
      USE Rcm_mod_subs, ONLY : iprec,rprec
      USE conversion_module
      USE CONSTANTS, ONLY: boltz
      IMPLICIT NONE
!
      REAL(rprec), PARAMETER :: den0=0.5E+6
      REAL(rprec), PARAMETER :: tiote = 7.8
      REAL(rprec), PARAMETER :: fac = tiote/(1.*tiote)
      REAL(rprec), PARAMETER :: alam0 = 4000.0
      REAL(rprec), PARAMETER :: e = 1.6E-19
!     alam0 is the alam for the peak of the p
!
      INTEGER(iprec) :: i,j,ierr
!
!    set the temperature:

      DO j=1,jdim
       DO i=1,idim
        IF (open(i,j) == -1 .and. den(i,j)>0.) THEN
         ti(i,j) = fac*press(i,j)/den(i,j)/boltz
         te(i,j) = ti(i,j)/tiote
        ELSE
         ti(i,j) = 0.0
         te(i,j) = 0.0
        END IF
       END DO
      END DO
!
      ierr = 0
!
      RETURN
      END SUBROUTINE Gettemp
!
!===========================================================
!
      function fc(x,y,ymax,yc,rm)
! increases electric field by a factor rm in a channel
! yc wide on the nightside
      USE Rcm_mod_subs, ONLY : iprec,rprec
      implicit none
      real(rprec) :: y,x,ymax,yc,rm
      real(rprec) :: rn,b,fc,yt,ycn

      if(x > 0)then
        fc = y/ymax
        return
      end if

      if(abs(y) <= yc)then
        fc = rm*y/ymax
        return
      end if
      ycn = yc/ymax
      
      if(y < -yc .and. y<0) then
       b = -ycn*(1-rm)/(ycn-1)
       rn = (rm*ycn-1)/(ycn-1)
       fc = rn*y/ymax+b
      else if (y>=0 .and. y>yc)then
       b =  ycn*(1-rm)/(ycn-1)
       rn = (rm*ycn-1)/(ycn-1)
       fc = rn*y/ymax+b
      else
       fc = y/ymax
      end if
      return
      end

!===============================================================================

      subroutine dip(ps,x,y,z,bx,by,bz)

! calculates gsm components of geodipole field with the dipole moment
! corresponding to epoch 1980

! input parameters:
! ps - geodipole tilt angle in radians
! x,y,z - gsm coordinates in re

! output parameters:
! bx,by,bz - field components in gsm system in nanotesla

      use dipole_params
      USE Rcm_mod_subs, ONLY : iprec,rprec
      implicit none
      real(rprec) :: ps,psi,x,y,z,bx,by,bz
      real(rprec) :: sps,cps
      real(rprec) :: p,u,v,t,q
      integer(iprec) :: m

! f (m.eq.1.and.abs(ps-psi).lt.1.d-5) goto 1

      if(abs(dm)<= 1.0e-10)then
        write(6,*)' ERROR IN DIP'
        write(6,*)' dm =',dm
        stop
      end if

      sps=sin(ps)
      cps=cos(ps)
      psi=ps
      m=1

    1 p=x**2
      u=z**2
      v=3.*z*x
       t=y**2
! d0 added frt 3/95
! check for inside earth

      if(sqrt(p+u+t) < 1.0)then
        bx = 0.0
        by = 0.0
        bz = - 2.0*dm

      else

        q=dm/sqrt(p+t+u)**5

        bx=q*((t+u-2.0*p)*sps-v*cps)
        by=-3.0*y*q*(x*sps+z*cps)
        bz=q*((p+t-2.0*u)*cps-v*sps)

      end if

      return

      end subroutine dip
!===================================================================
      SUBROUTINE Set_ellipse(idim,jdim,rmin,pmin,vm,big_vm,bndloc,open)

! routine that fits an ellipse and resets the modeling
! boundary to be inside the ellipse
! 6/03 frt
! inputs:
!	idim,jdim - size of the 2d rcm arrays (lat, long)
!	xe,ye - equatorial mapping point of field line from rcm grid
!	vm - computed flux tube volume ^(-2/3) set to big_vm if open
!            or outside the ellipse boundary (output)
! 	bndloc/imin_j - boundary location (output)
!	a1 - dayside end of ellipse
!	a2 - nightside location of the ellipse
!	b - semi minor axis (y) of ellipse

      USE Rcm_mod_subs, ONLY : iprec,rprec
      implicit none
      integer(iprec) :: idim,jdim
      real(rprec) :: rmin(idim,jdim), pmin(idim,jdim)
      real(rprec) :: xe(idim,jdim), ye(idim,jdim)
      real(rprec) :: vm(idim,jdim)
      integer(iprec) :: open(idim,jdim)
      real(rprec) :: bndloc(jdim)
      real(rprec) :: big_vm,a1,a2,a,b,x0,ell
      integer(iprec) :: i,j
!  x0 = (a1 + a2)/2
!   a = (a1 - a2)/2
!
!                   b
!           |       |
!           |       |
! x<a1------0-------x0-------------a2
!           |       |
!           |       |
!                   b
!

      xe = rmin * cos(pmin)
      ye = rmin * sin(pmin)
! uses computed boundary
!     a1 = maxval(xe)
!     a2 = minval(xe)
!      b = maxval(abs(ye))
! uses set boundary
      a1 =  8.
      a2 = -10.0 ! -35.0 !-20.0 !-25.
      b = 10. ! 12.
      x0 = (a1 + a2)/2.
      a  = (a1 - a2)/2.

       do j=1,jdim
        do i=ceiling(bndloc(j)),idim-1
! now check to see if the point is outside the ellipse, if so
! reset open and bndloc        
         ell = ((xe(i,j)-x0)/a)**2+(ye(i,j)/b)**2
         if(ell > 1.) then
          bndloc(j) = i+1
          open(i,j) = 0
          vm(i,j) = big_vm
         end if
        end do
       end do 

      return
      end subroutine Set_ellipse
!===================================================================
      SUBROUTINE Set_bndy_w_mach(idim,jdim,rmin,pmin,vm,big_vm,bndloc,open)

! routine that adjusts the rcm boundary so that the lfm sonic mach number is
! less than some specified amount 
! 10/08 frt
! inputs:
!	idim,jdim - size of the 2d rcm arrays (lat, long)
!	xe,ye - equatorial mapping point of field line from rcm grid
!	vm - computed flux tube volume ^(-2/3) set to big_vm if open
!            or outside the ellipse boundary (output)
! 	bndloc/imin_j - boundary location (output)

      USE Rcm_mod_subs, ONLY : iprec,rprec
      USE constants, ONLY: gamma, mu0
      implicit none
      integer(iprec) :: idim,jdim
      real(rprec) :: rmin(idim,jdim), pmin(idim,jdim)
      real(rprec) :: xe(idim,jdim), ye(idim,jdim)
      real(rprec) :: vm(idim,jdim)
      integer(iprec) :: open(idim,jdim)
      real(rprec) :: bndloc(jdim)
      real(rprec) :: big_vm
      real(rprec) :: xpos,ypos,zpos,p1,p2,p3,press,rho,mach,vel,cs,cfast
      real(rprec) :: bxe,bye,bze,bbe
      real(rprec), parameter :: mach_max = 0.05
      real(rprec), parameter :: cs_min = 1.0e5 ! m/s
      real(rprec), parameter :: vel_min = 10.0e3 ! m/s
      integer(iprec) :: i,j,ierr

      xe = rmin * cos(pmin)
      ye = rmin * sin(pmin)

       do j=1,jdim
        do i=ceiling(bndloc(j)),idim-1
! get the pressure
        xpos = xe(i,j)
        ypos = ye(i,j)
        zpos = 0.0
        call bgrid(2,xpos,ypos,zpos,p1,p2,p3,ierr)
        press = p1 ! in pa
        call bgrid(3,xpos,ypos,zpos,p1,p2,p3,ierr)
        rho = p1 ! in kg/m^3
        call bgrid(5,xpos,ypos,zpos,p1,p2,p3,ierr)
        vel = 1000.*p1 ! in m/s

!       cs = max(cs_min,sqrt(gamma*press/max(1.e-30,rho))) ! in m/s
!       mach = vel/cs
! now get bfield in nt
         call bfield_lfm(xpos,ypos,zpos,bxe,bye,bze,bbe,ierr)
! fast mode speed in m/s
         cfast= sqrt((gamma*press+(1.0e-9*bbe)**2/mu0)/max(1.0e-30,rho))

         mach = vel/cfast

        if(mach > mach_max.and.vel>vel_min)then
! reset open and bndloc        
          vm(1:i,j) = big_vm
          open(1:i,j) = 0
          write(6,*)' In set bndy_w_mach'
          write(6,*)' cfast = ',cfast/1000.,&
                    ' km/s, vel = ',vel/1000.,' km/s'
          write(6,*)' mach = ',mach
          write(6,*)' old bndloc(',j,') =',bndloc(j),&
                    ' xe =',xe(bndloc(j),j),' ye =',ye(bndloc(j),j)
          bndloc(j) = i+1
          write(6,*)' new bndloc(',j,') =',bndloc(j)
        end if
        end do
       end do 

      return
      end subroutine Set_bndy_w_mach



!------------------------------------


      SUBROUTINE Smooth_eta_at_boundary(idim,jdim,kdim,jwrap,eeta,imin_j)
! this routine attempts to smooth out high frequency noise at the boundary
! of the rcm 
! written 2/06 frt
      USE Rcm_mod_subs, ONLY : iprec,rprec
      IMPLICIT NONE
      INTEGER(iprec) :: idim,jdim,kdim,jwrap
      INTEGER(iprec) :: imin_j(jdim)
      INTEGER(iprec) :: i,j,k,jm,jmm,jp,jpp
      REAL(rprec) :: eeta(idim,jdim,kdim)
      REAL(rprec) :: eetas2d(jdim,kdim)
! these are the smoothing weights
      REAL(rprec), PARAMETER :: a1 = 1.0  
      REAL(rprec), PARAMETER :: a2 = 1.0  
      REAL(rprec), PARAMETER :: a3 = 2.0  
      REAL(rprec), PARAMETER :: a4 = 1.0  
      REAL(rprec), PARAMETER :: a5 = 1.0  
! now do the smoothing

      do k=1,kdim
       do j=1,jdim

! 1 <=> jdim -jwrap +1
! jdim <=> jwrap
       jmm = j - 2
       if(jmm < 1)jmm = jdim - jwrap - 1       
       jm  = j - 1
       if(jm < 1) jm = jdim - jwrap       
       jpp = j + 2
       if(jpp > jdim)jpp = jwrap + 2
       jp  = j + 1
       if(jp > jdim) jp = jwrap + 1
! now smooth
       eetas2d(j,k) = &
          ( a1*eeta(imin_j(jmm),jmm,k) + &
            a2*eeta(imin_j(jm ),jm ,k) + &
            a3*eeta(imin_j(j  ),j  ,k) + &
            a4*eeta(imin_j(jp ),jp ,k) + &
            a5*eeta(imin_j(jpp),jpp,k) )/(a1+a2+a3+a4+a5) 

           end do
          end do
! now reset the boundary values
        do k=1,kdim
         do j=1,jdim
          eeta(imin_j(j),j,k) = eetas2d(j,k)
         end do      
        end do      
       return
END SUBROUTINE Smooth_eta_at_boundary 


      SUBROUTINE Smooth_boundary_location(idim,jdim,jwrap,bndloc)
      USE Rcm_mod_subs, ONLY : iprec,rprec
      USE rice_housekeeping_module
      IMPLICIT NONE
      INTEGER(iprec), INTENT(IN) :: idim,jdim,jwrap
      REAL(rprec), INTENT(IN OUT) :: bndloc(jdim)

      INTEGER(iprec) :: i,j,jp,jm
      REAL(rprec) :: bndloc_new(jdim)
      REAL(rprec), PARAMETER :: am = 1, a0 =2, ap = 1

     
      if (L_write_vars_debug) then
       write(6,*)' smooth_boundary_location, old boundary'
       write(6,*)bndloc
      end if
! 1 <=> jdim -jwrap +1
! jdim <=> jwrap
      do j=1,jdim
       jm  = j - 1
       if(jm < 1) jm = jdim - jwrap 
       jp  = j + 1
       if(jp > jdim) jp = jwrap + 1

       bndloc_new(j) = (am*bndloc(jm) + a0*bndloc(j) + ap*bndloc(jp))/(am+a0+ap)

      end do

      bndloc(:) = max(bndloc_new(:),bndloc(:))

      if (L_write_vars_debug) then
       write(6,*)' smooth_boundary_location, new boundary'
       write(6,*)bndloc
      end if

      return
    
      END SUBROUTINE Smooth_boundary_location
!
      subroutine set_plasmasphere(idim,jdim,kdim,rmin,pmin,vm,eeta,imin_j)
! crude routine to set a plasmasphere model in the rcm
! alam(1) should be set to a small value (0.01)
! 2/07 frt
      USE Rcm_mod_subs, ONLY : iprec,rprec
      IMPLICIT NONE

      integer(iprec) :: idim,jdim,kdim
      integer(iprec) :: imin_j(jdim)
      real(rprec) :: vm(idim,jdim),rmin(idim,jdim),pmin(idim,jdim)
      real(rprec) :: eeta(idim,jdim,kdim)

      integer(iprec) :: i,j,k
      real(rprec), parameter :: radius_ps = 5.0
      real(rprec), parameter :: dens_ps = 10.0e6 ! ple/m^3

      do j=1,jdim
       do i=imin_j(j),idim
        if(rmin(i,j) < radius_ps .and. vm(i,j) > 0.0)then
        eeta(i,j,1) = dens_ps/(1.5695e-16*vm(i,j)**1.5)
        end if
       end do
      end do

      return

end subroutine set_plasmasphere

!-------------------------------------
      subroutine print_max(nx,ny,nz,label,array,x,y,z)
! routine to find, and print the max value of a quantity on a 3D array
! and print its location and value
      USE Rcm_mod_subs, ONLY : iprec,rprec
      IMPLICIT NONE
      INTEGER(iprec), INTENT(IN) :: nx,ny,nz
      REAL(rprec), INTENT(IN) :: x(nx),y(ny),z(nz)
      REAL(rprec), INTENT(IN) :: array(nx,ny,nz)
      CHARACTER(*), INTENT(IN) :: label

      INTEGER(iprec) :: location(3)
      REAL(rprec) :: max_array
!     INTEGER :: inlen

      max_array = maxval(array)
      location = maxloc(array)
!     inlen = 1 + int(log10(real(max(nx,ny,nz)))) ! length of the arrays

      write(6,'(a,a,a,g12.6,a,i4,a,f10.3,a,i4,a,f10.3,a,i4,a,f10.3)') &
                   '  max value of ',label,' is ',max_array,&
                   ' at: x(',location(1),')=',x(location(1)),&
                   ', y(',location(2),')=',y(location(2)),&
                   ', z(',location(3),')=',z(location(3))

      return
      end subroutine print_max
!-------------------------------------
      subroutine print_max_2d(nx,ny,label,array,x,y)
! routine to find, and print the max value of a quantity on a 3D array
! and print its location and value
      USE Rcm_mod_subs, ONLY : iprec,rprec
      IMPLICIT NONE
      INTEGER(iprec), INTENT(IN) :: nx,ny
      REAL(rprec), INTENT(IN) :: x(nx),y(ny)
      REAL(rprec), INTENT(IN) :: array(nx,ny)
      CHARACTER(*), INTENT(IN) :: label

      INTEGER(iprec) :: location(2)
      REAL(rprec) :: max_array
!     INTEGER :: inlen

      max_array = maxval(array)
      location = maxloc(array)
!     inlen = 1 + int(log10(real(max(nx,ny,nz)))) ! length of the arrays

      write(6,'(a,a,a,g12.6,a,i4,a,f10.3,a,i4,a,f10.3,a,i4,a,f10.3)') &
                   '  max value of ',label,' is ',max_array,&
                   ' at: x(',location(1),')=',x(location(1)),&
                   ', y(',location(2),')=',y(location(2))

      return
      end subroutine print_max_2d
!-------------------------------------
      subroutine print_max_integer(nx,ny,nz,label,array,x,y,z)
! routine to find, and print the max value of a quantity on a integer 3D array
! and print its location and value
      USE Rcm_mod_subs, ONLY : iprec,rprec
      IMPLICIT NONE
      INTEGER(iprec), INTENT(IN) :: nx,ny,nz
      REAL(rprec), INTENT(IN) :: x(nx),y(ny),z(nz)
      INTEGER(iprec), INTENT(IN) :: array(nx,ny,nz)
      CHARACTER(*), INTENT(IN) :: label

      INTEGER(iprec) :: location(3)
      INTEGER(iprec) :: max_array
!     INTEGER :: inlen

      max_array = maxval(array)
      location = maxloc(array)
!     inlen = 1 + int(log10(real(max(nx,ny,nz)))) ! length of the arrays

      write(6,'(a,a,a,i5,a,i4,a,f10.3,a,i4,a,f10.3,a,i4,a,f10.3)') &
                   '  max value of ',label,' is ',max_array,&
                   ' at: x(',location(1),')=',x(location(1)),&
                   ', y(',location(2),')=',y(location(2)),&
                   ', z(',location(3),')=',z(location(3))

      return
      end subroutine print_max_integer
!-------------------------------------
      subroutine print_min(nx,ny,nz,label,array,x,y,z)
! routine to find, and print the min value of a quantity on a 3D array
! and print its location and value
      USE Rcm_mod_subs, ONLY : iprec,rprec
      IMPLICIT NONE
      INTEGER(iprec), INTENT(IN) :: nx,ny,nz
      REAL(rprec), INTENT(IN) :: x(nx),y(ny),z(nz)
      REAL(rprec), INTENT(IN) :: array(nx,ny,nz)
      CHARACTER(*), INTENT(IN) :: label

      INTEGER(iprec) :: location(3)
      REAL(rprec) :: min_array
!     INTEGER :: inlen

      min_array = minval(array)
      location = minloc(array)
!     inlen = 1 + int(log10(real(max(nx,ny,nz)))) ! length of the arrays

      write(6,'(a,a,a,g12.6,a,i4,a,f10.3,a,i4,a,f10.3,a,i4,a,f10.3)') &
                   '  min value of ',label,' is ',min_array,&
                   ' at: x(',location(1),')=',x(location(1)),&
                   ', y(',location(2),')=',y(location(2)),&
                   ', z(',location(3),')=',z(location(3))

      return     
      end subroutine print_min
!-------------------------------------
      subroutine print_min_2d(nx,ny,label,array,x,y)
! routine to find, and print the min value of a quantity on a 3D array
! and print its location and value
      USE Rcm_mod_subs, ONLY : iprec,rprec
      IMPLICIT NONE
      INTEGER(iprec), INTENT(IN) :: nx,ny
      REAL(rprec), INTENT(IN) :: x(nx),y(ny)
      REAL(rprec), INTENT(IN) :: array(nx,ny)
      CHARACTER(*), INTENT(IN) :: label

      INTEGER(iprec) :: location(2)
      REAL(rprec) :: min_array
!     INTEGER :: inlen

      min_array = minval(array)
      location = minloc(array)
!     inlen = 1 + int(log10(real(max(nx,ny,nz)))) ! length of the arrays

      write(6,'(a,a,a,g12.6,a,i4,a,f10.3,a,i4,a,f10.3,a,i4,a,f10.3)') &
                   '  min value of ',label,' is ',min_array,&
                   ' at: x(',location(1),')=',x(location(1)),&
                   ', y(',location(2),')=',y(location(2))

      return     
      end subroutine print_min_2d
!-------------------------------------
      subroutine print_min_integer(nx,ny,nz,label,array,x,y,z)
! routine to find, and print the max value of a quantity on a 3D array
! and print its location and value
      USE Rcm_mod_subs, ONLY : iprec,rprec
      IMPLICIT NONE
      INTEGER(iprec), INTENT(IN) :: nx,ny,nz
      REAL(rprec), INTENT(IN) :: x(nx),y(ny),z(nz)
      INTEGER(iprec), INTENT(IN) :: array(nx,ny,nz)
      CHARACTER(*), INTENT(IN) :: label

      INTEGER(iprec) :: location(3)
      INTEGER(iprec) :: min_array
!     INTEGER :: inlen

      min_array = minval(array)
      location = minloc(array)
!     inlen = 1 + int(log10(real(max(nx,ny,nz)))) ! length of the arrays

      write(6,'(a,a,a,i5,a,i4,a,f10.3,a,i4,a,f10.3,a,i4,a,f10.3)') &
                   '  min value of ',label,' is ',min_array,&
                   ' at: x(',location(1),')=',x(location(1)),&
                   ', y(',location(2),')=',y(location(2)),&
                   ', z(',location(3),')=',z(location(3))

      return
      end subroutine print_min_integer
!------------------------------------------      
      subroutine reset_rcm_vm(idim,jdim,bndloc,big_vm,imin_j,vm,open)
! this routine resets imin_j, vm, and open based on a newly set bndloc      
      USE Rcm_mod_subs, ONLY: rprec,iprec
      implicit none
      integer(iprec), intent(in) :: idim,jdim
      integer(iprec) :: i,j
      integer(iprec),intent(inout) :: imin_j(jdim),open(idim,jdim)
      real(rprec), intent(in) :: bndloc(jdim)
      real(rprec), intent(in) :: big_vm
      real(rprec), intent(inout) :: vm(idim,jdim)

       imin_j = CEILING(bndloc)

       do j=1,jdim
        do i=1,imin_j(j)-1
          vm(i,j) = big_vm
          open(i,j) = 0
          end do
       end do
      return

      end subroutine reset_rcm_vm
!======================================
      subroutine allocate_conversion_arrays(isize,jsize,kcsize)
! used to allocate memory for the exchange arrays      
! 7/09 frt
      use conversion_module
      use rcm_mod_subs, only : iprec
      implicit none
      integer(iprec),intent(in) :: isize,jsize,kcsize
! if the arrays are allocated, then return
      if(allocated(x0))return

      idim = isize
      jdim = jsize
      kdim = kcsize

      ! 1d arrays
      allocate(imin_j_old(jdim))
      allocate(inner_bndy(jdim))
      allocate(bndloc_old(jdim))
      allocate(almmin(kdim))
      allocate(almmax(kdim))
      allocate(almdel(kdim))
      ! 2d arrays
      allocate(x0(idim,jdim))
      allocate(y0(idim,jdim))
      allocate(z0(idim,jdim))
      allocate(dir0(idim,jdim))
      allocate(den(idim,jdim))
      allocate(press(idim,jdim))
      allocate(deno(idim,jdim))
      allocate(presso(idim,jdim))
      allocate(te(idim,jdim))
      allocate(ti(idim,jdim))
      allocate(to(idim,jdim))
      allocate(beta_average(idim,jdim))
      allocate(eetabnd(jdim,kdim))
      allocate(open(idim,jdim))
      ! 3d arrays
      allocate(eeta_new(idim,jdim,kdim))
    
     return

     end subroutine allocate_conversion_arrays 

      SUBROUTINE write_intermediate_grid (mjdtime,ierr)
!=========================================================
! this routine spits out the intermediate grid arrays 
! for debugging pruposes      
! modified to use modules 7/09 frt
! updated 8/09 frt      
!=========================================================
      USE intermediate_grid
      USE dipole_params, ONLY : dm
      USE Rcm_mod_subs, ONLY: iprec,rprec
      USE CONSTANTS, ONLY : radius_earth_m
      IMPLICIT NONE

      INTEGER(iprec) :: ierr
      REAL(rprec) :: dmin
      REAL*8 :: mjdtime
      CHARACTER*(21) :: intermediate_file ='intermediate_grid.dat'

! convert dipole moment to nt/re^3
      dmin = dm*(radius_earth_m**3)/1.0e+9
! read in timestamp from MHD file
      open(91,file=intermediate_file,form='UNFORMATTED',status='unknown')

        WRITE(91,err=2002)MJDTIME
        WRITE(91,err=2002)dmin,idim_ig,jdim_ig,kdim_ig
        WRITE(*,*)' idim,jdim,kdim =',idim_ig,jdim_ig,kdim_ig
        WRITE(91,err=2002)x_ig*radius_earth_m,&
                          y_ig*radius_earth_m,z_ig*radius_earth_m ! in m
        WRITE(91,err=2002)rho_ig ! in kg/m^3
        WRITE(91,err=2002)press_ig !  Pa
! since it is converted int to nT, convert back to T
        WRITE(91,err=2002)bx_ig * 1.0e-9 
        WRITE(91,err=2002)by_ig * 1.0e-9
        WRITE(91,err=2002)bz_ig * 1.0e-9
        WRITE(91,err=2002)imask_ig

      GO TO 2001

 2002 ierr = -1
      write(6,*)' (2002) write error in torcm'
      STOP 'stopping in write_intermediate_grid'
      RETURN

 2001 CLOSE (91)

      if(ierr.lt.0)return

      end subroutine write_intermediate_grid

