#include <cassert>
#include <iostream>
#include "tinyxml.h"

int main(int argc, char ** argv)
{
  TiXmlDocument doc ("SAMPLE_FILE.xml" );
  
  /* Check that we can load the file properly. */
  assert(doc.LoadFile());

  /* Find xml_file in the document */
  TiXmlElement *xml_file = doc.FirstChildElement( "xml_file" );

  /* Find wtf this file should be */
  TiXmlElement *wtf = xml_file->FirstChildElement( "wtf" );
  assert(wtf != NULL);
  assert(strcmp(const_cast<char *>(wtf->GetText()), "Sample XML file for integration test of TinyXML Reader") == 0);


  /* Now we're going to loop through all the components in this file */
  TiXmlElement *components = xml_file->FirstChildElement( "components" );
  TiXmlNode *comps=0;

  TiXmlNode *id, *description, *value, *invalid_data;
  TiXmlElement *ID, *DESCRIPTION, *VALUE;
  const char *id_str, *description_str, *value_str;
  
  for( comps = components->FirstChild( "component" );
       comps;
       comps = comps->NextSibling( "component" ) )
    {
      id = comps->FirstChild("id");
      if (id != NULL){
	ID = id->ToElement();
	
	assert((strcmp(ID->Attribute("value"), "0") == 0) || 
	       (strcmp(ID->Attribute("value"), "10000") == 0) );
	id_str = ID->GetText();
	assert((strcmp(id_str, "NSTART") == 0) ||
	       (strcmp(id_str, "NSTOP") == 0));
	id = NULL;
      }
      description = comps->FirstChild("description");
      if (description != NULL){
	DESCRIPTION = description->ToElement();
	description_str = DESCRIPTION->GetText();
	assert((strcmp(description_str, "Initial Step of this segment") == 0) ||
	       (strcmp(description_str, "Final Step of this segment") == 0));
	description = NULL;
      }
      
      value = comps->FirstChild("value");
      if (value != NULL){
	VALUE = value->ToElement();
	value_str = VALUE->GetText();
	assert((strcmp(value_str, "0") == 0) ||
	       (strcmp(value_str, "10000") == 0));
	value = NULL;
      }

      invalid_data = comps->FirstChild("Not a value");
      assert(invalid_data == NULL);
    }

  std::cout << "Success!\n";
  return 0;
}
