#ifndef DATA_INPUT_TEST_H_
#define DATA_INPUT_TEST_H_

#include <cxxtest/TestSuite.h>

#include "ASCII_data.h"
#include "HDF_data.h"

class DataInputTest : public CxxTest::TestSuite
{
 public:
  void xtestOpenInvalidDataFile( void )
  {
    HDF_data<float> input_hdf;
    TS_ASSERT(input_hdf.open("zzz123xyz") == false);

    ASCII_data<float> input_ascii;
    TS_ASSERT(input_ascii.open("zyx321zzz") == false);
  }

  // 1-dimensional ASCII tests
  void testOpenAndRead1dData( void )
  {
    ASCII_data<float> input;
    TS_ASSERT(input.open("../../../../MIX/test/LFM-MIX/output/ej.dat"));
    int n_ijk[MAX_VAR_DIMS];
    int nDims = input.getDimensions("0", n_ijk);
    
    // All columns in an ASCII file should be 1-d
    TS_ASSERT_EQUALS(nDims, 1);
    TS_ASSERT_EQUALS(n_ijk[0], 792);

    int nElements = 1;
    for (int i = 0; i < nDims; i++){
      nElements *= n_ijk[i];
    }

    float *data = NULL;
    data = new float[nElements];
    TS_ASSERT(data != NULL);
    TS_ASSERT(input.getData("0", data));    

    delete[] data;
  }

  ///////////////////////////////////////
  // 2-dimensional (LFM ION HDF) tests //
  ///////////////////////////////////////

  void testOpenAndRead2dData( void )
  {
    HDF_data<float> input("../../../../runs/LFM/timesteps-short/BaseRun_ion_0080005.dmp");
    int n_ijk[MAX_VAR_DIMS];
    int nDims = input.getDimensions("potnorth", n_ijk);

    TS_ASSERT_EQUALS(nDims, 2);
    TS_ASSERT_EQUALS(n_ijk[0], 19);
    TS_ASSERT_EQUALS(n_ijk[1], 25);
   
    int nElements = 1;
    for (int i = 0; i < nDims; i++){
      nElements *= n_ijk[i];
    }

    float *data = NULL;
    data = new float[nElements];
    TS_ASSERT(data != NULL);
    TS_ASSERT(input.getData("potnorth", data));    

    delete[] data;
  }

  ///////////////////////////////////////
  // 3-dimensional (LFM MHD HDF) tests //
  ///////////////////////////////////////
  
  void testOpenAndRead3dData( void )
  {
    HDF_data<float> input;
    TS_ASSERT(input.open("../../../../runs/LFM/timesteps-short/BaseRun_mhd_0080005.dmp"));
    int n_ijk[MAX_VAR_DIMS];
    int nDims = input.getDimensions("bz_", n_ijk);

    TS_ASSERT_EQUALS(nDims, 3);
    TS_ASSERT_EQUALS(n_ijk[0], 32+1);
    TS_ASSERT_EQUALS(n_ijk[1], 24+1);
    TS_ASSERT_EQUALS(n_ijk[2], 53+1);

    int nElements = 1;
    for (int i = 0; i < nDims; i++){
      nElements *= n_ijk[i];
    }
    TS_ASSERT_EQUALS( nElements, (53+1)*(24+1)*(32+1) );

    float *data = NULL;
    data = new float[nElements];
    TS_ASSERT(data != NULL);
    TS_ASSERT(input.getData("bz_", data));    
    
    delete[] data;
  }

  /////////////////////////////////////////
  // 4-dimensional (TIEGCM NetCDF) tests //
  /////////////////////////////////////////
  void testOpenAndRead4dData( void )
  {
    HDF_data<double> input;
    TS_ASSERT(input.open("../../../../runs/TIEGCM/tiegcm1.9_data/TGCM.tiegcm1.9_d80.peqnx001.nc"));
    int n_ijk[MAX_VAR_DIMS];
    int nDims = input.getDimensions("TN", n_ijk);

    TS_ASSERT_EQUALS(nDims, 4);
    TS_ASSERT_EQUALS(n_ijk[0], 1);
    TS_ASSERT_EQUALS(n_ijk[1], 29);
    TS_ASSERT_EQUALS(n_ijk[2], 36);
    TS_ASSERT_EQUALS(n_ijk[3], 72);

    int nElements = 1;
    for (int i = 0; i < nDims; i++){
      nElements *= n_ijk[i];
    }
    TS_ASSERT_EQUALS( nElements, (1)*(29)*(36)*(72) );

    double *data = NULL;
    data = new double[nElements];
    TS_ASSERT(data != NULL);
    TS_ASSERT(input.getData("TN", data));    
    
    delete[] data;
  }
};

#endif /*HDF_DATA_TEST_H_*/
