"""
Functions that help get user input to Environment parameters.
"""
from MakeItSo.Environments import Environment
from MakeItSo import share
import sys

def stdin(include_optional, include_expert, has_InterComm = True):
    """
    Select a run environment & input its parameters

    Parameters:
      include_optional:  When true, query user to input optional variables
      include_expert: When true, query user to input expert variables
      has_InterComm:  When true, the user needs to input InterComm & PVM variables

    Returns:
      env - Environment object (i.e. SH, SGE, LSF, etc.)
    """
    env = Environment()
    input_parameter_values_from_stdin(env, include_optional, include_expert, has_InterComm)
        
    return env

def input_parameter_values_from_stdin(env, include_optional, include_expert, has_InterComm = True):
    """
    Query the user to select an environment & then enter all
    required input parameters via stdin input

    Parameters:
      env:  Environment object to get/set variables
      include_optional:  When true, query user to input optional variables
      include_expert: When true, query user to input expert variables
      has_InterComm:  When true, the user needs to input InterComm & PVM variables
    """
    # Determine out what queuing environment to use.
    options = [e['__metadata__']['environment_description'] + ' ' + e["__metadata__"]["environment_name"] for e in env.environments]
    selection = share.stdin_select('Environment', options)
    
    # Select the chosen environment
    env.selectEnvironment( selection )

    print ""
    print "Input queueing variables for " + env.metadata["environment_name"] + " (? for help)"
    print "---------------------------------"

    for variable in env.parameters["flags"]:
        variable_name = variable.keys()[0]
        variable_dict = variable[variable_name]
        
        if ((variable_dict["user_input"] == "required") |
            ((variable_dict["user_input"] == "optional") & (include_optional | include_expert))|
            ((variable_dict["user_input"] == "expert_only") & include_expert) ):
            # Input Variable
            variable_dict["value"] = share.stdin_get_variable(variable_name, variable_dict)
        elif ((variable_dict["user_input"] == "automated") |
              (variable_dict["user_input"] == "ignore")):
            # Automated variables are set elsewhere (grep
            # Environment.py for "automated"), and ignored variables
            # should not be set via user input....
            continue
        else:
            raise Exception, ("Error: Misunderstud user_input type: \"" +
                              variable_dict["user_input"] + "\".")

    is_header_type_printed = False
    
    for variable in env.parameters["environment"]:
        variable_name = variable.keys()[0]
        variable_dict = variable[variable_name]

        # Input PVM variables only if coulped via InterComm.
        if variable_dict.has_key("pvm_only") :
            if ((variable_dict["pvm_only"]) & (not has_InterComm) ):
                continue

        if  ((variable_dict["user_input"] == "required") |
             ((variable_dict["user_input"] == "optional") & (include_optional | include_expert))|
             ((variable_dict["user_input"] == "expert_only") & include_expert) ):
            # Input standard variables...

            if not is_header_type_printed:
                print ""
                print "Input environment variables for " + env.metadata["environment_name"] + " (? for help)"
                print "---------------------------------"
                is_header_type_printed = True
            # Input variable
            variable_dict["value"] = share.stdin_get_variable(variable_name, variable_dict)

        elif (variable_dict["user_input"] == "ignore"):
            # Ignored variables are not set via User Input.
            continue

    print ""
