#FIXME:  unit tests write a "rcm_lfm.param" file in the makeItSo.py/test.py directory! 
from Model import Model

import datetime 
import os
import shutil
import unittest

class RCM(Model):
    """ RCM Model Type """

    def __init__(self):
        """
        Instantiate model/parameter type
        """
        filename = os.path.join(os.path.dirname(__file__), "RCM.yml")
        Model.__init__(self, filename)

        assert( self.metadata["model_name"] == "RCM" )

        self.output_directory = ""

    def get_model_setup(self, model_name, has_spinup, run_identifier, start_datetime):
        shScript = ['# RCM']
        execName = self.get_run_instructions(model_name, 0)[1]
        shScript.append('cp ${LTRROOT}/build/bin/%s .' % (execName))
        shScript.append('')

        instructions = []

        return ('\n'.join(shScript), '\n'.join(instructions))

    def get_iteration_setup(self, model_name, iteration):
        """
        Setup environment to run a model, ends with newline when not empty

        RCM requires a few files:
          - Coupling parameters: rcm_lfm.param
          - RCMfiles subdirectory containing:
             - rcm.params namelist
             - enchan.dat (defines constants)
             - dktable (defines constants)
        """

        pathRCMfiles = os.path.join(self.output_directory, 'RCMfiles')
        if not os.path.exists(pathRCMfiles):
            os.mkdir(pathRCMfiles)

        ########################################################################
        # RCMfiles/rcm.params
        namelist = ["friction code run"]
        for variable in self.parameters["rcm.params"]:
            variable_name = variable.keys()[0]
            variable_dict = variable[variable_name]
            namelist.append(" " + str(variable_dict["value"]) + "\t" + str(variable_name) + "\t" + "! " + str(variable_dict["description"]))
            
        rcm_lfm_param = open(os.path.join(pathRCMfiles, "rcm.params"), "w")
        rcm_lfm_param.write('\n'.join(namelist))
        rcm_lfm_param.close()

        ########################################################################        
        # RCMfiles/enchan.dat
        pathEnchanDat = os.path.join(os.path.dirname(__file__), 'RCM_enchan.dat')
        shutil.copy2( pathEnchanDat, os.path.join(pathRCMfiles, 'enchan.dat') )

        ########################################################################
        # RCMfiles/dktable
        pathDktable = os.path.join(os.path.dirname(__file__), 'RCM_dktable')
        shutil.copy2( pathDktable, os.path.join(pathRCMfiles, 'dktable') )

        if (model_name == "LFM-RCM"):
            ####################################################################
            # RCM-rect.dat
            pathRCMrectDat = os.path.join(os.path.dirname(__file__), 'RCM_RCM-rect.dat')
            shutil.copy2( pathRCMrectDat, os.path.join(self.output_directory, 'RCM-rect.dat') )

            ####################################################################
            # rcm_lfm.param file
            namelist = ""
            for variable in self.parameters["rcm_lfm.param"]:
                variable_name = variable.keys()[0]
                variable_dict = variable[variable_name]
                namelist += " " + str(variable_dict["value"]) + "\t" + variable_name + "\n"

            rcm_lfm_param = open(os.path.join(self.output_directory, "rcm_lfm.param"), "w")
            rcm_lfm_param.write(namelist)
            rcm_lfm_param.close()
        else:
            raise Exception, "Do not understand coupled model \"" + model_name + "\"."
        
        return ""
    
    def get_iteration_teardown(self, model_name, iteration):
        """
        Teardown environment to run a model, ends with newline when not empty
        """        
        return ""

    def get_run_instructions(self, model_name, iteration):
        """
        Get the executable, inputs & outputs needed to run the model.

        For example usage, see 'test_get_run_instructions'.
        
        Note: Required function (standardized parameters & return variables)        

        Returns:
          boolean:  True when parallel
          executable:  Name of executable to run
          flags:  any command-line arguments
          stdin:  where to redirect stdin input from
          stdout_and_stderr:  Where to redirect stdout/stderr to?
        """
        executable = "RCM-LFM"
        flags = ""
        stdin = "/dev/null"
        stdout_and_stderr = "RCM-LFM-%03d.out" % (iteration)

        return (False, executable, flags, stdin, stdout_and_stderr)

    def get_input_file(self):
        """
        Get contents of [] input parameter file

        Returns:
          str_script: String of job script

        Note: Required function (standardized parameters & return variables)
        """
        None

    def set_shared_parameters(self, coupled_model_name, shared_model, env=None):
        """
        Note: Required function (standardized parameters & return variables)
        """
        self.output_directory = shared_model.get_variable_dict("SHARED", "Output directory")["value"]

        if (coupled_model_name == "LFM-RCM"):
            self.get_variable_dict("rcm_lfm.param","xjd_filename")["value"] = "lfm-mix-rcm.xjd"
        else:
            raise Exception, "Do not understand coupled model \"" + coupled_model_name + "\"."

        None

    def set_time_information(self, start_datetime, stop_datetime):
        """
        Take care of any special behavior for start/stop times for a run.
        
        Parameters:
          start_datetime: datetime object specifying the starting
            year-month-day-hour-minute-second for this segment.
          stop_datetime:  Unused.  Stop time determined by LFM kill signal.

        Note: Required function (standardized parameters & return variables)
        """
        None

class TestRCM(unittest.TestCase):
    def setUp(self):
        import tempfile
        
        self.params = RCM()
        self.params.output_directory = tempfile.mkdtemp()

    def test_get_model_setup(self):
        (shScript, instructions) = self.params.get_model_setup('LFM-RCM',
                                                               has_spinup=True,
                                                               run_identifier='test',
                                                               start_datetime=datetime.datetime(year=2000,month=3,day=14,hour=3,minute=23,second=int(53.60496)))
        # start date/time is pi day (and time! 0.14159265 percentage of the day => 3:23:53.60496)
        self.assertEqual(shScript,"""# RCM\ncp ${LTRROOT}/build/bin/RCM-LFM .\n""")
        self.assertEqual(instructions,'')            

    def test_get_iteration_setup(self):
        # Comment the following assertEqual test out to avoid writing
        # "rcm_lfm.param" file any time unit tests are executed.        
        self.assertEqual(self.params.get_iteration_setup("LFM-RCM", 2), "")
        self.assertRaises(Exception, self.params.get_iteration_setup, "Purple-Monkey-Dishwasher", 0)

        import filecmp

        # RCM_enchan.date
        self.assertTrue(filecmp.cmp(os.path.join(os.path.dirname(__file__), 'RCM_enchan.dat'),
                                    os.path.join(self.params.output_directory, 'RCMfiles', 'enchan.dat'))
                        )
                             
        # RCM_dktable
        self.assertTrue(filecmp.cmp(os.path.join(os.path.dirname(__file__), 'RCM_dktable'),
                                    os.path.join(self.params.output_directory, 'RCMfiles','dktable'))
                        )

        # Does "rcm.params" exist?
        self.assertTrue(os.path.exists(os.path.join(self.params.output_directory, 'RCMfiles', 'rcm.params')))
        
    def test_get_iteration_teardown(self):
        self.assertEqual(self.params.get_iteration_teardown("LFM-RCM", 2), "")

    def test_get_run_instructions(self):
        (isParallel, executable, flags, stdin, stdout_and_stderr) = self.params.get_run_instructions("LR", 2)
        self.assertFalse(isParallel)
        self.assertEqual(executable, "RCM-LFM")
        self.assertEqual(flags, "")
        self.assertEqual(stdin, "/dev/null")
        self.assertEqual(stdout_and_stderr, "RCM-LFM-002.out")        

    def test_set_shared_parameters(self):
        self.assertRaises(Exception,   self.params.set_shared_parameters,"LFM-RCM",None)

if __name__ == '__main__':
    import doctest
    doctest.testmod()
    unittest.main()
