from Model import Model
from MakeItSo import Release

import datetime
import math
import os
import sys
import unittest

##########################################################################################

class Shared(Model):
    """
    Shared model parameters
    """
    def __init__(self):
        filename = os.path.join(os.path.dirname(__file__), "Shared.yml")
        Model.__init__(self, filename)
        # Make sure the YAML file is for shared model parameters
        assert (self.metadata["model_name"] == "Shared" )

    def is_Coupled_Via_InterComm(self, coupled_model_name):
        """
        Check if USE_INTERCOMM is enabled if coupled_model_name is a
        coupled model that is capable of utilizing InterComm.

        Parameters:
          coupled_model_name:  name of coupled model (e.g. LFM-MIX, CMIT, etc.)
        """
        models = self.get_variable_dict("SHARED", "USE_INTERCOMM")["models"]

        for model in models.split(","):
            if (model.strip() == coupled_model_name):
                return bool(self.get_variable_dict("SHARED","USE_INTERCOMM")["value"])

        # if we made it this far, we're not using InterComm.
        return False

class TestShared(unittest.TestCase):
    def setUp(self):
        self.SharedModelParameters = Shared()

    def test_is_Coupled_Via_InterComm(self):
        self.assertEqual(self.SharedModelParameters.is_Coupled_Via_InterComm("LFM-MIX"), True)
        self.assertEqual(self.SharedModelParameters.is_Coupled_Via_InterComm("CMIT"), True)
        self.assertEqual(self.SharedModelParameters.is_Coupled_Via_InterComm("LFM"), False)
        self.assertEqual(self.SharedModelParameters.is_Coupled_Via_InterComm("TIEGCM"), False)

if __name__ == '__main__':
    import doctest
    doctest.testmod()
    unittest.main()
