#!/usr/bin/env python
"""
run `qsub_stic.rice.edu.py --help` for documentation, or see parseArguments
docstring, below.

Author: Peter Schmitt (schmitt <at> ucar.edu)
Date: February 22, 2010

Example:
>>> qsub_stic.rice.edu.py LTR-001.qsub
Issuing command 'qsub LTR-001.qsub'
Issuing command 'qsub -W depend=afterok:64092.sticman.stic.rice.edu LTR-002.qsub'
Issuing command 'qsub -W depend=afterok:64093.sticman.stic.rice.edu LTR-003.qsub'
Issuing command 'qsub -W depend=afterok:64094.sticman.stic.rice.edu LTR-004.qsub'
Issuing command 'qsub -W depend=afterok:64095.sticman.stic.rice.edu LTR-005.qsub'
Issuing command 'qsub -W depend=afterok:64096.sticman.stic.rice.edu LTR-006.qsub'
Issuing command 'qsub -W depend=afterok:64097.sticman.stic.rice.edu LTR-007.qsub'
Issuing command 'qsub -W depend=afterok:64098.sticman.stic.rice.edu LTR-008.qsub'
Issuing command 'qsub -W depend=afterok:64099.sticman.stic.rice.edu LTR-009.qsub'
"""

import glob
import optparse
import os
import re
import sys
import textwrap

def parseArguments():
    """
    This script forces multiple jobs to run in sequence on the STIC
    cluster at Rice University [http://rcsg.rice.edu/stic]
    
    This only works with jobs named sequentially in the format
       prefix-###.suffix
    where ### are 3 integer numbers.  Conveniently, this is the
    formatting used by the LTR MakeItSo script generator!
    
    This script implements the idea at
    http://rcsg.rice.edu/rcsg/shared/afterok.html
    
    arguments: 
      jobscript.qsub:  The first job script you wish to submit."""

    desc = parseArguments.__doc__
    
    parser = optparse.OptionParser()
    parser.set_usage('%prog [options] jobscript.qsub\n' + desc)
    
    (options, args) = parser.parse_args()
    
    if len(args) <= 0:
        print parser.print_help()
        sys.exit()

    return args[0]

def parseScriptName(filename):
    """
    Get the prefix & suffix for a job with the naming convention
          prefix-###.suffix
    where ### are 3 integer numbers.
    """
    r = re.match(r'([\S]+)-\d{3}([\S]+)', filename)
    if not r:
        print 'Unmatched regex for %s.  Did you specify the right script name?' % filename

    assert( len(r.groups()) == 2)
    prefix = r.groups()[0]
    suffix = r.groups()[1]

    return (prefix, suffix)

def qsub(filename, afterOk=None):
    """

    Submit a file to the queue:
        qsub filename

    When afterOk is set to a Moab/Torque jobId, this will submit the
    next script in the iteration via:
        qsub -W depend=afterok:[afterOk] filename
    """
    command = ''
    if afterOk:
        command = 'qsub -W depend=afterok:%s %s' % (afterOk.strip(), filename)
    else:
        command = 'qsub %s' % (filename)

    print 'Issuing command "%s"' % command
    p = os.popen(command)
    lines = p.readlines()
    assert( len(lines) == 1)

    return lines[0]
    

if __name__ == '__main__':
    submitScriptName = parseArguments()
    (prefix, suffix) = parseScriptName(submitScriptName)
    
    files = glob.glob(prefix+'*'+suffix)
    files.sort();

    jobId = None
    for filename in files:
        if filename == submitScriptName:
            # Submit first job         
            jobId = qsub(filename, afterOk=jobId)
        elif jobId:
            # Submit subsequent jobs that follow the first
            jobId = qsub(filename, afterOk = jobId)
            
