program rcm_lfm

  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ! Driver program that runs Virtual RCM and communicates with Virtual
  ! MIX and Virtual LFM.
  !
  ! Uses InterComm (v1.6.3) xjd-based low low-level API
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  use intercomm_module
  use intermediate_grid, only : setupIg, tearDownIg
  use ionosphere_intermediate, only : setupIon, tearDownIon

  implicit none

  !> Coupling Variables
  integer :: iaScalars(NUMBER_OF_SCALARS) !> Scalars received from LFM via ReceiveScalars
  integer :: iaUt(6)   !> Current date/time scalar array
  integer :: exchangeNum = 0

  logical :: isFirstExchange
  logical :: isLastExchange

  call setupIg()
  call setupIon()
  call Initialize() ! implemented in intercomm.f90

  ! MHD-Coupler-Inner Magnetosphere coupled time loop using LFM-MIX-RCM.
  ! Note there are three kinds of exchanges:
  !   1.  First Exchange:  Import data from MHD & Coupler
  !   2.  Compute:  Duplex Export/Import with MHD & Coupler
  !   3.  Last Exchange:  Export data to MHD & Coupler
  couplingTimeLoop: do
     call ReceiveScalars("mhd-im_scalars", iaScalars)

     ! Is there any more work to do?
     if (iaScalars(KILL_SIGNAL) == KILL_SIGNAL_SHUTDOWN) then
        exit couplingTimeLoop
     end if

     ! Is this the first exchange? (Import only)
     isFirstExchange = &
          (exchangeNum == 0) .and. &              
          (iaScalars(KILL_SIGNAL) /= KILL_SIGNAL_SHUTDOWN) .and. &
          (iaScalars(KILL_SIGNAL) /= KILL_SIGNAL_LAST_EXCHANGE)
     ! IntermediateExchange:  Not first, Not last (duplex exchange)
     ! Is this the last exchange? (Export only)
     isLastExchange = (iaScalars(KILL_SIGNAL) == KILL_SIGNAL_LAST_EXCHANGE)

     ! Tell the user what's going on via stdout message:
     if (isFirstExchange) then
        write(*,*) "First LTR exchange #", exchangeNum
     else if (isLastExchange) then
        write(*,*) "Last LTR exchange #", exchangeNum
     else
        write(*,*) "Intermediate LTR exchange # ", exchangeNum
     end if
     write(*,*) iaScalars

     ! Exchange with codes. . . 
     if ((.not. isFirstExchange) .or. isLastExchange) then
        call ExportIonosphere()
        call ExportMagnetosphere()
     end if

     if (isFirstExchange .or. (.not. isLastExchange)) then
        call ImportIonosphere()
        call ImportMagnetosphere()
     end if

     exchangeNum = exchangeNum + 1

     ! FIXME:
     ! This is where the RCM should advance its time loop from
     ! the current date/time (inside iaScalars(YEAR:SECOND)) by
     ! rcmDeltaT (iaScalars(DELTA_T)), the number of seconds
     ! between LFM-RCM exchange.
     !
     ! Will probably need to use the UTIO code to compare against
     ! modified julian dates.
  end do couplingTimeLoop

  call Finalize() ! implemented in intercomm.f90
  call tearDownIon()
  call tearDownIg() 

end program rcm_lfm
