/*
 * Hello Parallel Worlds InterComm app
 *     F90 uses low-level XJD API (section 4 of InterComm manual)
 *     C++ uses high-level XJD API (section 6 of InterComm manual)
 *
 * History lesson:  What was used for CMIT 2.0?
 *
 * LFM-omp:
 *     "High-Level" Fortran 90 interface
 *     See InterComm Manual "6.  High-Level XJD-based Programming Tasks"
 *     "... for sequential Fortran 90 programs"???
 *
 * LFM-para:
 *      "High-Level" C++ interface (EndpointSet & P++)
 *      See InterComm Manual "6.  High-Level XJD-based Programming Tasks"
 *
 * MIX:
 *      "Low-Level" C++ interface 
 *      See InterComm Manual "4.  Low-Level XJD-based Programming Tasks"
 *      
 * TIEGCM-wenbin:
 *     "Low-Level" Fortran interface
 *     See InterComm Manual "3.  Low-Level Programming Tasks"
 *
 */

#include <cassert>
#include <iostream>
#include <iomanip>
#include <math.h>

#include "InterComm.h"
#include "IC_EndPointSet.h"

bool is_within_delta(const double &x, const double &y, const double eps = 1e-6)
{
  return (fabs(x-y) < eps );
}

int main(int argc, char **argv)
{
  std::cout << "Hello parallel worlds!" << std::endl;

  double exportArray[10] = { 2.0, 4.0, 6.0, 8.0, 10.0, 12.0, 14.0, 16.0, 18.0, 20.0 };
  double importArray[10];

  int status;
  IC_XJD *xjd = IC_Initialize("helloCPP", 0, "hello.xjd", &status);
  if (status != IC_EndPointSet::IC_OK){
    std::cerr << "*** InterComm error** (code " << status << ")." << std::endl;
  }

  int tasks[1] = { 0 };  
  int blocks[1][2][1] = { { {0}, {9} } };
  IC_Desc *desc = IC_Create_bdecomp_desc(1, &blocks[0][0][0], tasks, 1, 1); 
  //IC_ROW_MAJOR = 0;   IC_COLUMN_MAJOR=1                              ^^^

  int lower[1] = { 0 };
  int upper[1] = { 9 };
  int stride[1] = { 1 };
  IC_Region *region[1] = { IC_Create_block_region(1, lower, upper, stride) };

  IC_Register_region(xjd, region, 1, "even", desc, &exportArray[0], &status);
  assert(status == 0);
  IC_Register_region(xjd, region, 1, "odd", desc, &importArray[0], &status);
  assert(status == 0);

  IC_Commit_region(xjd, &status);
  assert(status == 0);

  status = IC_Export(xjd, "even");
  assert(status == 0);
  status = IC_Import(xjd, "odd");
  assert(status == 0);

  IC_Finalize(xjd, &status);
  //IC_EndPointSet::printErrorMessage("msg", status);
  if (status != IC_EndPointSet::IC_OK){
    std::cerr << "*** InterComm error** (code " << status << ")." << std::endl;
  }  

  // print contents of arrays
  std::cout << "Export:";
  for (int i=0; i < 10; i++){
    std::cout << std::fixed << std::setw(2) << std::setprecision(1) << " " << exportArray[i];
  }  
  std::cout << std::endl;
  std::cout << "Import:";
  for (int i=0; i < 10; i++){
    std::cout << std::fixed << std::setw(2) << std::setprecision(1) << " " << importArray[i];
  }
  std::cout << std::endl;

  for (int i = 0; i < 10; i++){
    assert( is_within_delta( importArray[i], (i+1.0) * 2.0 - 1.0 ) );
  }

  return 0;
}
