// standard includes
#include <cassert>
#include <string>
#include <iostream>

// 3rd party includes
#define PPLUSPLUS
#include <A++.h>
#include <IC_EndPointSet.h>

bool is_within_delta(const double &x, const double &y, const double eps = 1e-6)
{
  return (fabs(x-y) < eps );
}

int main(int argc, char **argv)
{
  int num_procs = atoi(argv[1]);
  char *localName = "parallel";

  // Startup P++
  Optimization_Manager::Initialize_Virtual_Machine(localName,num_procs,argc,argv);

  int mype = Communication_Manager::My_Process_Number;
  std::cout << mype << ": Hello parallel worlds!" << std::endl;
  Communication_Manager::Sync();

  // Startup InterComm
  char *componentName = "parallelProgram";
  char *xjdFile = "hello.xjd";
  int icErr;
  IC_EndPointSet *epset = new IC_EndPointSet(xjdFile, componentName, icErr);

  // Check dims
  intArray serialSize(num_procs);
  serialSize = 0;
  epset->recvLocalArray("serialSize", serialSize, 1, icErr);
  Partitioning_Type p_zero(Range(0,0));
  intArray parallelSize(1, p_zero);
  parallelSize = num_procs;
  epset->bcastLocalArray("parallelSize", parallelSize, 1, icErr);

  // Initialize the arrays.
  Partitioning_Type all(num_procs);
  floatArray parallelArray(num_procs, all);
  floatArray serialArray(num_procs, all);

  epset->registerArray("parallel_to_serial", parallelArray, icErr);
  epset->registerArray("serial_to_parallel", serialArray, icErr);

  epset->commitArrays(icErr);

  // Do some work...
  parallelArray(mype) = mype;
  epset->exportArray("parallel_to_serial", icErr);
  epset->importArray("serial_to_parallel", icErr);

  cout << "Parallel Assert!" << endl;
  if ( !is_within_delta( serialArray(mype), num_procs-mype ) ){
    cout << mype << "\t" << serialArray(mype) << "\t" << num_procs-mype << endl;
  }
  assert( is_within_delta( serialArray(mype), num_procs-mype ) );
  cout << "Everything looks good.  Exiting (hopefully cleanly)" << endl;

  // Cleanup
  Communication_Manager::Sync();
  Optimization_Manager::Exit_Virtual_Machine();
  delete epset;

  return 0;
}
