// standard includes
#include <cassert>
#include <iostream>
#include <iomanip>
#include <math.h>

// 3rd-party includes
#include <A++.h>
#include <InterComm.h>
#include <IC_EndPointSet.h>

bool is_within_delta(const double &x, const double &y, const double eps = 1e-6)
{
  return (fabs(x-y) < eps );
}

int main(int argc, char **argv)
{
  int arrayLength = atoi(argv[1]);
  int pArrayLength = 0;
  char *componentName = "serialProgram";
  char *xjdFile = "hello.xjd";

  std::cout << "Hello world!" << std::endl;

  // Startup InterComm
  int icErr;
  IC_XJD *xjd = IC_Initialize(componentName, 0, xjdFile, &icErr);

  // Check dims
  icErr = IC_Bcast_local(xjd, "serialSize", &arrayLength, 1);
  icErr = IC_Recv_local(xjd, "parallelSize", &pArrayLength, 1);
  cout << "serial size: " << arrayLength  << "\t"
       << "parallel size: " << pArrayLength << endl;

  // Initialize the arrays
  floatArray serialArray(arrayLength);
  floatArray parallelArray(arrayLength);

  int iTasks[1] = {0};
  int iaBlocks1d[1][2][1];
  iaBlocks1d[0][0][0] = 0;
  iaBlocks1d[0][1][0] = arrayLength-1;
  int iLower1d = 0;
  int iUpper1d = arrayLength - 1;
  int iStride1d = 1;

  IC_Desc *desc_vars = IC_Create_bdecomp_desc(1,&iaBlocks1d[0][0][0],iTasks,1,1);
  IC_Region *region_vars[1];
  region_vars[0] = IC_Create_block_region(1,&iLower1d,&iUpper1d,&iStride1d);

  IC_Register_region(xjd,region_vars,1,"parallel_to_serial",desc_vars,parallelArray.getDataPointer(),&icErr);
  IC_Register_region(xjd,region_vars,1,"serial_to_parallel",desc_vars,serialArray.getDataPointer(),&icErr);

  IC_Commit_region(xjd,&icErr);

  // Do some work...  
  for (int i=0; i < arrayLength; i++){
    serialArray(i) = arrayLength-i;
  }

  IC_Import(xjd, "parallel_to_serial");
  IC_Export(xjd, "serial_to_parallel");

  cout << "Serial Assert!" << endl;
  for (int i=0; i < arrayLength; i++){
    assert( is_within_delta( parallelArray(i), i ) );
  }
  cout << "Everything looks good.  Exiting (hopefully cleanly)" << endl;
  

  // Cleanup
  IC_Free_desc(desc_vars);
  IC_Free_region(region_vars[0]);
  IC_Finalize(xjd, &icErr);

  return 0;
}
